-- Query generated by: templates/unnest_parquet_view.sql.py crash_summary_v1 telemetry_derived.crash_summary_v1
CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.crash_summary_v1`
AS
SELECT
  * REPLACE (
    experiments.key_value AS experiments,
    (SELECT AS STRUCT payload.* REPLACE (payload.metadata.key_value AS metadata)) AS payload
  )
FROM
  `moz-fx-data-shared-prod.telemetry_derived.crash_summary_v1`
