CREATE OR REPLACE FUNCTION udf.main_summary_scalars(processes ANY TYPE) AS (
  STRUCT(
    processes.parent.scalars.a11y_backplate AS scalar_parent_a11y_backplate,
    processes.parent.scalars.a11y_indicator_acted_on AS scalar_parent_a11y_indicator_acted_on,
    processes.parent.scalars.a11y_instantiators AS scalar_parent_a11y_instantiators,
    processes.parent.scalars.aushelper_websense_reg_version AS scalar_parent_aushelper_websense_reg_version,
    -- scalar_parent_blocklist_last_modified_rs_addons
    processes.parent.scalars.blocklist_last_modified_rs_addons AS scalar_parent_blocklist_lastModified_rs_addons,
    -- scalar_parent_blocklist_last_modified_rs_plugins
    processes.parent.scalars.blocklist_last_modified_rs_plugins AS scalar_parent_blocklist_lastModified_rs_plugins,
    -- scalar_parent_blocklist_last_modified_xml
    processes.parent.scalars.blocklist_last_modified_xml AS scalar_parent_blocklist_lastModified_xml,
    -- scalar_parent_blocklist_use_xml
    processes.parent.scalars.blocklist_use_xml AS scalar_parent_blocklist_useXML,
    processes.parent.scalars.browser_engagement_active_ticks AS scalar_parent_browser_engagement_active_ticks,
    processes.parent.scalars.browser_engagement_max_concurrent_tab_count AS scalar_parent_browser_engagement_max_concurrent_tab_count,
    processes.parent.scalars.browser_engagement_max_concurrent_tab_pinned_count AS scalar_parent_browser_engagement_max_concurrent_tab_pinned_count,
    processes.parent.scalars.browser_engagement_max_concurrent_window_count AS scalar_parent_browser_engagement_max_concurrent_window_count,
    processes.parent.scalars.browser_engagement_restored_pinned_tabs_count AS scalar_parent_browser_engagement_restored_pinned_tabs_count,
    processes.parent.scalars.browser_engagement_tab_open_event_count AS scalar_parent_browser_engagement_tab_open_event_count,
    processes.parent.scalars.browser_engagement_tab_pinned_event_count AS scalar_parent_browser_engagement_tab_pinned_event_count,
    processes.parent.scalars.browser_engagement_total_uri_count AS scalar_parent_browser_engagement_total_uri_count,
    processes.parent.scalars.browser_engagement_unfiltered_uri_count AS scalar_parent_browser_engagement_unfiltered_uri_count,
    processes.parent.scalars.browser_engagement_unique_domains_count AS scalar_parent_browser_engagement_unique_domains_count,
    processes.parent.scalars.browser_engagement_window_open_event_count AS scalar_parent_browser_engagement_window_open_event_count,
    processes.parent.scalars.browser_errors_collected_count AS scalar_parent_browser_errors_collected_count,
    processes.parent.scalars.browser_errors_collected_with_stack_count AS scalar_parent_browser_errors_collected_with_stack_count,
    processes.parent.scalars.browser_errors_reported_failure_count AS scalar_parent_browser_errors_reported_failure_count,
    processes.parent.scalars.browser_errors_reported_success_count AS scalar_parent_browser_errors_reported_success_count,
    processes.parent.scalars.browser_errors_sample_rate AS scalar_parent_browser_errors_sample_rate,
    processes.parent.scalars.browser_feeds_feed_subscribed AS scalar_parent_browser_feeds_feed_subscribed,
    processes.parent.scalars.browser_feeds_livebookmark_count AS scalar_parent_browser_feeds_livebookmark_count,
    processes.parent.scalars.browser_feeds_livebookmark_item_opened AS scalar_parent_browser_feeds_livebookmark_item_opened,
    processes.parent.scalars.browser_feeds_livebookmark_opened AS scalar_parent_browser_feeds_livebookmark_opened,
    processes.parent.scalars.browser_feeds_preview_loaded AS scalar_parent_browser_feeds_preview_loaded,
    processes.parent.scalars.browser_searchinit_engines_cache_corrupted AS scalar_parent_browser_searchinit_engines_cache_corrupted,
    processes.parent.scalars.browser_session_restore_browser_startup_page AS scalar_parent_browser_session_restore_browser_startup_page,
    processes.parent.scalars.browser_session_restore_browser_tabs_restorebutton AS scalar_parent_browser_session_restore_browser_tabs_restorebutton,
    processes.parent.scalars.browser_session_restore_number_of_tabs AS scalar_parent_browser_session_restore_number_of_tabs,
    processes.parent.scalars.browser_session_restore_number_of_win AS scalar_parent_browser_session_restore_number_of_win,
    processes.parent.scalars.browser_session_restore_tabbar_restore_available AS scalar_parent_browser_session_restore_tabbar_restore_available,
    processes.parent.scalars.browser_session_restore_tabbar_restore_clicked AS scalar_parent_browser_session_restore_tabbar_restore_clicked,
    processes.parent.scalars.browser_session_restore_worker_restart_count AS scalar_parent_browser_session_restore_worker_restart_count,
    processes.parent.scalars.browser_timings_last_shutdown AS scalar_parent_browser_timings_last_shutdown,
    processes.parent.scalars.browser_usage_graphite AS scalar_parent_browser_usage_graphite,
    processes.parent.scalars.browser_usage_plugin_instantiated AS scalar_parent_browser_usage_plugin_instantiated,
    processes.parent.scalars.contentblocking_category AS scalar_parent_contentblocking_category,
    processes.parent.scalars.contentblocking_cryptomining_blocking_enabled AS scalar_parent_contentblocking_cryptomining_blocking_enabled,
    processes.parent.scalars.contentblocking_enabled AS scalar_parent_contentblocking_enabled,
    processes.parent.scalars.contentblocking_exceptions AS scalar_parent_contentblocking_exceptions,
    processes.parent.scalars.contentblocking_fastblock_enabled AS scalar_parent_contentblocking_fastblock_enabled,
    processes.parent.scalars.contentblocking_fingerprinting_blocking_enabled AS scalar_parent_contentblocking_fingerprinting_blocking_enabled,
    processes.parent.scalars.contentblocking_trackers_blocked_count AS scalar_parent_contentblocking_trackers_blocked_count,
    processes.parent.scalars.corroborate_omnijar_corrupted AS scalar_parent_corroborate_omnijar_corrupted,
    processes.parent.scalars.corroborate_system_addons_corrupted AS scalar_parent_corroborate_system_addons_corrupted,
    processes.parent.scalars.devtools_aboutdevtools_installed AS scalar_parent_devtools_aboutdevtools_installed,
    processes.parent.scalars.devtools_aboutdevtools_noinstall_exits AS scalar_parent_devtools_aboutdevtools_noinstall_exits,
    processes.parent.scalars.devtools_aboutdevtools_opened AS scalar_parent_devtools_aboutdevtools_opened,
    processes.parent.scalars.devtools_accessibility_accessible_context_menu_opened AS scalar_parent_devtools_accessibility_accessible_context_menu_opened,
    processes.parent.scalars.devtools_accessibility_node_inspected_count AS scalar_parent_devtools_accessibility_node_inspected_count,
    processes.parent.scalars.devtools_accessibility_opened_count AS scalar_parent_devtools_accessibility_opened_count,
    processes.parent.scalars.devtools_accessibility_picker_used_count AS scalar_parent_devtools_accessibility_picker_used_count,
    processes.parent.scalars.devtools_accessibility_service_enabled_count AS scalar_parent_devtools_accessibility_service_enabled_count,
    processes.parent.scalars.devtools_application_opened_count AS scalar_parent_devtools_application_opened_count,
    processes.parent.scalars.devtools_changesview_contextmenu AS scalar_parent_devtools_changesview_contextmenu,
    processes.parent.scalars.devtools_changesview_contextmenu_copy AS scalar_parent_devtools_changesview_contextmenu_copy,
    processes.parent.scalars.devtools_changesview_contextmenu_copy_declaration AS scalar_parent_devtools_changesview_contextmenu_copy_declaration,
    processes.parent.scalars.devtools_changesview_contextmenu_copy_rule AS scalar_parent_devtools_changesview_contextmenu_copy_rule,
    processes.parent.scalars.devtools_changesview_copy AS scalar_parent_devtools_changesview_copy,
    processes.parent.scalars.devtools_changesview_copy_all_changes AS scalar_parent_devtools_changesview_copy_all_changes,
    processes.parent.scalars.devtools_changesview_copy_rule AS scalar_parent_devtools_changesview_copy_rule,
    processes.parent.scalars.devtools_changesview_opened_count AS scalar_parent_devtools_changesview_opened_count,
    processes.parent.scalars.devtools_copy_full_css_selector_opened AS scalar_parent_devtools_copy_full_css_selector_opened,
    processes.parent.scalars.devtools_copy_unique_css_selector_opened AS scalar_parent_devtools_copy_unique_css_selector_opened,
    processes.parent.scalars.devtools_copy_xpath_opened AS scalar_parent_devtools_copy_xpath_opened,
    processes.parent.scalars.devtools_grid_gridinspector_opened AS scalar_parent_devtools_grid_gridinspector_opened,
    -- scalar_parent_devtools_grid_show_grid_areas_overlay_checked
    processes.parent.scalars.devtools_grid_show_grid_areas_overlay_checked AS scalar_parent_devtools_grid_showGridAreasOverlay_checked,
    -- scalar_parent_devtools_grid_show_grid_line_numbers_checked
    processes.parent.scalars.devtools_grid_show_grid_line_numbers_checked AS scalar_parent_devtools_grid_showGridLineNumbers_checked,
    -- scalar_parent_devtools_grid_show_infinite_lines_checked
    processes.parent.scalars.devtools_grid_show_infinite_lines_checked AS scalar_parent_devtools_grid_showInfiniteLines_checked,
    processes.parent.scalars.devtools_inspector_element_picker_used AS scalar_parent_devtools_inspector_element_picker_used,
    processes.parent.scalars.devtools_inspector_node_selection_count AS scalar_parent_devtools_inspector_node_selection_count,
    processes.parent.scalars.devtools_layout_flexboxhighlighter_opened AS scalar_parent_devtools_layout_flexboxhighlighter_opened,
    processes.parent.scalars.devtools_markup_flexboxhighlighter_opened AS scalar_parent_devtools_markup_flexboxhighlighter_opened,
    processes.parent.scalars.devtools_markup_gridinspector_opened AS scalar_parent_devtools_markup_gridinspector_opened,
    processes.parent.scalars.devtools_onboarding_is_devtools_user AS scalar_parent_devtools_onboarding_is_devtools_user,
    processes.parent.scalars.devtools_responsive_toolbox_opened_first AS scalar_parent_devtools_responsive_toolbox_opened_first,
    processes.parent.scalars.devtools_rules_flexboxhighlighter_opened AS scalar_parent_devtools_rules_flexboxhighlighter_opened,
    processes.parent.scalars.devtools_rules_gridinspector_opened AS scalar_parent_devtools_rules_gridinspector_opened,
    processes.parent.scalars.devtools_shadowdom_reveal_link_clicked AS scalar_parent_devtools_shadowdom_reveal_link_clicked,
    processes.parent.scalars.devtools_shadowdom_shadow_root_displayed AS scalar_parent_devtools_shadowdom_shadow_root_displayed,
    processes.parent.scalars.devtools_shadowdom_shadow_root_expanded AS scalar_parent_devtools_shadowdom_shadow_root_expanded,
    processes.parent.scalars.devtools_toolbar_eyedropper_opened AS scalar_parent_devtools_toolbar_eyedropper_opened,
    processes.parent.scalars.devtools_webreplay_load_recording AS scalar_parent_devtools_webreplay_load_recording,
    processes.parent.scalars.devtools_webreplay_new_recording AS scalar_parent_devtools_webreplay_new_recording,
    processes.parent.scalars.devtools_webreplay_reload_recording AS scalar_parent_devtools_webreplay_reload_recording,
    processes.parent.scalars.devtools_webreplay_save_recording AS scalar_parent_devtools_webreplay_save_recording,
    processes.parent.scalars.devtools_webreplay_stop_recording AS scalar_parent_devtools_webreplay_stop_recording,
    -- scalar_parent_dom_contentprocess_build_id_mismatch
    processes.parent.scalars.dom_contentprocess_build_id_mismatch AS scalar_parent_dom_contentprocess_buildID_mismatch,
    processes.parent.scalars.dom_contentprocess_os_priority_change_considered AS scalar_parent_dom_contentprocess_os_priority_change_considered,
    processes.parent.scalars.dom_contentprocess_os_priority_lowered AS scalar_parent_dom_contentprocess_os_priority_lowered,
    processes.parent.scalars.dom_contentprocess_os_priority_raised AS scalar_parent_dom_contentprocess_os_priority_raised,
    processes.parent.scalars.dom_contentprocess_troubled_due_to_memory AS scalar_parent_dom_contentprocess_troubled_due_to_memory,
    processes.parent.scalars.dom_parentprocess_private_window_used AS scalar_parent_dom_parentprocess_private_window_used,
    processes.parent.scalars.encoding_override_used AS scalar_parent_encoding_override_used,
    -- scalar_parent_first_startup_elapsed
    processes.parent.scalars.first_startup_elapsed AS scalar_parent_firstStartup_elapsed,
    -- scalar_parent_first_startup_status_code
    processes.parent.scalars.first_startup_status_code AS scalar_parent_firstStartup_statusCode,
    processes.parent.scalars.formautofill_addresses_fill_type_autofill AS scalar_parent_formautofill_addresses_fill_type_autofill,
    processes.parent.scalars.formautofill_addresses_fill_type_autofill_update AS scalar_parent_formautofill_addresses_fill_type_autofill_update,
    processes.parent.scalars.formautofill_addresses_fill_type_manual AS scalar_parent_formautofill_addresses_fill_type_manual,
    processes.parent.scalars.formautofill_availability AS scalar_parent_formautofill_availability,
    -- scalar_parent_formautofill_credit_cards_fill_type_autofill
    processes.parent.scalars.formautofill_credit_cards_fill_type_autofill AS scalar_parent_formautofill_creditCards_fill_type_autofill,
    -- scalar_parent_formautofill_credit_cards_fill_type_autofill_modified
    processes.parent.scalars.formautofill_credit_cards_fill_type_autofill_modified AS scalar_parent_formautofill_creditCards_fill_type_autofill_modified,
    -- scalar_parent_formautofill_credit_cards_fill_type_manual
    processes.parent.scalars.formautofill_credit_cards_fill_type_manual AS scalar_parent_formautofill_creditCards_fill_type_manual,
    processes.parent.scalars.gfx_hdr_windows_display_colorspace_bitfield AS scalar_parent_gfx_hdr_windows_display_colorspace_bitfield,
    processes.parent.scalars.idb_failure_fileinfo_error AS scalar_parent_idb_failure_fileinfo_error,
    processes.parent.scalars.idb_type_persistent_count AS scalar_parent_idb_type_persistent_count,
    processes.parent.scalars.idb_type_temporary_count AS scalar_parent_idb_type_temporary_count,
    processes.parent.scalars.identity_fxaccounts_missed_commands_fetched AS scalar_parent_identity_fxaccounts_missed_commands_fetched,
    processes.parent.scalars.images_webp_content_observed AS scalar_parent_images_webp_content_observed,
    processes.parent.scalars.images_webp_probe_observed AS scalar_parent_images_webp_probe_observed,
    processes.parent.scalars.media_allowed_autoplay_no_audio_track_count AS scalar_parent_media_allowed_autoplay_no_audio_track_count,
    processes.parent.scalars.media_autoplay_default_blocked AS scalar_parent_media_autoplay_default_blocked,
    processes.parent.scalars.media_autoplay_would_be_allowed_count AS scalar_parent_media_autoplay_would_be_allowed_count,
    processes.parent.scalars.media_autoplay_would_not_be_allowed_count AS scalar_parent_media_autoplay_would_not_be_allowed_count,
    -- define removed field for schema compatiblity
    NULL AS scalar_parent_media_blocked_autoplay_no_audio_track_count,
    processes.parent.scalars.media_blocked_no_metadata AS scalar_parent_media_blocked_no_metadata,
    processes.parent.scalars.media_blocked_no_metadata_endup_no_audio_track AS scalar_parent_media_blocked_no_metadata_endup_no_audio_track,
    processes.parent.scalars.media_page_count AS scalar_parent_media_page_count,
    processes.parent.scalars.media_page_had_media_count AS scalar_parent_media_page_had_media_count,
    processes.parent.scalars.media_page_had_play_revoked_count AS scalar_parent_media_page_had_play_revoked_count,
    processes.parent.scalars.mediarecorder_recording_count AS scalar_parent_mediarecorder_recording_count,
    processes.parent.scalars.navigator_storage_estimate_count AS scalar_parent_navigator_storage_estimate_count,
    processes.parent.scalars.navigator_storage_persist_count AS scalar_parent_navigator_storage_persist_count,
    processes.parent.scalars.networking_data_transferred_captive_portal AS scalar_parent_networking_data_transferred_captive_portal,
    processes.parent.scalars.networking_ftp_opened_channels_files AS scalar_parent_networking_ftp_opened_channels_files,
    processes.parent.scalars.networking_ftp_opened_channels_listings AS scalar_parent_networking_ftp_opened_channels_listings,
    processes.parent.scalars.networking_http_connections_captive_portal AS scalar_parent_networking_http_connections_captive_portal,
    processes.parent.scalars.networking_http_transactions_captive_portal AS scalar_parent_networking_http_transactions_captive_portal,
    processes.parent.scalars.os_environment_is_admin_without_uac AS scalar_parent_os_environment_is_admin_without_uac,
    processes.parent.scalars.pdf_viewer_fallback_shown AS scalar_parent_pdf_viewer_fallback_shown,
    processes.parent.scalars.pdf_viewer_print AS scalar_parent_pdf_viewer_print,
    processes.parent.scalars.pdf_viewer_used AS scalar_parent_pdf_viewer_used,
    processes.parent.scalars.preferences_created_new_user_prefs_file AS scalar_parent_preferences_created_new_user_prefs_file,
    processes.parent.scalars.preferences_prefs_file_was_invalid AS scalar_parent_preferences_prefs_file_was_invalid,
    processes.parent.scalars.preferences_prevent_accessibility_services AS scalar_parent_preferences_prevent_accessibility_services,
    processes.parent.scalars.preferences_read_user_js AS scalar_parent_preferences_read_user_js,
    processes.parent.scalars.pwmgr_potentially_breached_passwords AS scalar_parent_pwmgr_potentially_breached_passwords,
    processes.parent.scalars.screenshots_copy AS scalar_parent_screenshots_copy,
    processes.parent.scalars.screenshots_download AS scalar_parent_screenshots_download,
    processes.parent.scalars.screenshots_upload AS scalar_parent_screenshots_upload,
    processes.parent.scalars.script_preloader_mainthread_recompile AS scalar_parent_script_preloader_mainthread_recompile,
    processes.parent.scalars.security_intermediate_preloading_num_pending AS scalar_parent_security_intermediate_preloading_num_pending,
    processes.parent.scalars.security_intermediate_preloading_num_preloaded AS scalar_parent_security_intermediate_preloading_num_preloaded,
    processes.parent.scalars.security_tls_delegated_credentials_txn AS scalar_parent_security_tls_delegated_credentials_txn,
    processes.parent.scalars.services_sync_fxa_verification_method AS scalar_parent_services_sync_fxa_verification_method,
    processes.parent.scalars.startup_is_cold AS scalar_parent_startup_is_cold,
    processes.parent.scalars.startup_profile_selection_reason AS scalar_parent_startup_profile_selection_reason,
    processes.parent.scalars.storage_sync_api_usage_extensions_using AS scalar_parent_storage_sync_api_usage_extensions_using,
    processes.parent.scalars.sw_alternative_body_used_count AS scalar_parent_sw_alternative_body_used_count,
    processes.parent.scalars.sw_cors_res_for_so_req_count AS scalar_parent_sw_cors_res_for_so_req_count,
    processes.parent.scalars.sw_synthesized_res_count AS scalar_parent_sw_synthesized_res_count,
    processes.parent.scalars.telemetry_about_telemetry_pageload AS scalar_parent_telemetry_about_telemetry_pageload,
    processes.parent.scalars.telemetry_data_upload_optin AS scalar_parent_telemetry_data_upload_optin,
    processes.parent.scalars.telemetry_ecosystem_new_send_time AS scalar_parent_telemetry_ecosystem_new_send_time,
    processes.parent.scalars.telemetry_ecosystem_old_send_time AS scalar_parent_telemetry_ecosystem_old_send_time,
    processes.parent.scalars.telemetry_os_shutting_down AS scalar_parent_telemetry_os_shutting_down,
    processes.parent.scalars.telemetry_pending_operations_highwatermark_reached AS scalar_parent_telemetry_pending_operations_highwatermark_reached,
    processes.parent.scalars.telemetry_persistence_timer_hit_count AS scalar_parent_telemetry_persistence_timer_hit_count,
    processes.parent.scalars.telemetry_process_creation_timestamp_inconsistent AS scalar_parent_telemetry_process_creation_timestamp_inconsistent,
    processes.parent.scalars.telemetry_profile_directory_scan_date AS scalar_parent_telemetry_profile_directory_scan_date,
    processes.parent.scalars.telemetry_profile_directory_scans AS scalar_parent_telemetry_profile_directory_scans,
    processes.parent.scalars.timestamps_about_home_topsites_first_paint AS scalar_parent_timestamps_about_home_topsites_first_paint,
    processes.parent.scalars.timestamps_first_paint AS scalar_parent_timestamps_first_paint,
    processes.parent.scalars.update_fix_permissions_attempted AS scalar_parent_update_fix_permissions_attempted,
    processes.parent.scalars.update_session_downloads_bits_complete_bytes AS scalar_parent_update_session_downloads_bits_complete_bytes,
    processes.parent.scalars.update_session_downloads_bits_complete_seconds AS scalar_parent_update_session_downloads_bits_complete_seconds,
    processes.parent.scalars.update_session_downloads_bits_partial_bytes AS scalar_parent_update_session_downloads_bits_partial_bytes,
    processes.parent.scalars.update_session_downloads_bits_partial_seconds AS scalar_parent_update_session_downloads_bits_partial_seconds,
    processes.parent.scalars.update_session_downloads_internal_complete_bytes AS scalar_parent_update_session_downloads_internal_complete_bytes,
    processes.parent.scalars.update_session_downloads_internal_complete_seconds AS scalar_parent_update_session_downloads_internal_complete_seconds,
    processes.parent.scalars.update_session_downloads_internal_partial_bytes AS scalar_parent_update_session_downloads_internal_partial_bytes,
    processes.parent.scalars.update_session_downloads_internal_partial_seconds AS scalar_parent_update_session_downloads_internal_partial_seconds,
    processes.parent.scalars.update_session_from_app_version AS scalar_parent_update_session_from_app_version,
    processes.parent.scalars.update_session_intervals_apply_complete AS scalar_parent_update_session_intervals_apply_complete,
    processes.parent.scalars.update_session_intervals_apply_partial AS scalar_parent_update_session_intervals_apply_partial,
    processes.parent.scalars.update_session_intervals_check AS scalar_parent_update_session_intervals_check,
    processes.parent.scalars.update_session_intervals_download_bits_complete AS scalar_parent_update_session_intervals_download_bits_complete,
    processes.parent.scalars.update_session_intervals_download_bits_partial AS scalar_parent_update_session_intervals_download_bits_partial,
    processes.parent.scalars.update_session_intervals_download_internal_complete AS scalar_parent_update_session_intervals_download_internal_complete,
    processes.parent.scalars.update_session_intervals_download_internal_partial AS scalar_parent_update_session_intervals_download_internal_partial,
    processes.parent.scalars.update_session_intervals_stage_complete AS scalar_parent_update_session_intervals_stage_complete,
    processes.parent.scalars.update_session_intervals_stage_partial AS scalar_parent_update_session_intervals_stage_partial,
    processes.parent.scalars.update_session_mar_complete_size_bytes AS scalar_parent_update_session_mar_complete_size_bytes,
    processes.parent.scalars.update_session_mar_partial_size_bytes AS scalar_parent_update_session_mar_partial_size_bytes,
    processes.parent.scalars.update_startup_downloads_bits_complete_bytes AS scalar_parent_update_startup_downloads_bits_complete_bytes,
    processes.parent.scalars.update_startup_downloads_bits_complete_seconds AS scalar_parent_update_startup_downloads_bits_complete_seconds,
    processes.parent.scalars.update_startup_downloads_bits_partial_bytes AS scalar_parent_update_startup_downloads_bits_partial_bytes,
    processes.parent.scalars.update_startup_downloads_bits_partial_seconds AS scalar_parent_update_startup_downloads_bits_partial_seconds,
    processes.parent.scalars.update_startup_downloads_internal_complete_bytes AS scalar_parent_update_startup_downloads_internal_complete_bytes,
    processes.parent.scalars.update_startup_downloads_internal_complete_seconds AS scalar_parent_update_startup_downloads_internal_complete_seconds,
    processes.parent.scalars.update_startup_downloads_internal_partial_bytes AS scalar_parent_update_startup_downloads_internal_partial_bytes,
    processes.parent.scalars.update_startup_downloads_internal_partial_seconds AS scalar_parent_update_startup_downloads_internal_partial_seconds,
    processes.parent.scalars.update_startup_from_app_version AS scalar_parent_update_startup_from_app_version,
    processes.parent.scalars.update_startup_intervals_apply_complete AS scalar_parent_update_startup_intervals_apply_complete,
    processes.parent.scalars.update_startup_intervals_apply_partial AS scalar_parent_update_startup_intervals_apply_partial,
    processes.parent.scalars.update_startup_intervals_check AS scalar_parent_update_startup_intervals_check,
    processes.parent.scalars.update_startup_intervals_download_bits_complete AS scalar_parent_update_startup_intervals_download_bits_complete,
    processes.parent.scalars.update_startup_intervals_download_bits_partial AS scalar_parent_update_startup_intervals_download_bits_partial,
    processes.parent.scalars.update_startup_intervals_download_internal_complete AS scalar_parent_update_startup_intervals_download_internal_complete,
    processes.parent.scalars.update_startup_intervals_download_internal_partial AS scalar_parent_update_startup_intervals_download_internal_partial,
    processes.parent.scalars.update_startup_intervals_stage_complete AS scalar_parent_update_startup_intervals_stage_complete,
    processes.parent.scalars.update_startup_intervals_stage_partial AS scalar_parent_update_startup_intervals_stage_partial,
    processes.parent.scalars.update_startup_mar_complete_size_bytes AS scalar_parent_update_startup_mar_complete_size_bytes,
    processes.parent.scalars.update_startup_mar_partial_size_bytes AS scalar_parent_update_startup_mar_partial_size_bytes,
    processes.parent.scalars.webrtc_hostnameobfuscation_disabled_failed AS scalar_parent_webrtc_hostnameobfuscation_disabled_failed,
    processes.parent.scalars.webrtc_hostnameobfuscation_disabled_succeeded AS scalar_parent_webrtc_hostnameobfuscation_disabled_succeeded,
    processes.parent.scalars.webrtc_hostnameobfuscation_enabled_failed AS scalar_parent_webrtc_hostnameobfuscation_enabled_failed,
    processes.parent.scalars.webrtc_hostnameobfuscation_enabled_succeeded AS scalar_parent_webrtc_hostnameobfuscation_enabled_succeeded,
    processes.parent.scalars.webrtc_nicer_stun_retransmits AS scalar_parent_webrtc_nicer_stun_retransmits,
    processes.parent.scalars.webrtc_nicer_turn_401s AS scalar_parent_webrtc_nicer_turn_401s,
    processes.parent.scalars.webrtc_nicer_turn_403s AS scalar_parent_webrtc_nicer_turn_403s,
    processes.parent.scalars.webrtc_nicer_turn_438s AS scalar_parent_webrtc_nicer_turn_438s,
    processes.parent.scalars.webrtc_peerconnection_connected AS scalar_parent_webrtc_peerconnection_connected,
    processes.parent.scalars.webrtc_peerconnection_datachannel_created AS scalar_parent_webrtc_peerconnection_datachannel_created,
    processes.parent.scalars.webrtc_peerconnection_datachannel_max_life_used AS scalar_parent_webrtc_peerconnection_datachannel_max_life_used,
    processes.parent.scalars.webrtc_peerconnection_datachannel_max_retx_and_life_used AS scalar_parent_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    processes.parent.scalars.webrtc_peerconnection_datachannel_max_retx_used AS scalar_parent_webrtc_peerconnection_datachannel_max_retx_used,
    processes.parent.scalars.webrtc_peerconnection_legacy_callback_stats_used AS scalar_parent_webrtc_peerconnection_legacy_callback_stats_used,
    processes.parent.scalars.webrtc_peerconnection_promise_and_callback_stats_used AS scalar_parent_webrtc_peerconnection_promise_and_callback_stats_used,
    processes.parent.scalars.webrtc_peerconnection_promise_stats_used AS scalar_parent_webrtc_peerconnection_promise_stats_used,
    processes.content.scalars.browser_feeds_preview_loaded AS scalar_content_browser_feeds_preview_loaded,
    processes.content.scalars.browser_usage_graphite AS scalar_content_browser_usage_graphite,
    processes.content.scalars.browser_usage_plugin_instantiated AS scalar_content_browser_usage_plugin_instantiated,
    processes.content.scalars.encoding_override_used AS scalar_content_encoding_override_used,
    processes.content.scalars.gfx_omtp_paint_wait_ratio AS scalar_content_gfx_omtp_paint_wait_ratio,
    processes.content.scalars.idb_type_persistent_count AS scalar_content_idb_type_persistent_count,
    processes.content.scalars.idb_type_temporary_count AS scalar_content_idb_type_temporary_count,
    processes.content.scalars.images_webp_content_observed AS scalar_content_images_webp_content_observed,
    processes.content.scalars.images_webp_probe_observed AS scalar_content_images_webp_probe_observed,
    processes.content.scalars.mathml_doc_count AS scalar_content_mathml_doc_count,
    processes.content.scalars.media_allowed_autoplay_no_audio_track_count AS scalar_content_media_allowed_autoplay_no_audio_track_count,
    processes.content.scalars.media_autoplay_default_blocked AS scalar_content_media_autoplay_default_blocked,
    processes.content.scalars.media_autoplay_would_be_allowed_count AS scalar_content_media_autoplay_would_be_allowed_count,
    processes.content.scalars.media_autoplay_would_not_be_allowed_count AS scalar_content_media_autoplay_would_not_be_allowed_count,
    -- define removed field for schema compatiblity
    NULL AS scalar_content_media_blocked_autoplay_no_audio_track_count,
    processes.content.scalars.media_blocked_no_metadata AS scalar_content_media_blocked_no_metadata,
    processes.content.scalars.media_blocked_no_metadata_endup_no_audio_track AS scalar_content_media_blocked_no_metadata_endup_no_audio_track,
    processes.content.scalars.media_page_count AS scalar_content_media_page_count,
    processes.content.scalars.media_page_had_media_count AS scalar_content_media_page_had_media_count,
    processes.content.scalars.media_page_had_play_revoked_count AS scalar_content_media_page_had_play_revoked_count,
    processes.content.scalars.mediarecorder_recording_count AS scalar_content_mediarecorder_recording_count,
    processes.content.scalars.memoryreporter_max_ghost_windows AS scalar_content_memoryreporter_max_ghost_windows,
    processes.content.scalars.navigator_storage_estimate_count AS scalar_content_navigator_storage_estimate_count,
    processes.content.scalars.navigator_storage_persist_count AS scalar_content_navigator_storage_persist_count,
    processes.content.scalars.pdf_viewer_fallback_shown AS scalar_content_pdf_viewer_fallback_shown,
    processes.content.scalars.pdf_viewer_print AS scalar_content_pdf_viewer_print,
    processes.content.scalars.pdf_viewer_used AS scalar_content_pdf_viewer_used,
    processes.content.scalars.script_preloader_mainthread_recompile AS scalar_content_script_preloader_mainthread_recompile,
    processes.content.scalars.sw_alternative_body_used_count AS scalar_content_sw_alternative_body_used_count,
    processes.content.scalars.sw_cors_res_for_so_req_count AS scalar_content_sw_cors_res_for_so_req_count,
    processes.content.scalars.sw_synthesized_res_count AS scalar_content_sw_synthesized_res_count,
    processes.content.scalars.telemetry_discarded_accumulations AS scalar_content_telemetry_discarded_accumulations,
    processes.content.scalars.telemetry_discarded_child_events AS scalar_content_telemetry_discarded_child_events,
    processes.content.scalars.telemetry_discarded_keyed_accumulations AS scalar_content_telemetry_discarded_keyed_accumulations,
    processes.content.scalars.telemetry_discarded_keyed_scalar_actions AS scalar_content_telemetry_discarded_keyed_scalar_actions,
    processes.content.scalars.telemetry_discarded_scalar_actions AS scalar_content_telemetry_discarded_scalar_actions,
    processes.content.scalars.telemetry_process_creation_timestamp_inconsistent AS scalar_content_telemetry_process_creation_timestamp_inconsistent,
    processes.content.scalars.telemetry_profile_directory_scans AS scalar_content_telemetry_profile_directory_scans,
    processes.content.scalars.webrtc_hostnameobfuscation_disabled_failed AS scalar_content_webrtc_hostnameobfuscation_disabled_failed,
    processes.content.scalars.webrtc_hostnameobfuscation_disabled_succeeded AS scalar_content_webrtc_hostnameobfuscation_disabled_succeeded,
    processes.content.scalars.webrtc_hostnameobfuscation_enabled_failed AS scalar_content_webrtc_hostnameobfuscation_enabled_failed,
    processes.content.scalars.webrtc_hostnameobfuscation_enabled_succeeded AS scalar_content_webrtc_hostnameobfuscation_enabled_succeeded,
    processes.content.scalars.webrtc_nicer_stun_retransmits AS scalar_content_webrtc_nicer_stun_retransmits,
    processes.content.scalars.webrtc_nicer_turn_401s AS scalar_content_webrtc_nicer_turn_401s,
    processes.content.scalars.webrtc_nicer_turn_403s AS scalar_content_webrtc_nicer_turn_403s,
    processes.content.scalars.webrtc_nicer_turn_438s AS scalar_content_webrtc_nicer_turn_438s,
    processes.content.scalars.webrtc_peerconnection_connected AS scalar_content_webrtc_peerconnection_connected,
    processes.content.scalars.webrtc_peerconnection_datachannel_created AS scalar_content_webrtc_peerconnection_datachannel_created,
    processes.content.scalars.webrtc_peerconnection_datachannel_max_life_used AS scalar_content_webrtc_peerconnection_datachannel_max_life_used,
    processes.content.scalars.webrtc_peerconnection_datachannel_max_retx_and_life_used AS scalar_content_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    processes.content.scalars.webrtc_peerconnection_datachannel_max_retx_used AS scalar_content_webrtc_peerconnection_datachannel_max_retx_used,
    processes.content.scalars.webrtc_peerconnection_legacy_callback_stats_used AS scalar_content_webrtc_peerconnection_legacy_callback_stats_used,
    processes.content.scalars.webrtc_peerconnection_promise_and_callback_stats_used AS scalar_content_webrtc_peerconnection_promise_and_callback_stats_used,
    processes.content.scalars.webrtc_peerconnection_promise_stats_used AS scalar_content_webrtc_peerconnection_promise_stats_used,
    processes.gpu.scalars.browser_usage_graphite AS scalar_gpu_browser_usage_graphite,
    processes.gpu.scalars.telemetry_discarded_accumulations AS scalar_gpu_telemetry_discarded_accumulations,
    processes.gpu.scalars.telemetry_discarded_child_events AS scalar_gpu_telemetry_discarded_child_events,
    processes.gpu.scalars.telemetry_discarded_keyed_accumulations AS scalar_gpu_telemetry_discarded_keyed_accumulations,
    processes.gpu.scalars.telemetry_discarded_keyed_scalar_actions AS scalar_gpu_telemetry_discarded_keyed_scalar_actions,
    processes.gpu.scalars.telemetry_discarded_scalar_actions AS scalar_gpu_telemetry_discarded_scalar_actions,
    processes.gpu.scalars.telemetry_profile_directory_scans AS scalar_gpu_telemetry_profile_directory_scans,
    processes.parent.keyed_scalars.a11y_theme AS scalar_parent_a11y_theme,
    processes.parent.keyed_scalars.browser_engagement_navigation_about_home AS scalar_parent_browser_engagement_navigation_about_home,
    processes.parent.keyed_scalars.browser_engagement_navigation_about_newtab AS scalar_parent_browser_engagement_navigation_about_newtab,
    processes.parent.keyed_scalars.browser_engagement_navigation_contextmenu AS scalar_parent_browser_engagement_navigation_contextmenu,
    processes.parent.keyed_scalars.browser_engagement_navigation_searchbar AS scalar_parent_browser_engagement_navigation_searchbar,
    processes.parent.keyed_scalars.browser_engagement_navigation_urlbar AS scalar_parent_browser_engagement_navigation_urlbar,
    processes.parent.keyed_scalars.browser_engagement_navigation_webextension AS scalar_parent_browser_engagement_navigation_webextension,
    processes.parent.keyed_scalars.browser_errors_collected_count_by_filename AS scalar_parent_browser_errors_collected_count_by_filename,
    processes.parent.keyed_scalars.browser_search_ad_clicks AS scalar_parent_browser_search_ad_clicks,
    processes.parent.keyed_scalars.browser_search_with_ads AS scalar_parent_browser_search_with_ads,
    processes.parent.keyed_scalars.devtools_accessibility_accessible_context_menu_item_activated AS scalar_parent_devtools_accessibility_accessible_context_menu_item_activated,
    processes.parent.keyed_scalars.devtools_accessibility_audit_activated AS scalar_parent_devtools_accessibility_audit_activated,
    processes.parent.keyed_scalars.devtools_accessibility_select_accessible_for_node AS scalar_parent_devtools_accessibility_select_accessible_for_node,
    processes.parent.keyed_scalars.devtools_accessibility_simulation_activated AS scalar_parent_devtools_accessibility_simulation_activated,
    processes.parent.keyed_scalars.devtools_current_theme AS scalar_parent_devtools_current_theme,
    processes.parent.keyed_scalars.devtools_inspector_three_pane_enabled AS scalar_parent_devtools_inspector_three_pane_enabled,
    processes.parent.keyed_scalars.devtools_responsive_open_trigger AS scalar_parent_devtools_responsive_open_trigger,
    processes.parent.keyed_scalars.devtools_tool_registered AS scalar_parent_devtools_tool_registered,
    processes.parent.keyed_scalars.devtools_toolbox_tabs_reordered AS scalar_parent_devtools_toolbox_tabs_reordered,
    processes.parent.keyed_scalars.devtools_tooltip_shown AS scalar_parent_devtools_tooltip_shown,
    processes.parent.keyed_scalars.extensions_updates_rdf AS scalar_parent_extensions_updates_rdf,
    processes.parent.keyed_scalars.gfx_advanced_layers_failure_id AS scalar_parent_gfx_advanced_layers_failure_id,
    processes.parent.keyed_scalars.images_webp_content_frequency AS scalar_parent_images_webp_content_frequency,
    -- define removed field for schema compatiblity
    ARRAY<STRUCT<key STRING, value INT64>>[] AS scalar_parent_networking_data_transferred,
    processes.parent.keyed_scalars.networking_data_transferred_kb AS scalar_parent_networking_data_transferred_kb,
    processes.parent.keyed_scalars.networking_data_transferred_v3_kb AS scalar_parent_networking_data_transferred_v3_kb,
    processes.parent.keyed_scalars.normandy_recipe_freshness AS scalar_parent_normandy_recipe_freshness,
    processes.parent.keyed_scalars.pictureinpicture_closed_method AS scalar_parent_pictureinpicture_closed_method,
    processes.parent.keyed_scalars.preferences_browser_home_page_change AS scalar_parent_preferences_browser_home_page_change,
    processes.parent.keyed_scalars.preferences_browser_home_page_count AS scalar_parent_preferences_browser_home_page_count,
    processes.parent.keyed_scalars.preferences_search_query AS scalar_parent_preferences_search_query,
    processes.parent.keyed_scalars.preferences_use_bookmark AS scalar_parent_preferences_use_bookmark,
    processes.parent.keyed_scalars.preferences_use_current_page AS scalar_parent_preferences_use_current_page,
    processes.parent.keyed_scalars.qm_origin_directory_unexpected_filename AS scalar_parent_qm_origin_directory_unexpected_filename,
    processes.parent.keyed_scalars.resistfingerprinting_content_window_size AS scalar_parent_resistfingerprinting_content_window_size,
    processes.parent.keyed_scalars.sandbox_no_job AS scalar_parent_sandbox_no_job,
    processes.parent.keyed_scalars.security_client_cert AS scalar_parent_security_client_cert,
    processes.parent.keyed_scalars.security_contentblocker_permissions AS scalar_parent_security_contentblocker_permissions,
    processes.parent.keyed_scalars.security_pkcs11_modules_loaded AS scalar_parent_security_pkcs11_modules_loaded,
    processes.parent.keyed_scalars.security_webauthn_used AS scalar_parent_security_webauthn_used,
    processes.parent.keyed_scalars.services_sync_sync_login_state_transitions AS scalar_parent_services_sync_sync_login_state_transitions,
    processes.parent.keyed_scalars.storage_sync_api_usage_items_stored AS scalar_parent_storage_sync_api_usage_items_stored,
    processes.parent.keyed_scalars.storage_sync_api_usage_storage_consumed AS scalar_parent_storage_sync_api_usage_storage_consumed,
    processes.parent.keyed_scalars.telemetry_accumulate_clamped_values AS scalar_parent_telemetry_accumulate_clamped_values,
    processes.parent.keyed_scalars.telemetry_accumulate_unknown_histogram_keys AS scalar_parent_telemetry_accumulate_unknown_histogram_keys,
    processes.parent.keyed_scalars.telemetry_event_counts AS scalar_parent_telemetry_event_counts,
    processes.parent.keyed_scalars.telemetry_keyed_scalars_exceed_limit AS scalar_parent_telemetry_keyed_scalars_exceed_limit,
    processes.parent.keyed_scalars.update_binarytransparencyresult AS scalar_parent_update_binarytransparencyresult,
    processes.parent.keyed_scalars.update_bitshresult AS scalar_parent_update_bitshresult,
    processes.parent.keyed_scalars.widget_ime_name_on_linux AS scalar_parent_widget_ime_name_on_linux,
    processes.parent.keyed_scalars.widget_ime_name_on_mac AS scalar_parent_widget_ime_name_on_mac,
    processes.parent.keyed_scalars.widget_ime_name_on_windows AS scalar_parent_widget_ime_name_on_windows,
    processes.content.keyed_scalars.dom_event_confluence_load_count AS scalar_content_dom_event_confluence_load_count,
    processes.content.keyed_scalars.dom_event_office_online_load_count AS scalar_content_dom_event_office_online_load_count,
    processes.content.keyed_scalars.gfx_small_paint_phase_weight AS scalar_content_gfx_small_paint_phase_weight,
    processes.content.keyed_scalars.images_webp_content_frequency AS scalar_content_images_webp_content_frequency,
    processes.content.keyed_scalars.pictureinpicture_opened_method AS scalar_content_pictureinpicture_opened_method,
    processes.content.keyed_scalars.telemetry_accumulate_unknown_histogram_keys AS scalar_content_telemetry_accumulate_unknown_histogram_keys,
    processes.content.keyed_scalars.telemetry_event_counts AS scalar_content_telemetry_event_counts,
    processes.content.keyed_scalars.webrtc_sdp_parser_diff AS scalar_content_webrtc_sdp_parser_diff,
    processes.content.keyed_scalars.webrtc_video_recv_codec_used AS scalar_content_webrtc_video_recv_codec_used,
    processes.content.keyed_scalars.webrtc_video_send_codec_used AS scalar_content_webrtc_video_send_codec_used,
    processes.gpu.keyed_scalars.telemetry_accumulate_unknown_histogram_keys AS scalar_gpu_telemetry_accumulate_unknown_histogram_keys,
    processes.gpu.keyed_scalars.telemetry_event_counts AS scalar_gpu_telemetry_event_counts
  )
);
