CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.search.mobile_search_aggregates_for_searchreport`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.search_derived.mobile_search_aggregates_for_searchreport_v1`
