CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.firefox_desktop_background_defaultagent.default_agent_agg`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.firefox_desktop_background_defaultagent_derived.default_agent_agg_v1`
