CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.org_mozilla_mozregression.mozregression_aggregates`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.org_mozilla_mozregression_derived.mozregression_aggregates_v1`
