CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.monitoring.bigquery_table_storage_timeline_daily`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.monitoring_derived.bigquery_table_storage_timeline_daily_v1`
