CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.experiment_cumulative_search_with_ads_count`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.telemetry_derived.experiment_cumulative_search_with_ads_count_v1`
