CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.desktop_acquisition_funnel_aggregates`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.telemetry_derived.desktop_acquisition_funnel_aggregates_v1`
