CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.active_users_aggregates_attribution`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.telemetry_derived.active_users_aggregates_attribution_v1`
