SELECT
  id,
  _fivetran_synced,
  address_country,
  address_postal_code,
  address_state,
  bank_account_id,
  default_card_id,
  invoice_settings_footer,
  metadata,
  shipping_carrier,
  shipping_tracking_number,
  source_id,
  tax_info_tax_id,
  tax_info_type,
  tax_info_verification_status,
  tax_info_verification_verified_name,
  is_deleted,
  tax_exempt,
  created,
  shipping_address_postal_code,
  shipping_address_state,
  shipping_address_country,
FROM
  `moz-fx-data-bq-fivetran`.stripe.customer
