CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.contextual_services.suggest_revenue_levers_daily`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.contextual_services_derived.suggest_revenue_levers_daily_v1`
