CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.fx_cert_error_unique_users_normalized_channel`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.telemetry_derived.fx_cert_error_unique_users_normalized_channel_v1`
