CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.subscription_platform.logical_subscription_events`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.subscription_platform_derived.logical_subscription_events_v1`
