CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.firefox_desktop.urlbar_events_daily_engagement_by_position`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.firefox_desktop_derived.urlbar_events_daily_engagement_by_position_v1`
