-- Query generated by: templates/unnest_parquet_view.sql.py crash_aggregates_v1 telemetry_derived.crash_aggregates_v1
CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.crash_aggregates_v1`
AS
SELECT
  * REPLACE (dimensions.key_value AS dimensions, stats.key_value AS stats)
FROM
  `moz-fx-data-shared-prod.telemetry_derived.crash_aggregates_v1`
