SELECT
  id,
  _fivetran_synced,
  active,
  caption,
  created,
  description,
  is_deleted,
  metadata,
  name,
  package_dimensions_height,
  package_dimensions_length,
  package_dimensions_weight,
  package_dimensions_width,
  shippable,
  statement_descriptor,
  type,
  unit_label,
  updated,
  url,
FROM
  `dev-fivetran`.stripe_nonprod.product
