SELECT
  id,
  _fivetran_synced,
  account_id,
  address_state,
  address_zip,
  address_zip_check,
  connected_account_id,
  country,
  customer_id,
  three_d_secure_usage_supported,
  wallet_type,
  description,
  issuer,
  iin,
  three_d_secure_authentication_flow,
  three_d_secure_version,
  three_d_secure_authenticated,
  three_d_secure_succeeded,
  three_d_secure_result,
  three_d_secure_result_reason,
  three_d_secure,
  address_country,
  created,
FROM
  `dev-fivetran`.stripe_nonprod.card
