CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.contextual_services.event_aggregates_spons_tiles`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.contextual_services_derived.event_aggregates_spons_tiles_v1`
