CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.search.desktop_search_aggregates_by_userstate`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.search_derived.desktop_search_aggregates_by_userstate_v1`
