SELECT
  name,
  value,
FROM
  EXTERNAL_QUERY(
    "moz-fx-fxa-nonprod.us.fxa-oauth-nonprod-stage-fxa-oauth",
    """SELECT
         name,
         value
       FROM
         fxa_oauth.dbMetadata
    """
  )
