CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.org_mozilla_ios_firefox.review_checker_clients`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.org_mozilla_ios_firefox_derived.review_checker_clients_v1`
