CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.mozilla_vpn.funnel_product_page_to_subscribed`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod`.mozilla_vpn_derived.funnel_product_page_to_subscribed_v1
