SELECT
  id,
  _fivetran_synced,
  active,
  code,
  coupon_id,
  created,
  customer_id,
  expires_at,
  first_time_transaction,
  max_redemptions,
  metadata,
  minimum_amount,
  minimum_amount_currency,
  times_redeemed,
FROM
  `dev-fivetran`.stripe_nonprod.promotion_code
