-- Query generated by: sql/telemetry_derived/clients_daily_histogram_aggregates_v1.sql.py
CREATE TEMP FUNCTION udf_aggregate_json_sum(histograms ARRAY<STRING>) AS (
  ARRAY(
    SELECT AS STRUCT
      FORMAT('%d', values_entry.key) AS key,
      SUM(values_entry.value) AS value
    FROM
      UNNEST(histograms) AS histogram,
      UNNEST(mozfun.hist.extract(histogram).values) AS values_entry
    WHERE
      histogram IS NOT NULL
    GROUP BY
      values_entry.key
    ORDER BY
      values_entry.key
  )
);

WITH valid_build_ids AS (
  SELECT DISTINCT
    (build.build.id) AS build_id
  FROM
    `moz-fx-data-shared-prod.telemetry.buildhub2`
),
filtered AS (
  SELECT
    *,
    SPLIT(application.version, '.')[OFFSET(0)] AS app_version,
    DATE(submission_timestamp) AS submission_date,
    normalized_os AS os,
    application.build_id AS app_build_id,
    normalized_channel AS channel
  FROM
    `moz-fx-data-shared-prod.telemetry_stable.main_v5`
  INNER JOIN
    valid_build_ids
    ON (application.build_id = build_id)
  WHERE
    DATE(submission_timestamp) = @submission_date
    AND normalized_channel IN ("release", "beta", "nightly")
    AND client_id IS NOT NULL
),
sampled_data AS (
  SELECT
    *
  FROM
    filtered
  WHERE
    channel IN ("nightly", "beta")
    OR (channel = "release" AND os != "Windows")
    OR (channel = "release" AND os = "Windows" AND MOD(sample_id, @sample_size) = 0)
),
histograms AS (
  SELECT
    submission_date,
    sample_id,
    client_id,
    os,
    app_version,
    app_build_id,
    channel,
    ARRAY<
      STRUCT<
        metric STRING,
        metric_type STRING,
        process STRING,
        value STRING,
        bucket_range STRUCT<first_bucket INT64, last_bucket INT64, num_buckets INT64>
      >
    >[
      (
        'a11y_consumers',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.a11y_consumers,
        (1, 11, 12)
      ),
      (
        'a11y_iatable_usage_flag',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.a11y_iatable_usage_flag,
        (1, 2, 3)
      ),
      (
        'a11y_instantiated_flag',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.a11y_instantiated_flag,
        (1, 2, 3)
      ),
      (
        'a11y_isimpledom_usage_flag',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.a11y_isimpledom_usage_flag,
        (1, 2, 3)
      ),
      (
        'a11y_tree_update_timing_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.a11y_tree_update_timing_ms,
        (1, 60000, 50)
      ),
      (
        'active_docgroups_per_tabgroup',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.active_docgroups_per_tabgroup,
        (1, 50, 20)
      ),
      (
        'active_http_docgroups_per_tabgroup',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.active_http_docgroups_per_tabgroup,
        (1, 50, 20)
      ),
      (
        'addon_manager_upgrade_ui_shown',
        'histogram-count',
        'content',
        payload.processes.content.histograms.addon_manager_upgrade_ui_shown,
        (1, 2, 3)
      ),
      (
        'alerts_service_dnd_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.alerts_service_dnd_enabled,
        (1, 2, 3)
      ),
      (
        'alerts_service_dnd_supported_flag',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.alerts_service_dnd_supported_flag,
        (1, 2, 3)
      ),
      (
        'application_reputation_allowlist_match',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.application_reputation_allowlist_match,
        (1, 4, 5)
      ),
      (
        'application_reputation_binary',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.application_reputation_binary,
        (1, 50, 51)
      ),
      (
        'application_reputation_binary_archive',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.application_reputation_binary_archive,
        (1, 50, 51)
      ),
      (
        'application_reputation_binary_type',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.application_reputation_binary_type,
        (1, 50, 51)
      ),
      (
        'application_reputation_blocklist_match',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.application_reputation_blocklist_match,
        (1, 4, 5)
      ),
      (
        'application_reputation_count',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.application_reputation_count,
        (1, 2, 3)
      ),
      (
        'application_reputation_hash_length',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.application_reputation_hash_length,
        (1, 50, 51)
      ),
      (
        'application_reputation_local',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.application_reputation_local,
        (1, 3, 4)
      ),
      (
        'application_reputation_reason',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.application_reputation_reason,
        (1, 50, 51)
      ),
      (
        'application_reputation_remote_lookup_response_time',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.application_reputation_remote_lookup_response_time,
        (1000, 15000, 17)
      ),
      (
        'application_reputation_remote_lookup_timeout',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.application_reputation_remote_lookup_timeout,
        (1, 2, 3)
      ),
      (
        'application_reputation_server',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.application_reputation_server,
        (1, 3, 4)
      ),
      (
        'application_reputation_server_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.application_reputation_server_2,
        (1, 50, 51)
      ),
      (
        'application_reputation_server_verdict',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.application_reputation_server_verdict,
        (1, 8, 9)
      ),
      (
        'application_reputation_should_block',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.application_reputation_should_block,
        (1, 2, 3)
      ),
      (
        'apz_aware_key_listeners',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.apz_aware_key_listeners,
        (1, 2, 3)
      ),
      (
        'apz_aware_mousemove_listeners',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.apz_aware_mousemove_listeners,
        (1, 2, 3)
      ),
      (
        'apz_zoom_activity',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.apz_zoom_activity,
        (1, 2, 3)
      ),
      (
        'apz_zoom_activity_rdm',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.apz_zoom_activity_rdm,
        (1, 2, 3)
      ),
      (
        'async_animation_content_too_large_frame_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.async_animation_content_too_large_frame_size,
        (1, 80000000, 100)
      ),
      (
        'async_animation_content_too_large_percentage',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.async_animation_content_too_large_percentage,
        (100, 1000, 50)
      ),
      (
        'async_animation_frame_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.async_animation_frame_size,
        (1, 80000000, 100)
      ),
      (
        'audio_mft_output_null_samples',
        'histogram-count',
        'content',
        payload.processes.content.histograms.audio_mft_output_null_samples,
        (1, 2, 3)
      ),
      (
        'audio_track_silence_proportion',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.audio_track_silence_proportion,
        (1, 100, 10)
      ),
      (
        'audiostream_backend_used',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.audiostream_backend_used,
        (1, 16, 17)
      ),
      (
        'audiostream_first_open_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.audiostream_first_open_ms,
        (1, 10000, 50)
      ),
      (
        'audiostream_later_open_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.audiostream_later_open_ms,
        (1, 10000, 50)
      ),
      (
        'aushelper_cpu_error_code',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.aushelper_cpu_error_code,
        (1, 16, 17)
      ),
      (
        'aushelper_cpu_result_code',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.aushelper_cpu_result_code,
        (1, 5, 6)
      ),
      (
        'aushelper_websense_error_code',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.aushelper_websense_error_code,
        (1, 8, 9)
      ),
      (
        'aushelper_websense_reg_exists',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.aushelper_websense_reg_exists,
        (1, 2, 3)
      ),
      (
        'auto_rejected_translation_offers',
        'histogram-count',
        'content',
        payload.processes.content.histograms.auto_rejected_translation_offers,
        (1, 2, 3)
      ),
      (
        'autoplay_default_setting_change',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.autoplay_default_setting_change,
        (1, 50, 51)
      ),
      (
        'autoplay_sites_setting_change',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.autoplay_sites_setting_change,
        (1, 50, 51)
      ),
      (
        'backgroundfilesaver_thread_count',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.backgroundfilesaver_thread_count,
        (1, 21, 22)
      ),
      (
        'bad_fallback_font',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.bad_fallback_font,
        (1, 2, 3)
      ),
      (
        'base_font_families_per_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.base_font_families_per_page,
        (1, 256, 64)
      ),
      (
        'bfcache_combo',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.bfcache_combo,
        (1, 50, 51)
      ),
      (
        'bfcache_page_restored',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.bfcache_page_restored,
        (1, 2, 3)
      ),
      (
        'blink_filesystem_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.blink_filesystem_used,
        (1, 2, 3)
      ),
      (
        'blocklist_sync_file_load',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.blocklist_sync_file_load,
        (1, 2, 3)
      ),
      (
        'box_align_props_in_blocks_flag',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.box_align_props_in_blocks_flag,
        (1, 2, 3)
      ),
      (
        'br_9_2_1_subject_alt_names',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.br_9_2_1_subject_alt_names,
        (1, 8, 9)
      ),
      (
        'br_9_2_2_subject_common_name',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.br_9_2_2_subject_common_name,
        (1, 8, 9)
      ),
      (
        'browser_attribution_errors',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.browser_attribution_errors,
        (1, 50, 51)
      ),
      (
        'browser_is_assist_default',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.browser_is_assist_default,
        (1, 2, 3)
      ),
      (
        'browser_is_user_default',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.browser_is_user_default,
        (1, 2, 3)
      ),
      (
        'browser_is_user_default_error',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.browser_is_user_default_error,
        (1, 2, 3)
      ),
      (
        'browser_set_default_always_check',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.browser_set_default_always_check,
        (1, 2, 3)
      ),
      (
        'browser_set_default_dialog_prompt_rawcount',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.browser_set_default_dialog_prompt_rawcount,
        (1, 250, 15)
      ),
      (
        'browser_set_default_error',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.browser_set_default_error,
        (1, 2, 3)
      ),
      (
        'browser_set_default_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.browser_set_default_result,
        (1, 4, 5)
      ),
      (
        'browser_shim_usage_blocked',
        'histogram-count',
        'content',
        payload.processes.content.histograms.browser_shim_usage_blocked,
        (1, 2, 3)
      ),
      (
        'browserprovider_xul_import_bookmarks',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.browserprovider_xul_import_bookmarks,
        (1, 50000, 20)
      ),
      (
        'bucket_order_errors',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.bucket_order_errors,
        (1, 3000, 10)
      ),
      (
        'busy_tab_abandoned',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.busy_tab_abandoned,
        (1, 50, 51)
      ),
      (
        'cache_device_search_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_device_search_2,
        (1, 10000, 50)
      ),
      (
        'cache_disk_search_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_disk_search_2,
        (1, 10000, 50)
      ),
      (
        'cache_lm_inconsistent',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cache_lm_inconsistent,
        (1, 2, 3)
      ),
      (
        'cache_memory_search_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_memory_search_2,
        (1, 10000, 50)
      ),
      (
        'cache_offline_search_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_offline_search_2,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_2,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_2,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsasyncdoomevent_run',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsasyncdoomevent_run,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsblockoncachethreadevent_run',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsblockoncachethreadevent_run,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_close',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_close,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_doom',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_doom,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_doomandfailpendingrequests',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_doomandfailpendingrequests,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getcacheelement',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getcacheelement,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getclientid',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getclientid,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getdatasize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getdatasize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getdeviceid',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getdeviceid,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getexpirationtime',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getexpirationtime,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getfetchcount',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getfetchcount,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getfile',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getfile,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getkey',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getkey,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getlastfetched',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getlastfetched,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getlastmodified',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getlastmodified,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getmetadataelement',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getmetadataelement,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getpredicteddatasize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getpredicteddatasize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getsecurityinfo',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getsecurityinfo,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getstoragedatasize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getstoragedatasize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_getstoragepolicy',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_getstoragepolicy,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_isstreambased',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_isstreambased,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_markvalid',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_markvalid,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_openinputstream',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_openinputstream,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_openoutputstream',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_openoutputstream,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_requestdatasizechange',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_requestdatasizechange,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setcacheelement',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setcacheelement,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setdatasize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setdatasize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setexpirationtime',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setexpirationtime,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setmetadataelement',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setmetadataelement,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setpredicteddatasize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setpredicteddatasize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setsecurityinfo',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setsecurityinfo,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_setstoragepolicy',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_setstoragepolicy,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheentrydescriptor_visitmetadata',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheentrydescriptor_visitmetadata,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_closeallstreams',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_closeallstreams,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_diskdeviceheapsize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_diskdeviceheapsize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_evictentriesforclient',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_evictentriesforclient,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_getcacheiotarget',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_getcacheiotarget,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_isstorageenabledforpolicy',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_isstorageenabledforpolicy,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_onprofilechanged',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_onprofilechanged,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_onprofileshutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_onprofileshutdown,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_opencacheentry',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_opencacheentry,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_processrequest',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_processrequest,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setdiskcachecapacity',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setdiskcachecapacity,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setdiskcacheenabled',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setdiskcacheenabled,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setdiskcachemaxentrysize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setdiskcachemaxentrysize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setdisksmartsize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setdisksmartsize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setmemorycache',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setmemorycache,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setmemorycachemaxentrysize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setmemorycachemaxentrysize,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setofflinecachecapacity',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setofflinecachecapacity,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_setofflinecacheenabled',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_setofflinecacheenabled,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_shutdown,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscacheservice_visitentries',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscacheservice_visitentries,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nscompressoutputstreamwrapper_release',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nscompressoutputstreamwrapper_release,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsdecompressinputstreamwrapper_release',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsdecompressinputstreamwrapper_release,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsinputstreamwrapper_closeinternal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsinputstreamwrapper_closeinternal,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsinputstreamwrapper_lazyinit',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsinputstreamwrapper_lazyinit,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsinputstreamwrapper_release',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsinputstreamwrapper_release,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsoutputstreamwrapper_closeinternal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsoutputstreamwrapper_closeinternal,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsoutputstreamwrapper_lazyinit',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsoutputstreamwrapper_lazyinit,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsoutputstreamwrapper_release',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsoutputstreamwrapper_release,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nsprocessrequestevent_run',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nsprocessrequestevent_run,
        (1, 10000, 50)
      ),
      (
        'cache_service_lock_wait_mainthread_nssetdisksmartsizecallback_notify',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cache_service_lock_wait_mainthread_nssetdisksmartsizecallback_notify,
        (1, 10000, 50)
      ),
      (
        'canvas_2d_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.canvas_2d_used,
        (1, 2, 3)
      ),
      (
        'canvas_webgl2_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.canvas_webgl2_success,
        (1, 2, 3)
      ),
      (
        'canvas_webgl_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.canvas_webgl_success,
        (1, 2, 3)
      ),
      (
        'canvas_webgl_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.canvas_webgl_used,
        (1, 2, 3)
      ),
      (
        'cert_chain_key_size_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_chain_key_size_status,
        (1, 4, 5)
      ),
      (
        'cert_chain_sha1_policy_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_chain_sha1_policy_status,
        (1, 6, 7)
      ),
      (
        'cert_ev_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_ev_status,
        (1, 10, 11)
      ),
      (
        'cert_ocsp_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cert_ocsp_enabled,
        (1, 2, 3)
      ),
      (
        'cert_ocsp_required',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cert_ocsp_required,
        (1, 2, 3)
      ),
      (
        'cert_pinning_failures_by_ca',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_pinning_failures_by_ca,
        (1, 256, 257)
      ),
      (
        'cert_pinning_moz_results',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cert_pinning_moz_results,
        (1, 2, 3)
      ),
      (
        'cert_pinning_moz_results_by_host',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_pinning_moz_results_by_host,
        (1, 512, 513)
      ),
      (
        'cert_pinning_moz_test_results',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cert_pinning_moz_test_results,
        (1, 2, 3)
      ),
      (
        'cert_pinning_moz_test_results_by_host',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_pinning_moz_test_results_by_host,
        (1, 512, 513)
      ),
      (
        'cert_pinning_results',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cert_pinning_results,
        (1, 2, 3)
      ),
      (
        'cert_pinning_test_results',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cert_pinning_test_results,
        (1, 2, 3)
      ),
      (
        'cert_validation_http_request_canceled_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cert_validation_http_request_canceled_time,
        (1, 60000, 200)
      ),
      (
        'cert_validation_http_request_failed_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cert_validation_http_request_failed_time,
        (1, 60000, 200)
      ),
      (
        'cert_validation_http_request_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_validation_http_request_result,
        (1, 16, 17)
      ),
      (
        'cert_validation_http_request_succeeded_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cert_validation_http_request_succeeded_time,
        (1, 60000, 200)
      ),
      (
        'cert_validation_success_by_ca',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cert_validation_success_by_ca,
        (1, 256, 257)
      ),
      (
        'changes_of_detected_language',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.changes_of_detected_language,
        (1, 2, 3)
      ),
      (
        'changes_of_target_language',
        'histogram-count',
        'content',
        payload.processes.content.histograms.changes_of_target_language,
        (1, 2, 3)
      ),
      (
        'charset_override_situation',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.charset_override_situation,
        (1, 8, 9)
      ),
      (
        'charset_override_used',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.charset_override_used,
        (1, 2, 3)
      ),
      (
        'check_addons_modified_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.check_addons_modified_ms,
        (1, 5000, 15)
      ),
      (
        'check_java_enabled',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.check_java_enabled,
        (1, 3000, 10)
      ),
      (
        'checkerboard_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.checkerboard_duration,
        (1, 100000, 50)
      ),
      (
        'checkerboard_peak',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.checkerboard_peak,
        (1, 66355200, 50)
      ),
      (
        'checkerboard_potential_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.checkerboard_potential_duration,
        (1, 1000000, 50)
      ),
      (
        'checkerboard_severity',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.checkerboard_severity,
        (1, 1073741824, 50)
      ),
      (
        'components_shim_accessed_by_content',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.components_shim_accessed_by_content,
        (1, 2, 3)
      ),
      (
        'composite_frame_roundtrip_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.composite_frame_roundtrip_time,
        (1, 1000, 50)
      ),
      (
        'composite_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.composite_time,
        (1, 1000, 50)
      ),
      (
        'compositor_animation_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.compositor_animation_duration,
        (1, 20000, 50)
      ),
      (
        'compositor_animation_max_contiguous_drops_apz',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.compositor_animation_max_contiguous_drops_apz,
        (1, 7200, 50)
      ),
      (
        'compositor_animation_max_contiguous_drops_chrome',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.compositor_animation_max_contiguous_drops_chrome,
        (1, 7200, 50)
      ),
      (
        'compositor_animation_max_contiguous_drops_content',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.compositor_animation_max_contiguous_drops_content,
        (1, 7200, 50)
      ),
      (
        'compositor_animation_max_layer_area',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.compositor_animation_max_layer_area,
        (1, 8294400, 50)
      ),
      (
        'compositor_animation_throughput_apz',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.compositor_animation_throughput_apz,
        (1, 1500, 50)
      ),
      (
        'compositor_animation_throughput_chrome',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.compositor_animation_throughput_chrome,
        (1, 1500, 50)
      ),
      (
        'compositor_animation_throughput_content',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.compositor_animation_throughput_content,
        (1, 1500, 50)
      ),
      (
        'container_used',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.container_used,
        (1, 5, 6)
      ),
      (
        'content_documents_destroyed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.content_documents_destroyed,
        (1, 2, 3)
      ),
      (
        'content_full_paint_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_full_paint_time,
        (1, 1000, 50)
      ),
      (
        'content_js_background_tick_delay_total_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_js_background_tick_delay_total_ms,
        (1, 100, 10)
      ),
      (
        'content_js_foreground_tick_delay_total_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_js_foreground_tick_delay_total_ms,
        (1, 100, 10)
      ),
      (
        'content_js_known_tick_delay_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_js_known_tick_delay_ms,
        (1, 100, 10)
      ),
      (
        'content_paint_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_paint_time,
        (1, 1000, 50)
      ),
      (
        'content_process_launch_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_process_launch_time_ms,
        (1, 64000, 100)
      ),
      (
        'content_response_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.content_response_duration,
        (1, 60000, 50)
      ),
      (
        'content_signature_verification_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.content_signature_verification_status,
        (1, 20, 21)
      ),
      (
        'cookie_leave_secure_alone',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cookie_leave_secure_alone,
        (1, 10, 11)
      ),
      (
        'cookie_scheme_https',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cookie_scheme_https,
        (1, 10, 11)
      ),
      (
        'cookie_scheme_security',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.cookie_scheme_security,
        (1, 10, 11)
      ),
      (
        'cookie_time_moving_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cookie_time_moving_ms,
        (1, 3000, 20)
      ),
      (
        'crash_store_compressed_bytes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.crash_store_compressed_bytes,
        (1, 1000000, 202)
      ),
      (
        'create_event_beforeunloadevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_beforeunloadevent,
        (1, 2, 3)
      ),
      (
        'create_event_compositionevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_compositionevent,
        (1, 2, 3)
      ),
      (
        'create_event_customevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_customevent,
        (1, 2, 3)
      ),
      (
        'create_event_devicemotionevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_devicemotionevent,
        (1, 2, 3)
      ),
      (
        'create_event_deviceorientationevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_deviceorientationevent,
        (1, 2, 3)
      ),
      (
        'create_event_dragevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_dragevent,
        (1, 2, 3)
      ),
      (
        'create_event_errorevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_errorevent,
        (1, 2, 3)
      ),
      (
        'create_event_event',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_event,
        (1, 2, 3)
      ),
      (
        'create_event_events',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_events,
        (1, 2, 3)
      ),
      (
        'create_event_hashchangeevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_hashchangeevent,
        (1, 2, 3)
      ),
      (
        'create_event_htmlevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_htmlevents,
        (1, 2, 3)
      ),
      (
        'create_event_keyboardevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_keyboardevent,
        (1, 2, 3)
      ),
      (
        'create_event_keyevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_keyevents,
        (1, 2, 3)
      ),
      (
        'create_event_messageevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_messageevent,
        (1, 2, 3)
      ),
      (
        'create_event_mouseevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_mouseevent,
        (1, 2, 3)
      ),
      (
        'create_event_mouseevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_mouseevents,
        (1, 2, 3)
      ),
      (
        'create_event_mousescrollevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_mousescrollevents,
        (1, 2, 3)
      ),
      (
        'create_event_mutationevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_mutationevent,
        (1, 2, 3)
      ),
      (
        'create_event_mutationevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_mutationevents,
        (1, 2, 3)
      ),
      (
        'create_event_popstateevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_popstateevent,
        (1, 2, 3)
      ),
      (
        'create_event_scrollareaevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_scrollareaevent,
        (1, 2, 3)
      ),
      (
        'create_event_storageevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_storageevent,
        (1, 2, 3)
      ),
      (
        'create_event_svgevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_svgevents,
        (1, 2, 3)
      ),
      (
        'create_event_textevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_textevent,
        (1, 2, 3)
      ),
      (
        'create_event_timeevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_timeevent,
        (1, 2, 3)
      ),
      (
        'create_event_touchevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_touchevent,
        (1, 2, 3)
      ),
      (
        'create_event_uievent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_uievent,
        (1, 2, 3)
      ),
      (
        'create_event_uievents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_uievents,
        (1, 2, 3)
      ),
      (
        'create_event_xulcommandevent',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_xulcommandevent,
        (1, 2, 3)
      ),
      (
        'create_event_xulcommandevents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.create_event_xulcommandevents,
        (1, 2, 3)
      ),
      (
        'csp_documents_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.csp_documents_count,
        (1, 2, 3)
      ),
      (
        'csp_referrer_directive',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.csp_referrer_directive,
        (1, 2, 3)
      ),
      (
        'csp_unsafe_eval_documents_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.csp_unsafe_eval_documents_count,
        (1, 2, 3)
      ),
      (
        'csp_unsafe_inline_documents_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.csp_unsafe_inline_documents_count,
        (1, 2, 3)
      ),
      (
        'cycle_collector',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector,
        (1, 10000, 50)
      ),
      (
        'cycle_collector_async_snow_white_freeing',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_async_snow_white_freeing,
        (1, 10000, 50)
      ),
      (
        'cycle_collector_collected',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_collected,
        (1, 100000, 50)
      ),
      (
        'cycle_collector_finish_igc',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cycle_collector_finish_igc,
        (1, 2, 3)
      ),
      (
        'cycle_collector_full',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_full,
        (1, 10000, 50)
      ),
      (
        'cycle_collector_max_pause',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_max_pause,
        (1, 10000, 50)
      ),
      (
        'cycle_collector_need_gc',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cycle_collector_need_gc,
        (1, 2, 3)
      ),
      (
        'cycle_collector_oom',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.cycle_collector_oom,
        (1, 2, 3)
      ),
      (
        'cycle_collector_slice_during_idle',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.cycle_collector_slice_during_idle,
        (1, 100, 50)
      ),
      (
        'cycle_collector_sync_skippable',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cycle_collector_sync_skippable,
        (1, 2, 3)
      ),
      (
        'cycle_collector_time_between',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_time_between,
        (1, 120, 50)
      ),
      (
        'cycle_collector_visited_gced',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_visited_gced,
        (1, 300000, 50)
      ),
      (
        'cycle_collector_visited_ref_counted',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_visited_ref_counted,
        (1, 300000, 50)
      ),
      (
        'cycle_collector_worker',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_worker,
        (1, 10000, 50)
      ),
      (
        'cycle_collector_worker_collected',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_worker_collected,
        (1, 100000, 50)
      ),
      (
        'cycle_collector_worker_need_gc',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.cycle_collector_worker_need_gc,
        (1, 2, 3)
      ),
      (
        'cycle_collector_worker_oom',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.cycle_collector_worker_oom,
        (1, 2, 3)
      ),
      (
        'cycle_collector_worker_visited_gced',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_worker_visited_gced,
        (1, 300000, 50)
      ),
      (
        'cycle_collector_worker_visited_ref_counted',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.cycle_collector_worker_visited_ref_counted,
        (1, 300000, 50)
      ),
      (
        'd3d11_sync_handle_failure',
        'histogram-count',
        'content',
        payload.processes.content.histograms.d3d11_sync_handle_failure,
        (1, 2, 3)
      ),
      (
        'data_storage_entries',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.data_storage_entries,
        (1, 1024, 16)
      ),
      (
        'database_locked_exception',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.database_locked_exception,
        (1, 5, 6)
      ),
      (
        'database_successful_unlock',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.database_successful_unlock,
        (1, 5, 6)
      ),
      (
        'decoder_instantiated_ibm866',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_ibm866,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_iso2022jp',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_iso2022jp,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_iso_8859_5',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_iso_8859_5,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_koi8r',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_koi8r,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_koi8u',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_koi8u,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macarabic',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macarabic,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macce',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macce,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_maccroatian',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_maccroatian,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_maccyrillic',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_maccyrillic,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macdevanagari',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macdevanagari,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macfarsi',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macfarsi,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macgreek',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macgreek,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macgujarati',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macgujarati,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macgurmukhi',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macgurmukhi,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_machebrew',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_machebrew,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macicelandic',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macicelandic,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macromanian',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macromanian,
        (1, 2, 3)
      ),
      (
        'decoder_instantiated_macturkish',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.decoder_instantiated_macturkish,
        (1, 2, 3)
      ),
      (
        'dedicated_worker_destroyed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.dedicated_worker_destroyed,
        (1, 2, 3)
      ),
      (
        'dedicated_worker_spawn_gets_queued',
        'histogram-count',
        'content',
        payload.processes.content.histograms.dedicated_worker_spawn_gets_queued,
        (1, 2, 3)
      ),
      (
        'defective_permissions_sql_removed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.defective_permissions_sql_removed,
        (1, 2, 3)
      ),
      (
        'deferred_finalize_async',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.deferred_finalize_async,
        (1, 10000, 50)
      ),
      (
        'denied_translation_offers',
        'histogram-count',
        'content',
        payload.processes.content.histograms.denied_translation_offers,
        (1, 2, 3)
      ),
      (
        'device_reset_reason',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.device_reset_reason,
        (1, 10, 11)
      ),
      (
        'devtools_aboutdebugging_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_aboutdebugging_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_aboutdebugging_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_aboutdebugging_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_accessibility_picker_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_accessibility_picker_time_active_seconds,
        (1, 86400, 100)
      ),
      (
        'devtools_accessibility_service_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_accessibility_service_time_active_seconds,
        (1, 86400, 100)
      ),
      (
        'devtools_accessibility_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_accessibility_time_active_seconds,
        (1, 86400, 100)
      ),
      (
        'devtools_animationinspector_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_animationinspector_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_animationinspector_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_animationinspector_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_application_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_application_time_active_seconds,
        (1, 86400, 100)
      ),
      (
        'devtools_browserconsole_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_browserconsole_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_browserconsole_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_browserconsole_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_canvasdebugger_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_canvasdebugger_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_canvasdebugger_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_canvasdebugger_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_changesview_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_changesview_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_compatibilityview_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_compatibilityview_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_compatibilityview_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_compatibilityview_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_computedview_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_computedview_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_computedview_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_computedview_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_custom_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_custom_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_custom_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_custom_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_debugger_display_source_local_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_debugger_display_source_local_ms,
        (1, 10000, 1000)
      ),
      (
        'devtools_debugger_display_source_remote_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_debugger_display_source_remote_ms,
        (1, 10000, 1000)
      ),
      (
        'devtools_debugger_load_source_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_debugger_load_source_ms,
        (1, 20000, 100)
      ),
      (
        'devtools_developertoolbar_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_developertoolbar_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_developertoolbar_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_developertoolbar_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_dom_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_dom_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_dom_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_dom_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_eyedropper_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_eyedropper_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_flexbox_highlighter_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_flexbox_highlighter_time_active_seconds,
        (1, 86400, 100)
      ),
      (
        'devtools_fontinspector_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_fontinspector_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_fontinspector_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_fontinspector_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_grid_highlighter_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_grid_highlighter_time_active_seconds,
        (1, 86400, 100)
      ),
      (
        'devtools_heap_snapshot_edge_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_heap_snapshot_edge_count,
        (1, 10000000, 10000)
      ),
      (
        'devtools_heap_snapshot_node_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_heap_snapshot_node_count,
        (1, 10000000, 10000)
      ),
      (
        'devtools_inspector_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_inspector_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_inspector_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_inspector_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_jsbrowserdebugger_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_jsbrowserdebugger_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_jsbrowserdebugger_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_jsbrowserdebugger_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_jsdebugger_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_jsdebugger_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_jsdebugger_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_jsdebugger_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_jsprofiler_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_jsprofiler_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_jsprofiler_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_jsprofiler_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_layoutview_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_layoutview_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_layoutview_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_layoutview_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_memory_diff_census',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_memory_diff_census,
        (1, 2, 3)
      ),
      (
        'devtools_memory_dominator_tree_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_memory_dominator_tree_count,
        (1, 2, 3)
      ),
      (
        'devtools_memory_export_snapshot_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_memory_export_snapshot_count,
        (1, 2, 3)
      ),
      (
        'devtools_memory_filter_census',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_memory_filter_census,
        (1, 2, 3)
      ),
      (
        'devtools_memory_import_snapshot_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_memory_import_snapshot_count,
        (1, 2, 3)
      ),
      (
        'devtools_memory_inverted_census',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_memory_inverted_census,
        (1, 2, 3)
      ),
      (
        'devtools_memory_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_memory_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_memory_take_snapshot_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_memory_take_snapshot_count,
        (1, 2, 3)
      ),
      (
        'devtools_memory_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_memory_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_menu_eyedropper_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_menu_eyedropper_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_netmonitor_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_netmonitor_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_netmonitor_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_netmonitor_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_number_of_css_grids_in_a_page',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_number_of_css_grids_in_a_page,
        (1, 30, 29)
      ),
      (
        'devtools_options_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_options_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_options_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_options_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_os_enumerated_per_user',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.devtools_os_enumerated_per_user,
        (1, 13, 14)
      ),
      (
        'devtools_os_is_64_bits_per_user',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.devtools_os_is_64_bits_per_user,
        (1, 3, 4)
      ),
      (
        'devtools_paintflashing_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_paintflashing_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_paintflashing_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_paintflashing_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_perftools_console_recording_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_perftools_console_recording_count,
        (1, 2, 3)
      ),
      (
        'devtools_perftools_recording_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_perftools_recording_count,
        (1, 2, 3)
      ),
      (
        'devtools_perftools_recording_duration_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_perftools_recording_duration_ms,
        (1, 600000, 20)
      ),
      (
        'devtools_perftools_recording_export_flag',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.devtools_perftools_recording_export_flag,
        (1, 2, 3)
      ),
      (
        'devtools_perftools_recording_import_flag',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.devtools_perftools_recording_import_flag,
        (1, 2, 3)
      ),
      (
        'devtools_picker_eyedropper_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_picker_eyedropper_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_read_heap_snapshot_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_read_heap_snapshot_ms,
        (1, 100000, 1000)
      ),
      (
        'devtools_reload_addon_installed_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_reload_addon_installed_count,
        (1, 2, 3)
      ),
      (
        'devtools_reload_addon_reload_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_reload_addon_reload_count,
        (1, 2, 3)
      ),
      (
        'devtools_responsive_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_responsive_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_responsive_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_responsive_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_ruleview_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_ruleview_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_ruleview_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_ruleview_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_save_heap_snapshot_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_save_heap_snapshot_ms,
        (1, 100000, 1000)
      ),
      (
        'devtools_scratchpad_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_scratchpad_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_scratchpad_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_scratchpad_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_scratchpad_window_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_scratchpad_window_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_screen_resolution_enumerated_per_user',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.devtools_screen_resolution_enumerated_per_user,
        (1, 13, 14)
      ),
      (
        'devtools_shadereditor_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_shadereditor_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_shadereditor_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_shadereditor_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_storage_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_storage_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_storage_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_storage_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_styleeditor_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_styleeditor_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_styleeditor_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_styleeditor_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_tabs_open_average_linear',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_tabs_open_average_linear,
        (1, 101, 100)
      ),
      (
        'devtools_tabs_open_peak_linear',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_tabs_open_peak_linear,
        (1, 101, 100)
      ),
      (
        'devtools_tabs_pinned_average_linear',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_tabs_pinned_average_linear,
        (1, 101, 100)
      ),
      (
        'devtools_tabs_pinned_peak_linear',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.devtools_tabs_pinned_peak_linear,
        (1, 101, 100)
      ),
      (
        'devtools_tilt_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_tilt_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_tilt_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_tilt_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_toolbox_host',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.devtools_toolbox_host,
        (1, 9, 10)
      ),
      (
        'devtools_toolbox_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_toolbox_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_toolbox_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_toolbox_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_webaudioeditor_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_webaudioeditor_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_webaudioeditor_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_webaudioeditor_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_webconsole_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_webconsole_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_webconsole_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_webconsole_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_webide_connection_debug_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_connection_debug_used,
        (1, 2, 3)
      ),
      (
        'devtools_webide_connection_play_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_connection_play_used,
        (1, 2, 3)
      ),
      (
        'devtools_webide_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_connection_result,
        (1, 2, 3)
      ),
      (
        'devtools_webide_connection_time_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_webide_connection_time_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_webide_import_project_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_webide_import_project_count,
        (1, 2, 3)
      ),
      (
        'devtools_webide_local_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_local_connection_result,
        (1, 2, 3)
      ),
      (
        'devtools_webide_new_project_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_webide_new_project_count,
        (1, 2, 3)
      ),
      (
        'devtools_webide_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.devtools_webide_opened_count,
        (1, 2, 3)
      ),
      (
        'devtools_webide_other_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_other_connection_result,
        (1, 2, 3)
      ),
      (
        'devtools_webide_remote_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_remote_connection_result,
        (1, 2, 3)
      ),
      (
        'devtools_webide_simulator_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_simulator_connection_result,
        (1, 2, 3)
      ),
      (
        'devtools_webide_time_active_seconds',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.devtools_webide_time_active_seconds,
        (1, 10000000, 100)
      ),
      (
        'devtools_webide_usb_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_usb_connection_result,
        (1, 2, 3)
      ),
      (
        'devtools_webide_wifi_connection_result',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.devtools_webide_wifi_connection_result,
        (1, 2, 3)
      ),
      (
        'display_item_usage_count',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.display_item_usage_count,
        (1, 99, 100)
      ),
      (
        'display_scaling',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.display_scaling,
        (1, 500, 100)
      ),
      (
        'dns_blacklist_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dns_blacklist_count,
        (1, 21, 20)
      ),
      (
        'dns_cleanup_age',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dns_cleanup_age,
        (1, 1440, 50)
      ),
      (
        'dns_failed_lookup_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dns_failed_lookup_time,
        (1, 60000, 50)
      ),
      (
        'dns_lookup_method2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.dns_lookup_method2,
        (1, 16, 17)
      ),
      (
        'dns_lookup_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dns_lookup_time,
        (1, 60000, 50)
      ),
      (
        'dns_renewal_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dns_renewal_time,
        (1, 60000, 50)
      ),
      (
        'dns_renewal_time_for_ttl',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dns_renewal_time_for_ttl,
        (1, 60000, 50)
      ),
      (
        'dns_trr_request_per_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dns_trr_request_per_conn,
        (1, 5000, 100)
      ),
      (
        'dnt_usage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.dnt_usage,
        (1, 3, 4)
      ),
      (
        'document_analytics_tracker_fastblocked',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.document_analytics_tracker_fastblocked,
        (1, 50, 51)
      ),
      (
        'document_data_uri_loads',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.document_data_uri_loads,
        (1, 3, 4)
      ),
      (
        'document_with_expanded_principal',
        'histogram-count',
        'content',
        payload.processes.content.histograms.document_with_expanded_principal,
        (1, 2, 3)
      ),
      (
        'dom_script_bytecode_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_bytecode_size,
        (64, 16000000, 50)
      ),
      (
        'dom_script_encoding_ms_per_document',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_encoding_ms_per_document,
        (1, 64, 32)
      ),
      (
        'dom_script_encoding_status',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.dom_script_encoding_status,
        (1, 50, 51)
      ),
      (
        'dom_script_eval_per_document',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_eval_per_document,
        (1, 200, 50)
      ),
      (
        'dom_script_fetch_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_fetch_count,
        (1, 100, 99)
      ),
      (
        'dom_script_inline_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_inline_size,
        (64, 16000000, 50)
      ),
      (
        'dom_script_is_streamed',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.dom_script_is_streamed,
        (1, 2, 3)
      ),
      (
        'dom_script_kind',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.dom_script_kind,
        (1, 50, 51)
      ),
      (
        'dom_script_load_emit_time_percent',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_load_emit_time_percent,
        (2, 100, 99)
      ),
      (
        'dom_script_load_incremental_avg_transfer_rate',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_load_incremental_avg_transfer_rate,
        (1, 1000000, 60)
      ),
      (
        'dom_script_load_incremental_ratio',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_load_incremental_ratio,
        (2, 100, 99)
      ),
      (
        'dom_script_load_parse_time_percent',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_load_parse_time_percent,
        (2, 100, 99)
      ),
      (
        'dom_script_load_stream_time_percent',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_load_stream_time_percent,
        (2, 100, 99)
      ),
      (
        'dom_script_load_streamparse_estimate_percent',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.dom_script_load_streamparse_estimate_percent,
        (2, 100, 99)
      ),
      (
        'dom_script_loading_source',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.dom_script_loading_source,
        (1, 50, 51)
      ),
      (
        'dom_script_main_thread_decode_exec_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_main_thread_decode_exec_ms,
        (1, 50000, 100)
      ),
      (
        'dom_script_main_thread_parse_encode_exec_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_main_thread_parse_encode_exec_ms,
        (1, 50000, 100)
      ),
      (
        'dom_script_main_thread_parse_exec_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_main_thread_parse_exec_ms,
        (1, 50000, 100)
      ),
      (
        'dom_script_off_thread_decode_exec_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_off_thread_decode_exec_ms,
        (1, 50000, 100)
      ),
      (
        'dom_script_off_thread_parse_encode_exec_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_off_thread_parse_encode_exec_ms,
        (1, 50000, 100)
      ),
      (
        'dom_script_off_thread_parse_exec_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_off_thread_parse_exec_ms,
        (1, 50000, 100)
      ),
      (
        'dom_script_preload_result',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.dom_script_preload_result,
        (1, 50, 51)
      ),
      (
        'dom_script_source_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dom_script_source_size,
        (64, 16000000, 50)
      ),
      (
        'dwritefont_delayedinitfontlist_collect',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dwritefont_delayedinitfontlist_collect,
        (1, 30000, 10)
      ),
      (
        'dwritefont_delayedinitfontlist_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dwritefont_delayedinitfontlist_count,
        (1, 10000, 10)
      ),
      (
        'dwritefont_delayedinitfontlist_total',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.dwritefont_delayedinitfontlist_total,
        (1, 30000, 10)
      ),
      (
        'dwritefont_init_problem',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.dwritefont_init_problem,
        (1, 8, 9)
      ),
      (
        'e10s_blocked_from_running',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.e10s_blocked_from_running,
        (1, 2, 3)
      ),
      (
        'e10s_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.e10s_status,
        (1, 12, 13)
      ),
      (
        'enable_privilege_ever_called',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.enable_privilege_ever_called,
        (1, 2, 3)
      ),
      (
        'encoding_override_situation',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.encoding_override_situation,
        (1, 50, 51)
      ),
      (
        'eventloop_ui_activity_exp_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.eventloop_ui_activity_exp_ms,
        (1, 60000, 50)
      ),
      (
        'extension_install_prompt_result',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.extension_install_prompt_result,
        (1, 50, 51)
      ),
      (
        'fallback_to_base_font',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fallback_to_base_font,
        (1, 2, 3)
      ),
      (
        'fallback_to_langpack_font',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fallback_to_langpack_font,
        (1, 2, 3)
      ),
      (
        'fallback_to_prefs_font',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fallback_to_prefs_font,
        (1, 2, 3)
      ),
      (
        'fallback_to_user_font',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fallback_to_user_font,
        (1, 2, 3)
      ),
      (
        'family_safety',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.family_safety,
        (1, 16, 17)
      ),
      (
        'feed_protocol_usage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.feed_protocol_usage,
        (1, 8, 9)
      ),
      (
        'fennec_activity_stream_highlights_loader_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_activity_stream_highlights_loader_time_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_activity_stream_topsites_loader_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_activity_stream_topsites_loader_time_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_bookmarks_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_bookmarks_count,
        (1, 8000, 20)
      ),
      (
        'fennec_custom_homepage',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fennec_custom_homepage,
        (1, 2, 3)
      ),
      (
        'fennec_distribution_code_category',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fennec_distribution_code_category,
        (1, 20, 21)
      ),
      (
        'fennec_distribution_download_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_distribution_download_time_ms,
        (100, 40000, 30)
      ),
      (
        'fennec_distribution_referrer_invalid',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_distribution_referrer_invalid,
        (1, 2, 3)
      ),
      (
        'fennec_globalhistory_add_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_globalhistory_add_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_globalhistory_update_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_globalhistory_update_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_globalhistory_visited_build_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_globalhistory_visited_build_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_homepanels_custom',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fennec_homepanels_custom,
        (1, 2, 3)
      ),
      (
        'fennec_load_saved_page',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fennec_load_saved_page,
        (1, 10, 11)
      ),
      (
        'fennec_loop_other_latency',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_loop_other_latency,
        (10, 10485760, 22)
      ),
      (
        'fennec_loop_ui_latency',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_loop_ui_latency,
        (10, 10485760, 22)
      ),
      (
        'fennec_orbot_installed',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_orbot_installed,
        (1, 2, 3)
      ),
      (
        'fennec_reader_view_cache_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_reader_view_cache_size,
        (32, 51200, 20)
      ),
      (
        'fennec_reading_list_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_reading_list_count,
        (1, 1000, 10)
      ),
      (
        'fennec_restoring_activity',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_restoring_activity,
        (1, 2, 3)
      ),
      (
        'fennec_search_loader_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_search_loader_time_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_sessionstore_all_files_damaged',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_sessionstore_all_files_damaged,
        (1, 2, 3)
      ),
      (
        'fennec_sessionstore_damaged_session_file',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_sessionstore_damaged_session_file,
        (1, 2, 3)
      ),
      (
        'fennec_sessionstore_restoring_from_backup',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_sessionstore_restoring_from_backup,
        (1, 2, 3)
      ),
      (
        'fennec_startup_time_geckoready',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_startup_time_geckoready,
        (500, 20000, 20)
      ),
      (
        'fennec_startup_time_javaui',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_startup_time_javaui,
        (100, 5000, 20)
      ),
      (
        'fennec_sync11_migration_notifications_offered',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_sync11_migration_notifications_offered,
        (1, 500, 5)
      ),
      (
        'fennec_sync11_migration_sentinels_seen',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync11_migration_sentinels_seen,
        (1, 2, 3)
      ),
      (
        'fennec_sync11_migrations_completed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync11_migrations_completed,
        (1, 2, 3)
      ),
      (
        'fennec_sync11_migrations_failed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync11_migrations_failed,
        (1, 2, 3)
      ),
      (
        'fennec_sync11_migrations_succeeded',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync11_migrations_succeeded,
        (1, 2, 3)
      ),
      (
        'fennec_sync_number_of_syncs_completed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync_number_of_syncs_completed,
        (1, 2, 3)
      ),
      (
        'fennec_sync_number_of_syncs_failed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync_number_of_syncs_failed,
        (1, 2, 3)
      ),
      (
        'fennec_sync_number_of_syncs_failed_backoff',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync_number_of_syncs_failed_backoff,
        (1, 2, 3)
      ),
      (
        'fennec_sync_number_of_syncs_started',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fennec_sync_number_of_syncs_started,
        (1, 2, 3)
      ),
      (
        'fennec_tabqueue_queuesize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_tabqueue_queuesize,
        (1, 50, 10)
      ),
      (
        'fennec_topsites_loader_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fennec_topsites_loader_time_ms,
        (10, 20000, 20)
      ),
      (
        'fennec_tracking_protection_state',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fennec_tracking_protection_state,
        (1, 5, 6)
      ),
      (
        'fennec_was_killed',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fennec_was_killed,
        (1, 2, 3)
      ),
      (
        'fetch_is_mainthread',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fetch_is_mainthread,
        (1, 2, 3)
      ),
      (
        'file_embedded_serviceworkers',
        'histogram-count',
        'content',
        payload.processes.content.histograms.file_embedded_serviceworkers,
        (1, 2, 3)
      ),
      (
        'find_plugins',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.find_plugins,
        (1, 3000, 10)
      ),
      (
        'fips_enabled',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fips_enabled,
        (1, 2, 3)
      ),
      (
        'font_cache_hit',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.font_cache_hit,
        (1, 2, 3)
      ),
      (
        'fontlist_initfacenamelists',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fontlist_initfacenamelists,
        (1, 30000, 50)
      ),
      (
        'fontlist_initotherfamilynames',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fontlist_initotherfamilynames,
        (1, 30000, 50)
      ),
      (
        'fontlist_initotherfamilynames_no_deferring',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fontlist_initotherfamilynames_no_deferring,
        (1, 30000, 50)
      ),
      (
        'forced_device_reset_reason',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.forced_device_reset_reason,
        (1, 50, 51)
      ),
      (
        'forget_skippable_during_idle',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.forget_skippable_during_idle,
        (1, 100, 50)
      ),
      (
        'forget_skippable_frequency',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.forget_skippable_frequency,
        (1, 500, 100)
      ),
      (
        'forget_skippable_max',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.forget_skippable_max,
        (1, 10000, 50)
      ),
      (
        'form_isindex_used',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.form_isindex_used,
        (1, 2, 3)
      ),
      (
        'fullscreen_change_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fullscreen_change_ms,
        (100, 5000, 50)
      ),
      (
        'fullscreen_transition_black_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fullscreen_transition_black_ms,
        (100, 5000, 50)
      ),
      (
        'fx_bookmarks_toolbar_init_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_bookmarks_toolbar_init_ms,
        (50, 5000, 10)
      ),
      (
        'fx_browser_fullscreen_used',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fx_browser_fullscreen_used,
        (1, 2, 3)
      ),
      (
        'fx_content_crash_dump_unavailable',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fx_content_crash_dump_unavailable,
        (1, 2, 3)
      ),
      (
        'fx_content_crash_not_submitted',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fx_content_crash_not_submitted,
        (1, 2, 3)
      ),
      (
        'fx_content_crash_presented',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fx_content_crash_presented,
        (1, 2, 3)
      ),
      (
        'fx_gesture_compress_snapshot_of_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_gesture_compress_snapshot_of_page,
        (1, 1000, 30)
      ),
      (
        'fx_gesture_install_snapshot_of_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_gesture_install_snapshot_of_page,
        (1, 1000, 30)
      ),
      (
        'fx_migration_entry_point',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_migration_entry_point,
        (1, 10, 11)
      ),
      (
        'fx_migration_source_browser',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_migration_source_browser,
        (1, 15, 16)
      ),
      (
        'fx_new_window_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_new_window_ms,
        (1, 10000, 20)
      ),
      (
        'fx_page_load_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_page_load_ms,
        (1, 10000, 20)
      ),
      (
        'fx_page_load_ms_2',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.fx_page_load_ms_2,
        (1, 10000, 200)
      ),
      (
        'fx_preferences_category_opened',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.fx_preferences_category_opened,
        (1, 50, 51)
      ),
      (
        'fx_preferences_category_opened_v2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.fx_preferences_category_opened_v2,
        (1, 50, 51)
      ),
      (
        'fx_preferences_opened_via',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.fx_preferences_opened_via,
        (1, 50, 51)
      ),
      (
        'fx_refresh_driver_chrome_frame_delay_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_refresh_driver_chrome_frame_delay_ms,
        (1, 10000, 50)
      ),
      (
        'fx_refresh_driver_content_frame_delay_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_refresh_driver_content_frame_delay_ms,
        (1, 10000, 50)
      ),
      (
        'fx_refresh_driver_sync_scroll_frame_delay_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_refresh_driver_sync_scroll_frame_delay_ms,
        (1, 10000, 50)
      ),
      (
        'fx_sanitize_cache',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_cache,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_cookies_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_cookies_2,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_downloads',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_downloads,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_formdata',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_formdata,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_history',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_history,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_openwindows',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_openwindows,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_sessions',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_sessions,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_sitesettings',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_sitesettings,
        (1, 30000, 20)
      ),
      (
        'fx_sanitize_total',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_sanitize_total,
        (1, 30000, 20)
      ),
      (
        'fx_searchbar_selected_result_method',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.fx_searchbar_selected_result_method,
        (1, 50, 51)
      ),
      (
        'fx_session_restore_all_files_corrupt',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fx_session_restore_all_files_corrupt,
        (1, 2, 3)
      ),
      (
        'fx_session_restore_auto_restore_duration_until_eager_tabs_restored_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_auto_restore_duration_until_eager_tabs_restored_ms,
        (100, 100000, 20)
      ),
      (
        'fx_session_restore_collect_all_windows_data_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_collect_all_windows_data_ms,
        (1, 30000, 10)
      ),
      (
        'fx_session_restore_collect_data_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_collect_data_ms,
        (1, 30000, 10)
      ),
      (
        'fx_session_restore_corrupt_file',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fx_session_restore_corrupt_file,
        (1, 2, 3)
      ),
      (
        'fx_session_restore_dom_storage_size_estimate_chars',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_dom_storage_size_estimate_chars,
        (1, 30000000, 20)
      ),
      (
        'fx_session_restore_file_size_bytes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_file_size_bytes,
        (1, 50000000, 30)
      ),
      (
        'fx_session_restore_manual_restore_duration_until_eager_tabs_restored_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_manual_restore_duration_until_eager_tabs_restored_ms,
        (100, 100000, 20)
      ),
      (
        'fx_session_restore_number_of_eager_tabs_restored',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_session_restore_number_of_eager_tabs_restored,
        (1, 50, 51)
      ),
      (
        'fx_session_restore_number_of_tabs_restored',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_number_of_tabs_restored,
        (1, 500, 20)
      ),
      (
        'fx_session_restore_number_of_windows_restored',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_session_restore_number_of_windows_restored,
        (1, 50, 51)
      ),
      (
        'fx_session_restore_privacy_level',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_session_restore_privacy_level,
        (1, 3, 4)
      ),
      (
        'fx_session_restore_read_file_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_read_file_ms,
        (1, 3000, 10)
      ),
      (
        'fx_session_restore_restore_window_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_restore_window_ms,
        (1, 3000, 10)
      ),
      (
        'fx_session_restore_send_update_caused_oom',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fx_session_restore_send_update_caused_oom,
        (1, 2, 3)
      ),
      (
        'fx_session_restore_serialize_data_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_serialize_data_ms,
        (1, 1000, 10)
      ),
      (
        'fx_session_restore_startup_init_session_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_startup_init_session_ms,
        (1, 30000, 20)
      ),
      (
        'fx_session_restore_startup_onload_initial_window_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_startup_onload_initial_window_ms,
        (1, 30000, 20)
      ),
      (
        'fx_session_restore_write_file_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_session_restore_write_file_ms,
        (1, 3000, 10)
      ),
      (
        'fx_startup_migration_automated_import_process_success',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_startup_migration_automated_import_process_success,
        (1, 27, 28)
      ),
      (
        'fx_startup_migration_automated_import_undo',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_startup_migration_automated_import_undo,
        (1, 31, 32)
      ),
      (
        'fx_startup_migration_browser_count',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_startup_migration_browser_count,
        (1, 15, 16)
      ),
      (
        'fx_startup_migration_existing_default_browser',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_startup_migration_existing_default_browser,
        (1, 15, 16)
      ),
      (
        'fx_startup_migration_undo_offered',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_startup_migration_undo_offered,
        (1, 5, 6)
      ),
      (
        'fx_tab_click_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_click_ms,
        (1, 1000, 20)
      ),
      (
        'fx_tab_close_permit_unload_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_close_permit_unload_time_ms,
        (1, 10000, 50)
      ),
      (
        'fx_tab_close_time_anim_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_close_time_anim_ms,
        (1, 10000, 50)
      ),
      (
        'fx_tab_close_time_no_anim_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_close_time_no_anim_ms,
        (1, 10000, 50)
      ),
      (
        'fx_tab_switch_spinner_visible_long_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_switch_spinner_visible_long_ms,
        (1000, 64000, 7)
      ),
      (
        'fx_tab_switch_spinner_visible_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_switch_spinner_visible_ms,
        (1, 1000, 20)
      ),
      (
        'fx_tab_switch_total_e10s_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_switch_total_e10s_ms,
        (1, 1000, 20)
      ),
      (
        'fx_tab_switch_total_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_switch_total_ms,
        (1, 1000, 20)
      ),
      (
        'fx_tab_switch_update_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_tab_switch_update_ms,
        (1, 1000, 20)
      ),
      (
        'fx_thumbnails_bg_capture_canvas_draw_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_bg_capture_canvas_draw_time_ms,
        (1, 500, 15)
      ),
      (
        'fx_thumbnails_bg_capture_done_reason_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_thumbnails_bg_capture_done_reason_2,
        (1, 10, 11)
      ),
      (
        'fx_thumbnails_bg_capture_page_load_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_bg_capture_page_load_time_ms,
        (1, 60000, 20)
      ),
      (
        'fx_thumbnails_bg_capture_queue_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_bg_capture_queue_time_ms,
        (1, 300000, 20)
      ),
      (
        'fx_thumbnails_bg_capture_service_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_bg_capture_service_time_ms,
        (1, 30000, 20)
      ),
      (
        'fx_thumbnails_bg_queue_size_on_capture',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_bg_queue_size_on_capture,
        (1, 100, 15)
      ),
      (
        'fx_thumbnails_capture_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_capture_time_ms,
        (1, 500, 15)
      ),
      (
        'fx_thumbnails_hit_or_miss',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fx_thumbnails_hit_or_miss,
        (1, 2, 3)
      ),
      (
        'fx_thumbnails_store_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.fx_thumbnails_store_time_ms,
        (1, 500, 15)
      ),
      (
        'fx_total_top_visits',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.fx_total_top_visits,
        (1, 2, 3)
      ),
      (
        'fx_touch_used',
        'histogram-count',
        'content',
        payload.processes.content.histograms.fx_touch_used,
        (1, 2, 3)
      ),
      (
        'fx_urlbar_selected_result_index',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_urlbar_selected_result_index,
        (1, 17, 18)
      ),
      (
        'fx_urlbar_selected_result_method',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.fx_urlbar_selected_result_method,
        (1, 50, 51)
      ),
      (
        'fx_urlbar_selected_result_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_urlbar_selected_result_type,
        (1, 14, 15)
      ),
      (
        'fx_urlbar_selected_result_type_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.fx_urlbar_selected_result_type_2,
        (1, 32, 33)
      ),
      (
        'fxa_configured',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.fxa_configured,
        (1, 2, 3)
      ),
      (
        'gc_animation_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_animation_ms,
        (1, 10000, 50)
      ),
      (
        'gc_budget_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_budget_ms,
        (1, 100, 10)
      ),
      (
        'gc_budget_ms_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_budget_ms_2,
        (1, 200, 50)
      ),
      (
        'gc_budget_overrun',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_budget_overrun,
        (1, 100000, 100)
      ),
      (
        'gc_compact_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_compact_ms,
        (1, 10000, 50)
      ),
      (
        'gc_effectiveness',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_effectiveness,
        (1, 50000, 100)
      ),
      (
        'gc_in_progress_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_in_progress_ms,
        (1, 10000, 50)
      ),
      (
        'gc_incremental_disabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.gc_incremental_disabled,
        (1, 2, 3)
      ),
      (
        'gc_is_compartmental',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.gc_is_compartmental,
        (1, 2, 3)
      ),
      (
        'gc_mark_gray_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_mark_gray_ms,
        (1, 200, 50)
      ),
      (
        'gc_mark_gray_ms_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_mark_gray_ms_2,
        (1, 10000, 50)
      ),
      (
        'gc_mark_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_mark_ms,
        (1, 10000, 50)
      ),
      (
        'gc_mark_rate',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_mark_rate,
        (1000, 1000000, 100)
      ),
      (
        'gc_mark_rate_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_mark_rate_2,
        (1000, 300000, 50)
      ),
      (
        'gc_mark_roots_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_mark_roots_ms,
        (1, 200, 50)
      ),
      (
        'gc_mark_roots_us',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_mark_roots_us,
        (1, 150000, 50)
      ),
      (
        'gc_mark_weak_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_mark_weak_ms,
        (1, 10000, 50)
      ),
      (
        'gc_max_pause_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_max_pause_ms,
        (1, 1000, 50)
      ),
      (
        'gc_max_pause_ms_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_max_pause_ms_2,
        (1, 10000, 50)
      ),
      (
        'gc_minor_reason',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_minor_reason,
        (1, 100, 101)
      ),
      (
        'gc_minor_reason_long',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_minor_reason_long,
        (1, 100, 101)
      ),
      (
        'gc_minor_us',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_minor_us,
        (1, 1000000, 100)
      ),
      (
        'gc_mmu_50',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_mmu_50,
        (1, 100, 20)
      ),
      (
        'gc_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_ms,
        (1, 10000, 50)
      ),
      (
        'gc_non_incremental',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.gc_non_incremental,
        (1, 2, 3)
      ),
      (
        'gc_non_incremental_reason',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_non_incremental_reason,
        (1, 20, 21)
      ),
      (
        'gc_nursery_bytes',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_nursery_bytes,
        (1, 16777216, 16)
      ),
      (
        'gc_nursery_bytes_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_nursery_bytes_2,
        (16384, 16777216, 24)
      ),
      (
        'gc_nursery_promotion_rate',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_nursery_promotion_rate,
        (1, 100, 50)
      ),
      (
        'gc_prepare_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_prepare_ms,
        (1, 1000, 50)
      ),
      (
        'gc_pretenure_count',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_pretenure_count,
        (1, 32, 33)
      ),
      (
        'gc_pretenure_count_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_pretenure_count_2,
        (1, 100, 20)
      ),
      (
        'gc_reason_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_reason_2,
        (1, 100, 101)
      ),
      (
        'gc_reset',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.gc_reset,
        (1, 2, 3)
      ),
      (
        'gc_reset_reason',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_reset_reason,
        (1, 20, 21)
      ),
      (
        'gc_scc_sweep_max_pause_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_scc_sweep_max_pause_ms,
        (1, 500, 50)
      ),
      (
        'gc_scc_sweep_total_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_scc_sweep_total_ms,
        (1, 500, 50)
      ),
      (
        'gc_slice_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_slice_count,
        (1, 200, 50)
      ),
      (
        'gc_slice_during_idle',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_slice_during_idle,
        (1, 100, 50)
      ),
      (
        'gc_slice_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_slice_ms,
        (1, 10000, 50)
      ),
      (
        'gc_slow_phase',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_slow_phase,
        (1, 75, 76)
      ),
      (
        'gc_slow_task',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gc_slow_task,
        (1, 75, 76)
      ),
      (
        'gc_sweep_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_sweep_ms,
        (1, 10000, 50)
      ),
      (
        'gc_tenured_survival_rate',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gc_tenured_survival_rate,
        (1, 100, 50)
      ),
      (
        'gc_time_between_s',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_time_between_s,
        (1, 120, 50)
      ),
      (
        'gc_time_between_slices_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gc_time_between_slices_ms,
        (1, 2000, 50)
      ),
      (
        'gdi_initfontlist_total',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gdi_initfontlist_total,
        (1, 30000, 10)
      ),
      (
        'geolocation_accuracy_exponential',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.geolocation_accuracy_exponential,
        (1, 100000, 50)
      ),
      (
        'geolocation_error',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.geolocation_error,
        (1, 2, 3)
      ),
      (
        'geolocation_getcurrentposition_secure_origin',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.geolocation_getcurrentposition_secure_origin,
        (1, 10, 11)
      ),
      (
        'geolocation_getcurrentposition_visible',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.geolocation_getcurrentposition_visible,
        (1, 2, 3)
      ),
      (
        'geolocation_osx_source_is_mls',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.geolocation_osx_source_is_mls,
        (1, 2, 3)
      ),
      (
        'geolocation_request_granted',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.geolocation_request_granted,
        (1, 20, 21)
      ),
      (
        'geolocation_watchposition_secure_origin',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.geolocation_watchposition_secure_origin,
        (1, 10, 11)
      ),
      (
        'geolocation_watchposition_visible',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.geolocation_watchposition_visible,
        (1, 2, 3)
      ),
      (
        'geolocation_win8_source_is_mls',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.geolocation_win8_source_is_mls,
        (1, 2, 3)
      ),
      (
        'gfx_content_failed_to_acquire_device',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gfx_content_failed_to_acquire_device,
        (1, 6, 7)
      ),
      (
        'gfx_crash',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gfx_crash,
        (1, 100, 101)
      ),
      (
        'gfx_omtp_paint_task_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gfx_omtp_paint_task_count,
        (1, 100, 25)
      ),
      (
        'gfx_omtp_paint_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gfx_omtp_paint_time,
        (1, 10000, 50)
      ),
      (
        'gfx_omtp_paint_wait_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gfx_omtp_paint_wait_time,
        (1, 200, 50)
      ),
      (
        'ghost_windows',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ghost_windows,
        (1, 128, 32)
      ),
      (
        'gpu_process_crash_fallbacks',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.gpu_process_crash_fallbacks,
        (1, 10, 11)
      ),
      (
        'gpu_process_initialization_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gpu_process_initialization_time_ms,
        (1, 64000, 100)
      ),
      (
        'gpu_process_launch_time_ms_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gpu_process_launch_time_ms_2,
        (1, 64000, 100)
      ),
      (
        'gradient_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gradient_duration,
        (1, 50000000, 20)
      ),
      (
        'gradient_retention_time',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.gradient_retention_time,
        (1, 10000, 20)
      ),
      (
        'graphics_driver_startup_test',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.graphics_driver_startup_test,
        (1, 20, 21)
      ),
      (
        'graphics_sanity_test',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.graphics_sanity_test,
        (1, 20, 21)
      ),
      (
        'graphics_sanity_test_os_snapshot',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.graphics_sanity_test_os_snapshot,
        (1, 10, 11)
      ),
      (
        'graphics_sanity_test_reason',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.graphics_sanity_test_reason,
        (1, 20, 21)
      ),
      (
        'gv_page_load_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gv_page_load_ms,
        (1, 100000, 100)
      ),
      (
        'gv_page_load_progress_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gv_page_load_progress_ms,
        (1, 100000, 100)
      ),
      (
        'gv_page_reload_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gv_page_reload_ms,
        (1, 100000, 100)
      ),
      (
        'gv_startup_runtime_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.gv_startup_runtime_ms,
        (1, 10000, 50)
      ),
      (
        'hidden_viewport_overflow_type',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.hidden_viewport_overflow_type,
        (1, 50, 51)
      ),
      (
        'history_lastvisited_tree_query_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.history_lastvisited_tree_query_time_ms,
        (50, 2000, 30)
      ),
      (
        'hpack_bytes_evicted_compressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_bytes_evicted_compressor,
        (1, 8192, 50)
      ),
      (
        'hpack_bytes_evicted_decompressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_bytes_evicted_decompressor,
        (1, 8192, 50)
      ),
      (
        'hpack_bytes_evicted_ratio_compressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_bytes_evicted_ratio_compressor,
        (1, 256, 50)
      ),
      (
        'hpack_bytes_evicted_ratio_decompressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_bytes_evicted_ratio_decompressor,
        (1, 256, 50)
      ),
      (
        'hpack_elements_evicted_compressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_elements_evicted_compressor,
        (1, 256, 50)
      ),
      (
        'hpack_elements_evicted_decompressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_elements_evicted_decompressor,
        (1, 256, 50)
      ),
      (
        'hpack_peak_count_compressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_peak_count_compressor,
        (1, 1024, 50)
      ),
      (
        'hpack_peak_count_decompressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_peak_count_decompressor,
        (1, 1024, 50)
      ),
      (
        'hpack_peak_size_compressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_peak_size_compressor,
        (1, 16384, 100)
      ),
      (
        'hpack_peak_size_decompressor',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hpack_peak_size_decompressor,
        (1, 16384, 100)
      ),
      (
        'htmleditors_whose_absolute_positioner_used_by_user',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.htmleditors_whose_absolute_positioner_used_by_user,
        (1, 50, 20)
      ),
      (
        'htmleditors_whose_inline_table_editor_used_by_user',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.htmleditors_whose_inline_table_editor_used_by_user,
        (1, 50, 20)
      ),
      (
        'htmleditors_whose_resizers_used_by_user',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.htmleditors_whose_resizers_used_by_user,
        (1, 50, 20)
      ),
      (
        'htmleditors_with_absolute_positioner',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.htmleditors_with_absolute_positioner,
        (1, 2, 3)
      ),
      (
        'htmleditors_with_inline_table_editor',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.htmleditors_with_inline_table_editor,
        (1, 2, 3)
      ),
      (
        'htmleditors_with_resizers',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.htmleditors_with_resizers,
        (1, 2, 3)
      ),
      (
        'http_09_info',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_09_info,
        (1, 4, 5)
      ),
      (
        'http_altsvc_entries_per_header',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_altsvc_entries_per_header,
        (1, 5, 6)
      ),
      (
        'http_altsvc_mapping_changed_target',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_altsvc_mapping_changed_target,
        (1, 2, 3)
      ),
      (
        'http_auth_dialog_stats_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_auth_dialog_stats_2,
        (1, 32, 33)
      ),
      (
        'http_auth_dialog_stats_3',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_auth_dialog_stats_3,
        (1, 64, 65)
      ),
      (
        'http_auth_type_stats',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_auth_type_stats,
        (1, 8, 9)
      ),
      (
        'http_cache_disposition_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_disposition_2,
        (1, 5, 6)
      ),
      (
        'http_cache_disposition_2_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_disposition_2_v2,
        (1, 5, 6)
      ),
      (
        'http_cache_entry_alive_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_cache_entry_alive_time,
        (1, 7200000, 50)
      ),
      (
        'http_cache_entry_reload_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_cache_entry_reload_time,
        (1, 900000, 50)
      ),
      (
        'http_cache_entry_reuse_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.http_cache_entry_reuse_count,
        (1, 20, 19)
      ),
      (
        'http_cache_io_queue_2_evict',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_evict,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_index',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_index,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_management',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_management,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_open',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_open,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_open_priority',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_open_priority,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_read',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_read,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_read_priority',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_read_priority,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_write',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_write,
        (1, 10, 11)
      ),
      (
        'http_cache_io_queue_2_write_priority',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_io_queue_2_write_priority,
        (1, 10, 11)
      ),
      (
        'http_cache_miss_halflife_experiment_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_cache_miss_halflife_experiment_2,
        (1, 4, 5)
      ),
      (
        'http_channel_disposition',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_channel_disposition,
        (1, 16, 17)
      ),
      (
        'http_connection_entry_cache_hit_1',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_connection_entry_cache_hit_1,
        (1, 2, 3)
      ),
      (
        'http_content_encoding',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_content_encoding,
        (1, 6, 7)
      ),
      (
        'http_disk_cache_overhead',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_disk_cache_overhead,
        (1, 32000000, 100)
      ),
      (
        'http_kbread_per_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_kbread_per_conn,
        (1, 3000, 50)
      ),
      (
        'http_kbread_per_conn2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_kbread_per_conn2,
        (1, 100000, 50)
      ),
      (
        'http_net_vs_cache_onstart_notrevalidated_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_notrevalidated_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_qbig_highpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_qbig_highpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_qbig_normalpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_qbig_normalpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_qmed_highpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_qmed_highpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_qmed_normalpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_qmed_normalpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_qsmall_highpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_qsmall_highpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_qsmall_normalpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_qsmall_normalpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstart_revalidated_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstart_revalidated_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_large_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_large_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_notrevalidated_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_notrevalidated_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_qbig_highpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_qbig_highpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_qbig_normalpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_qbig_normalpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_qmed_highpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_qmed_highpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_qmed_normalpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_qmed_normalpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_qsmall_highpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_qsmall_highpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_qsmall_normalpri_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_qsmall_normalpri_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_revalidated_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_revalidated_v2,
        (1, 80, 81)
      ),
      (
        'http_net_vs_cache_onstop_small_v2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_net_vs_cache_onstop_small_v2,
        (1, 80, 81)
      ),
      (
        'http_offline_cache_document_load',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_offline_cache_document_load,
        (1, 2, 3)
      ),
      (
        'http_onstart_suspend_total_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_onstart_suspend_total_time,
        (1, 60000, 100)
      ),
      (
        'http_page_cache_read_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_cache_read_time,
        (1, 30000, 50)
      ),
      (
        'http_page_cache_read_time_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_cache_read_time_v2,
        (1, 30000, 50)
      ),
      (
        'http_page_complete_load',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_complete_load,
        (1, 30000, 50)
      ),
      (
        'http_page_complete_load_cached',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_complete_load_cached,
        (1, 30000, 50)
      ),
      (
        'http_page_complete_load_cached_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_complete_load_cached_v2,
        (1, 30000, 50)
      ),
      (
        'http_page_complete_load_net',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_complete_load_net,
        (1, 30000, 50)
      ),
      (
        'http_page_complete_load_net_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_complete_load_net_v2,
        (1, 30000, 50)
      ),
      (
        'http_page_complete_load_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_complete_load_v2,
        (1, 30000, 50)
      ),
      (
        'http_page_dns_issue_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_dns_issue_time,
        (1, 30000, 50)
      ),
      (
        'http_page_dns_lookup_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_dns_lookup_time,
        (1, 30000, 50)
      ),
      (
        'http_page_first_sent_to_last_received',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_first_sent_to_last_received,
        (1, 30000, 50)
      ),
      (
        'http_page_open_to_first_from_cache',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_open_to_first_from_cache,
        (1, 30000, 50)
      ),
      (
        'http_page_open_to_first_from_cache_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_open_to_first_from_cache_v2,
        (1, 30000, 50)
      ),
      (
        'http_page_open_to_first_received',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_open_to_first_received,
        (1, 30000, 50)
      ),
      (
        'http_page_open_to_first_sent',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_open_to_first_sent,
        (1, 30000, 50)
      ),
      (
        'http_page_revalidation',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_revalidation,
        (1, 30000, 50)
      ),
      (
        'http_page_tcp_connection',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_tcp_connection,
        (1, 30000, 50)
      ),
      (
        'http_page_tcp_connection_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_tcp_connection_2,
        (1, 30000, 50)
      ),
      (
        'http_page_tls_handshake',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_page_tls_handshake,
        (1, 30000, 50)
      ),
      (
        'http_pageload_is_ssl',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_pageload_is_ssl,
        (1, 2, 3)
      ),
      (
        'http_proxy_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_proxy_type,
        (1, 8, 9)
      ),
      (
        'http_request_per_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_request_per_conn,
        (1, 1000, 50)
      ),
      (
        'http_request_per_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_request_per_page,
        (1, 1000, 50)
      ),
      (
        'http_request_per_page_from_cache',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_request_per_page_from_cache,
        (1, 101, 102)
      ),
      (
        'http_response_status_code',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_response_status_code,
        (1, 12, 13)
      ),
      (
        'http_response_version',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_response_version,
        (1, 48, 49)
      ),
      (
        'http_saw_quic_alt_protocol',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_saw_quic_alt_protocol,
        (1, 2, 3)
      ),
      (
        'http_saw_quic_alt_protocol_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_saw_quic_alt_protocol_2,
        (1, 3, 4)
      ),
      (
        'http_scheme_upgrade',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.http_scheme_upgrade,
        (1, 10, 11)
      ),
      (
        'http_scheme_upgrade_type',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.http_scheme_upgrade_type,
        (1, 50, 51)
      ),
      (
        'http_sub_cache_read_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_cache_read_time,
        (1, 30000, 50)
      ),
      (
        'http_sub_cache_read_time_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_cache_read_time_v2,
        (1, 30000, 50)
      ),
      (
        'http_sub_complete_load',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_complete_load,
        (1, 30000, 50)
      ),
      (
        'http_sub_complete_load_cached',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_complete_load_cached,
        (1, 30000, 50)
      ),
      (
        'http_sub_complete_load_cached_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_complete_load_cached_v2,
        (1, 30000, 50)
      ),
      (
        'http_sub_complete_load_net',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_complete_load_net,
        (1, 30000, 50)
      ),
      (
        'http_sub_complete_load_net_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_complete_load_net_v2,
        (1, 30000, 50)
      ),
      (
        'http_sub_complete_load_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_complete_load_v2,
        (1, 30000, 50)
      ),
      (
        'http_sub_dns_issue_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_dns_issue_time,
        (1, 30000, 50)
      ),
      (
        'http_sub_dns_lookup_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_dns_lookup_time,
        (1, 30000, 50)
      ),
      (
        'http_sub_first_sent_to_last_received',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_first_sent_to_last_received,
        (1, 30000, 50)
      ),
      (
        'http_sub_open_to_first_from_cache',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_open_to_first_from_cache,
        (1, 30000, 50)
      ),
      (
        'http_sub_open_to_first_from_cache_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_open_to_first_from_cache_v2,
        (1, 30000, 50)
      ),
      (
        'http_sub_open_to_first_received',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_open_to_first_received,
        (1, 30000, 50)
      ),
      (
        'http_sub_open_to_first_sent',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_open_to_first_sent,
        (1, 30000, 50)
      ),
      (
        'http_sub_revalidation',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_revalidation,
        (1, 30000, 50)
      ),
      (
        'http_sub_tcp_connection',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_tcp_connection,
        (1, 30000, 50)
      ),
      (
        'http_sub_tcp_connection_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_tcp_connection_2,
        (1, 30000, 50)
      ),
      (
        'http_sub_tls_handshake',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_sub_tls_handshake,
        (1, 30000, 50)
      ),
      (
        'http_subitem_first_byte_latency_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_subitem_first_byte_latency_time,
        (1, 30000, 50)
      ),
      (
        'http_subitem_open_latency_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.http_subitem_open_latency_time,
        (1, 30000, 50)
      ),
      (
        'http_transaction_is_ssl',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_transaction_is_ssl,
        (1, 2, 3)
      ),
      (
        'http_transaction_use_altsvc',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_transaction_use_altsvc,
        (1, 2, 3)
      ),
      (
        'http_transaction_use_altsvc_oe',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.http_transaction_use_altsvc_oe,
        (1, 2, 3)
      ),
      (
        'httpconnmgr_total_speculative_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.httpconnmgr_total_speculative_conn,
        (1, 1000000, 50)
      ),
      (
        'httpconnmgr_unused_speculative_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.httpconnmgr_unused_speculative_conn,
        (1, 1000000, 50)
      ),
      (
        'httpconnmgr_used_speculative_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.httpconnmgr_used_speculative_conn,
        (1, 1000000, 50)
      ),
      (
        'hyphenation_load_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hyphenation_load_time,
        (1, 1000, 100)
      ),
      (
        'hyphenation_memory',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.hyphenation_memory,
        (1, 8000, 100)
      ),
      (
        'idb_custom_open_with_options_count',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.idb_custom_open_with_options_count,
        (1, 50, 51)
      ),
      (
        'idle_notify_idle_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.idle_notify_idle_ms,
        (1, 5000, 10)
      ),
      (
        'image_animated_decode_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_animated_decode_count,
        (1, 500, 50)
      ),
      (
        'image_animated_decode_on_draw_latency',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_animated_decode_on_draw_latency,
        (50, 50000000, 100)
      ),
      (
        'image_animated_decode_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_animated_decode_time,
        (50, 50000000, 100)
      ),
      (
        'image_decode_chunks',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_chunks,
        (1, 500, 50)
      ),
      (
        'image_decode_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_count,
        (1, 500, 50)
      ),
      (
        'image_decode_latency_us',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_latency_us,
        (50, 5000000, 100)
      ),
      (
        'image_decode_on_draw_latency',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_on_draw_latency,
        (50, 50000000, 100)
      ),
      (
        'image_decode_speed_gif',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_speed_gif,
        (500, 50000000, 50)
      ),
      (
        'image_decode_speed_jpeg',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_speed_jpeg,
        (500, 50000000, 50)
      ),
      (
        'image_decode_speed_png',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_speed_png,
        (500, 50000000, 50)
      ),
      (
        'image_decode_speed_webp',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_speed_webp,
        (500, 50000000, 50)
      ),
      (
        'image_decode_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.image_decode_time,
        (50, 50000000, 100)
      ),
      (
        'image_request_dispatched',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.image_request_dispatched,
        (1, 2, 3)
      ),
      (
        'innerwindows_with_mutation_listeners',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.innerwindows_with_mutation_listeners,
        (1, 2, 3)
      ),
      (
        'innerwindows_with_text_event_listeners',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.innerwindows_with_text_event_listeners,
        (1, 2, 3)
      ),
      (
        'input_event_handled_apz_mouse_move_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_handled_apz_mouse_move_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_handled_apz_touch_move_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_handled_apz_touch_move_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_handled_apz_wheel_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_handled_apz_wheel_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_handled_keyboard_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_handled_keyboard_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_handled_mouse_down_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_handled_mouse_down_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_handled_mouse_up_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_handled_mouse_up_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_queued_apz_mouse_move_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_queued_apz_mouse_move_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_queued_apz_touch_move_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_queued_apz_touch_move_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_queued_apz_wheel_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_queued_apz_wheel_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_queued_click_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_queued_click_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_queued_keyboard_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_queued_keyboard_ms,
        (1, 5000, 100)
      ),
      (
        'input_event_response_coalesced_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_response_coalesced_ms,
        (1, 10000, 50)
      ),
      (
        'input_event_response_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_response_ms,
        (1, 10000, 50)
      ),
      (
        'input_event_response_post_startup_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_response_post_startup_ms,
        (1, 10000, 50)
      ),
      (
        'input_event_response_startup_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.input_event_response_startup_ms,
        (1, 10000, 50)
      ),
      (
        'ipc_message_size2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ipc_message_size2,
        (1, 8000000, 50)
      ),
      (
        'ipc_same_process_message_copy_oom_kb',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ipc_same_process_message_copy_oom_kb,
        (100, 10000000, 10)
      ),
      (
        'ipc_transaction_cancel',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.ipc_transaction_cancel,
        (1, 2, 3)
      ),
      (
        'ipv4_and_ipv6_address_connectivity',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ipv4_and_ipv6_address_connectivity,
        (1, 4, 5)
      ),
      (
        'js_aot_usage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.js_aot_usage,
        (1, 4, 5)
      ),
      (
        'js_deprecated_array_generics',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.js_deprecated_array_generics,
        (1, 20, 21)
      ),
      (
        'js_deprecated_language_extensions_in_addons',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.js_deprecated_language_extensions_in_addons,
        (1, 10, 11)
      ),
      (
        'js_deprecated_language_extensions_in_content',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.js_deprecated_language_extensions_in_content,
        (1, 10, 11)
      ),
      (
        'js_deprecated_string_generics',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.js_deprecated_string_generics,
        (1, 25, 26)
      ),
      (
        'js_privileged_parser_compile_lazy_after_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.js_privileged_parser_compile_lazy_after_ms,
        (10, 10000, 10)
      ),
      (
        'js_web_parser_compile_lazy_after_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.js_web_parser_compile_lazy_after_ms,
        (10, 10000, 10)
      ),
      (
        'keypress_present_latency',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.keypress_present_latency,
        (1, 200000, 50)
      ),
      (
        'langpack_font_families_per_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.langpack_font_families_per_page,
        (1, 256, 64)
      ),
      (
        'link_icon_sizes_attr_dimension',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.link_icon_sizes_attr_dimension,
        (1, 513, 64)
      ),
      (
        'link_icon_sizes_attr_usage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.link_icon_sizes_attr_usage,
        (1, 4, 5)
      ),
      (
        'load_input_event_response_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.load_input_event_response_ms,
        (1, 10000, 50)
      ),
      (
        'localdomstorage_clear_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_clear_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_getallkeys_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_getallkeys_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_getkey_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_getkey_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_getlength_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_getlength_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_getvalue_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_getvalue_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_preload_pending_on_first_access',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.localdomstorage_preload_pending_on_first_access,
        (1, 2, 3)
      ),
      (
        'localdomstorage_removekey_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_removekey_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_sessiononly_preload_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_sessiononly_preload_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_setvalue_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_setvalue_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_shutdown_database_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_shutdown_database_ms,
        (1, 3000, 10)
      ),
      (
        'localdomstorage_unload_blocking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.localdomstorage_unload_blocking_ms,
        (1, 3000, 10)
      ),
      (
        'login_reputation_login_whitelist_lookup_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.login_reputation_login_whitelist_lookup_time,
        (1, 5000, 30)
      ),
      (
        'login_reputation_login_whitelist_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.login_reputation_login_whitelist_result,
        (1, 3, 4)
      ),
      (
        'long_reflow_interruptible',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.long_reflow_interruptible,
        (1, 2, 3)
      ),
      (
        'low_memory_events_commit_space',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.low_memory_events_commit_space,
        (1, 1024, 21)
      ),
      (
        'low_memory_events_physical',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.low_memory_events_physical,
        (1, 1024, 21)
      ),
      (
        'low_memory_events_virtual',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.low_memory_events_virtual,
        (1, 1024, 21)
      ),
      (
        'mac_initfontlist_total',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.mac_initfontlist_total,
        (1, 30000, 10)
      ),
      (
        'master_password_enabled',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.master_password_enabled,
        (1, 2, 3)
      ),
      (
        'media_android_video_tunneling_support',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_android_video_tunneling_support,
        (1, 2, 3)
      ),
      (
        'media_decoder_backend_used',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.media_decoder_backend_used,
        (1, 10, 11)
      ),
      (
        'media_decoding_process_crash',
        'histogram-count',
        'content',
        payload.processes.content.histograms.media_decoding_process_crash,
        (1, 2, 3)
      ),
      (
        'media_eme_request_deprecated_warnings',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.media_eme_request_deprecated_warnings,
        (1, 10, 11)
      ),
      (
        'media_eme_secure_context',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_eme_secure_context,
        (1, 2, 3)
      ),
      (
        'media_hls_canplay_requested',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_hls_canplay_requested,
        (1, 2, 3)
      ),
      (
        'media_hls_canplay_supported',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_hls_canplay_supported,
        (1, 2, 3)
      ),
      (
        'media_hls_decoder_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_hls_decoder_success,
        (1, 2, 3)
      ),
      (
        'media_mkv_canplay_requested',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_mkv_canplay_requested,
        (1, 2, 3)
      ),
      (
        'media_mp4_parse_num_sample_description_entries',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.media_mp4_parse_num_sample_description_entries,
        (1, 5, 6)
      ),
      (
        'media_mp4_parse_sample_description_entries_have_multiple_codecs',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_mp4_parse_sample_description_entries_have_multiple_codecs,
        (1, 2, 3)
      ),
      (
        'media_mp4_parse_sample_description_entries_have_multiple_crypto',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_mp4_parse_sample_description_entries_have_multiple_crypto,
        (1, 2, 3)
      ),
      (
        'media_ogg_loaded_is_chained',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_ogg_loaded_is_chained,
        (1, 2, 3)
      ),
      (
        'media_play_promise_resolution',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.media_play_promise_resolution,
        (1, 50, 51)
      ),
      (
        'media_played_time_after_autoplay_blocked',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.media_played_time_after_autoplay_blocked,
        (1, 50, 51)
      ),
      (
        'media_recorder_recording_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.media_recorder_recording_duration,
        (1, 3600, 100)
      ),
      (
        'media_recorder_track_encoder_init_timeout_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.media_recorder_track_encoder_init_timeout_type,
        (1, 4, 5)
      ),
      (
        'media_rust_mp4parse_error_code',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.media_rust_mp4parse_error_code,
        (1, 32, 33)
      ),
      (
        'media_rust_mp4parse_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_rust_mp4parse_success,
        (1, 2, 3)
      ),
      (
        'media_rust_mp4parse_track_match_audio',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_rust_mp4parse_track_match_audio,
        (1, 2, 3)
      ),
      (
        'media_rust_mp4parse_track_match_video',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.media_rust_mp4parse_track_match_video,
        (1, 2, 3)
      ),
      (
        'media_wmf_decode_error',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.media_wmf_decode_error,
        (1, 256, 257)
      ),
      (
        'mediacache_blockowners_watermark',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mediacache_blockowners_watermark,
        (1, 32, 33)
      ),
      (
        'mediacache_memory_watermark',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.mediacache_memory_watermark,
        (1, 520000000, 66)
      ),
      (
        'mediacache_watermark_kb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.mediacache_watermark_kb,
        (1, 520000, 66)
      ),
      (
        'mediacachestream_length_kb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.mediacachestream_length_kb,
        (1, 520000, 66)
      ),
      (
        'mediacachestream_notified_length',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.mediacachestream_notified_length,
        (1, 16777216, 66)
      ),
      (
        'memory_free_purged_pages_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_free_purged_pages_ms,
        (1, 1024, 10)
      ),
      (
        'memory_heap_allocated',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_heap_allocated,
        (1024, 16777216, 200)
      ),
      (
        'memory_heap_committed_unused',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_heap_committed_unused,
        (1024, 524288, 50)
      ),
      (
        'memory_heap_overhead_fraction',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.memory_heap_overhead_fraction,
        (1, 100, 25)
      ),
      (
        'memory_images_content_used_uncompressed',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_images_content_used_uncompressed,
        (1024, 1048576, 50)
      ),
      (
        'memory_js_compartments_system',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_js_compartments_system,
        (1, 1000, 50)
      ),
      (
        'memory_js_compartments_user',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_js_compartments_user,
        (1, 1000, 50)
      ),
      (
        'memory_js_gc_heap',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_js_gc_heap,
        (1024, 16777216, 200)
      ),
      (
        'memory_js_realms_system',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_js_realms_system,
        (1, 1000, 50)
      ),
      (
        'memory_js_realms_user',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_js_realms_user,
        (1, 1000, 50)
      ),
      (
        'memory_resident_fast',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_resident_fast,
        (32768, 16777216, 100)
      ),
      (
        'memory_resident_peak',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_resident_peak,
        (32768, 16777216, 100)
      ),
      (
        'memory_storage_sqlite',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_storage_sqlite,
        (1024, 524288, 50)
      ),
      (
        'memory_total',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_total,
        (32768, 16777216, 100)
      ),
      (
        'memory_unique',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_unique,
        (32768, 16777216, 100)
      ),
      (
        'memory_unique_content_startup',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_unique_content_startup,
        (2048, 131072, 100)
      ),
      (
        'memory_vsize',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_vsize,
        (32768, 16777216, 100)
      ),
      (
        'memory_vsize_max_contiguous',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.memory_vsize_max_contiguous,
        (32768, 16777216, 100)
      ),
      (
        'memoryblockcache_errors',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.memoryblockcache_errors,
        (1, 16, 17)
      ),
      (
        'missing_font',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.missing_font,
        (1, 2, 3)
      ),
      (
        'missing_font_langpack',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.missing_font_langpack,
        (1, 2, 3)
      ),
      (
        'missing_font_user',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.missing_font_user,
        (1, 2, 3)
      ),
      (
        'mixed_content_hsts',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mixed_content_hsts,
        (1, 10, 11)
      ),
      (
        'mixed_content_hsts_priming',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mixed_content_hsts_priming,
        (1, 16, 17)
      ),
      (
        'mixed_content_hsts_priming_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mixed_content_hsts_priming_result,
        (1, 16, 17)
      ),
      (
        'mixed_content_object_subrequest',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mixed_content_object_subrequest,
        (1, 10, 11)
      ),
      (
        'mixed_content_page_load',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mixed_content_page_load,
        (1, 4, 5)
      ),
      (
        'mixed_content_unblock_counter',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.mixed_content_unblock_counter,
        (1, 3, 4)
      ),
      (
        'moz_blob_in_xhr',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.moz_blob_in_xhr,
        (1, 2, 3)
      ),
      (
        'moz_chunked_arraybuffer_in_xhr',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.moz_chunked_arraybuffer_in_xhr,
        (1, 2, 3)
      ),
      (
        'moz_chunked_text_in_xhr',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.moz_chunked_text_in_xhr,
        (1, 2, 3)
      ),
      (
        'moz_sqlite_cookies_open_readahead_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_open_readahead_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_cookies_read_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_read_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_cookies_read_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_read_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_cookies_read_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_read_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_cookies_sync_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_sync_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_cookies_sync_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_sync_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_cookies_write_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_write_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_cookies_write_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_write_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_cookies_write_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_cookies_write_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_open_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_open_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_open_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_open_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_other_read_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_read_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_other_read_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_read_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_other_read_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_read_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_other_sync_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_sync_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_other_sync_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_sync_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_other_write_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_write_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_other_write_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_write_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_other_write_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_other_write_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_places_read_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_read_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_places_read_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_read_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_places_read_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_read_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_places_sync_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_sync_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_places_sync_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_sync_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_places_write_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_write_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_places_write_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_write_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_places_write_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_places_write_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_truncate_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_truncate_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_truncate_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_truncate_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_webapps_read_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_read_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_webapps_read_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_read_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_webapps_read_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_read_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_webapps_sync_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_sync_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_webapps_sync_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_sync_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_webapps_write_b',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_write_b,
        (1, 32768, 3)
      ),
      (
        'moz_sqlite_webapps_write_main_thread_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_write_main_thread_ms,
        (1, 3000, 10)
      ),
      (
        'moz_sqlite_webapps_write_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_sqlite_webapps_write_ms,
        (1, 3000, 10)
      ),
      (
        'moz_storage_async_requests_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.moz_storage_async_requests_ms,
        (1, 32768, 20)
      ),
      (
        'moz_storage_async_requests_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.moz_storage_async_requests_success,
        (1, 2, 3)
      ),
      (
        'narrate_content_speaktime_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.narrate_content_speaktime_ms,
        (1, 300000, 30)
      ),
      (
        'network_cache_fs_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_cache_fs_type,
        (1, 5, 6)
      ),
      (
        'network_cache_hash_stats',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_cache_hash_stats,
        (1, 160, 161)
      ),
      (
        'network_cache_hit_miss_stat_per_cache_size',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_cache_hit_miss_stat_per_cache_size,
        (1, 40, 41)
      ),
      (
        'network_cache_hit_rate_per_cache_size',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_cache_hit_rate_per_cache_size,
        (1, 400, 401)
      ),
      (
        'network_cache_metadata_first_read_size',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.network_cache_metadata_first_read_size,
        (1, 5119, 256)
      ),
      (
        'network_cache_metadata_first_read_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_metadata_first_read_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_metadata_second_read_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_metadata_second_read_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_metadata_size',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.network_cache_metadata_size,
        (1, 5119, 256)
      ),
      (
        'network_cache_size_full_fat',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.network_cache_size_full_fat,
        (1, 500, 50)
      ),
      (
        'network_cache_v1_hit_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_v1_hit_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_v1_miss_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_v1_miss_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_v1_truncate_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_v1_truncate_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_v2_hit_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_v2_hit_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_v2_input_stream_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_cache_v2_input_stream_status,
        (1, 7, 8)
      ),
      (
        'network_cache_v2_miss_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_cache_v2_miss_time_ms,
        (1, 10000, 50)
      ),
      (
        'network_cache_v2_output_stream_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_cache_v2_output_stream_status,
        (1, 7, 8)
      ),
      (
        'network_connection_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.network_connection_count,
        (1, 2, 3)
      ),
      (
        'network_disk_cache2_shutdown_clear_private',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache2_shutdown_clear_private,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_deletedir',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_deletedir,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_deletedir_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_deletedir_shutdown,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_open',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_open,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_shutdown,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_shutdown_clear_private',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_shutdown_clear_private,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_shutdown_v2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_shutdown_v2,
        (1, 10000, 10)
      ),
      (
        'network_disk_cache_trashrename',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_disk_cache_trashrename,
        (1, 10000, 10)
      ),
      (
        'network_id',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_id,
        (1, 6, 7)
      ),
      (
        'network_probe_maxcount',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.network_probe_maxcount,
        (50, 1000, 10)
      ),
      (
        'network_race_cache_bandwidth_not_race',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_race_cache_bandwidth_not_race,
        (32, 16777216, 100)
      ),
      (
        'network_race_cache_bandwidth_race_cache_win',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_race_cache_bandwidth_race_cache_win,
        (32, 16777216, 100)
      ),
      (
        'network_race_cache_bandwidth_race_network_win',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_race_cache_bandwidth_race_network_win,
        (32, 16777216, 100)
      ),
      (
        'network_race_cache_with_network_ocec_on_start_diff',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.network_race_cache_with_network_ocec_on_start_diff,
        (1, 1000, 100)
      ),
      (
        'network_race_cache_with_network_saved_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.network_race_cache_with_network_saved_time,
        (1, 60000, 100)
      ),
      (
        'network_race_cache_with_network_usage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.network_race_cache_with_network_usage,
        (1, 4, 5)
      ),
      (
        'network_race_cache_with_network_usage_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.network_race_cache_with_network_usage_2,
        (1, 50, 51)
      ),
      (
        'network_session_at_900fd',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.network_session_at_900fd,
        (1, 2, 3)
      ),
      (
        'newtab_page_blocked_sites_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.newtab_page_blocked_sites_count,
        (1, 100, 10)
      ),
      (
        'newtab_page_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.newtab_page_enabled,
        (1, 2, 3)
      ),
      (
        'newtab_page_enhanced',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.newtab_page_enhanced,
        (1, 2, 3)
      ),
      (
        'newtab_page_life_span',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.newtab_page_life_span,
        (1, 1200, 100)
      ),
      (
        'newtab_page_life_span_suggested',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.newtab_page_life_span_suggested,
        (1, 1200, 100)
      ),
      (
        'newtab_page_pinned_sites_count',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.newtab_page_pinned_sites_count,
        (1, 9, 10)
      ),
      (
        'newtab_page_shown',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.newtab_page_shown,
        (1, 2, 3)
      ),
      (
        'newtab_page_site_clicked',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.newtab_page_site_clicked,
        (1, 10, 11)
      ),
      (
        'ntlm_module_used_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ntlm_module_used_2,
        (1, 8, 9)
      ),
      (
        'number_of_profiles',
        'histogram-count',
        'content',
        payload.processes.content.histograms.number_of_profiles,
        (1, 2, 3)
      ),
      (
        'onbeforeunload_prompt_action',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.onbeforeunload_prompt_action,
        (1, 3, 4)
      ),
      (
        'onbeforeunload_prompt_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.onbeforeunload_prompt_count,
        (1, 2, 3)
      ),
      (
        'osfile_worker_launch_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.osfile_worker_launch_ms,
        (1, 5000, 10)
      ),
      (
        'osfile_worker_ready_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.osfile_worker_ready_ms,
        (1, 5000, 10)
      ),
      (
        'osfile_writeatomic_jank_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.osfile_writeatomic_jank_ms,
        (1, 5000, 10)
      ),
      (
        'page_faults_hard',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.page_faults_hard,
        (8, 65536, 13)
      ),
      (
        'page_max_scroll_y',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.page_max_scroll_y,
        (1, 100000, 50)
      ),
      (
        'page_metadata_size',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.page_metadata_size,
        (1, 500, 10)
      ),
      (
        'paint_build_displaylist_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.paint_build_displaylist_time,
        (1, 1000, 50)
      ),
      (
        'paint_build_layers_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.paint_build_layers_time,
        (1, 1000, 50)
      ),
      (
        'paint_rasterize_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.paint_rasterize_time,
        (1, 1000, 50)
      ),
      (
        'pdf_viewer_document_generator',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pdf_viewer_document_generator,
        (1, 30, 31)
      ),
      (
        'pdf_viewer_document_generator_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_document_generator_2,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_document_size_kb',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.pdf_viewer_document_size_kb,
        (2, 65536, 20)
      ),
      (
        'pdf_viewer_document_version',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pdf_viewer_document_version,
        (1, 20, 21)
      ),
      (
        'pdf_viewer_document_version_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_document_version_2,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_embed',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pdf_viewer_embed,
        (1, 2, 3)
      ),
      (
        'pdf_viewer_embed_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_embed_2,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_fallback_error',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_fallback_error,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_fallback_reason',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_fallback_reason,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_fallback_shown',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pdf_viewer_fallback_shown,
        (1, 2, 3)
      ),
      (
        'pdf_viewer_font_types',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pdf_viewer_font_types,
        (1, 19, 20)
      ),
      (
        'pdf_viewer_font_types_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_font_types_2,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_form',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pdf_viewer_form,
        (1, 2, 3)
      ),
      (
        'pdf_viewer_form_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_form_2,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_print',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pdf_viewer_print,
        (1, 2, 3)
      ),
      (
        'pdf_viewer_stream_types',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pdf_viewer_stream_types,
        (1, 19, 20)
      ),
      (
        'pdf_viewer_stream_types_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.pdf_viewer_stream_types_2,
        (1, 50, 51)
      ),
      (
        'pdf_viewer_time_to_view_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.pdf_viewer_time_to_view_ms,
        (1, 10000, 50)
      ),
      (
        'pdf_viewer_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pdf_viewer_used,
        (1, 2, 3)
      ),
      (
        'pending_critical_input_when_timeout',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pending_critical_input_when_timeout,
        (1, 80, 81)
      ),
      (
        'perf_monitoring_test_cpu_rescheduling_proportion_moved',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.perf_monitoring_test_cpu_rescheduling_proportion_moved,
        (1, 100, 20)
      ),
      (
        'permissions_migration_7_error',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.permissions_migration_7_error,
        (1, 2, 3)
      ),
      (
        'permissions_remigration_comparison',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.permissions_remigration_comparison,
        (1, 10, 11)
      ),
      (
        'permissions_sql_corrupted',
        'histogram-count',
        'content',
        payload.processes.content.histograms.permissions_sql_corrupted,
        (1, 2, 3)
      ),
      (
        'places_annos_bookmarks_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_annos_bookmarks_count,
        (50, 5000, 10)
      ),
      (
        'places_annos_pages_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_annos_pages_count,
        (50, 5000, 10)
      ),
      (
        'places_autocomplete_1st_result_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_autocomplete_1st_result_time_ms,
        (50, 500, 10)
      ),
      (
        'places_autocomplete_6_first_results_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_autocomplete_6_first_results_time_ms,
        (50, 1000, 30)
      ),
      (
        'places_autocomplete_urlinline_domain_query_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_autocomplete_urlinline_domain_query_time_ms,
        (50, 2000, 10)
      ),
      (
        'places_backups_bookmarkstree_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_backups_bookmarkstree_ms,
        (50, 2000, 10)
      ),
      (
        'places_backups_daysfromlast',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.places_backups_daysfromlast,
        (1, 15, 16)
      ),
      (
        'places_backups_tojson_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_backups_tojson_ms,
        (50, 2000, 10)
      ),
      (
        'places_bookmarks_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_bookmarks_count,
        (100, 8000, 15)
      ),
      (
        'places_database_filesize_mb',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_database_filesize_mb,
        (5, 200, 10)
      ),
      (
        'places_database_pagesize_b',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_database_pagesize_b,
        (1024, 32768, 10)
      ),
      (
        'places_database_size_per_page_b',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_database_size_per_page_b,
        (500, 10240, 20)
      ),
      (
        'places_expiration_steps_to_clean2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.places_expiration_steps_to_clean2,
        (1, 10, 11)
      ),
      (
        'places_export_tohtml_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_export_tohtml_ms,
        (50, 2000, 10)
      ),
      (
        'places_favicon_bmp_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_bmp_sizes,
        (1, 524288, 100)
      ),
      (
        'places_favicon_gif_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_gif_sizes,
        (1, 524288, 100)
      ),
      (
        'places_favicon_ico_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_ico_sizes,
        (1, 524288, 100)
      ),
      (
        'places_favicon_jpeg_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_jpeg_sizes,
        (1, 524288, 100)
      ),
      (
        'places_favicon_other_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_other_sizes,
        (1, 524288, 100)
      ),
      (
        'places_favicon_png_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_png_sizes,
        (1, 524288, 100)
      ),
      (
        'places_favicon_svg_sizes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_favicon_svg_sizes,
        (1, 524288, 100)
      ),
      (
        'places_history_library_search_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_history_library_search_time_ms,
        (50, 1000, 30)
      ),
      (
        'places_idle_frecency_decay_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_idle_frecency_decay_time_ms,
        (50, 10000, 10)
      ),
      (
        'places_idle_maintenance_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_idle_maintenance_time_ms,
        (1000, 30000, 10)
      ),
      (
        'places_keywords_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_keywords_count,
        (1, 200, 10)
      ),
      (
        'places_maintenance_daysfromlast',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_maintenance_daysfromlast,
        (7, 60, 10)
      ),
      (
        'places_most_recent_expired_visit_days',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.places_most_recent_expired_visit_days,
        (30, 730, 12)
      ),
      (
        'places_pages_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_pages_count,
        (1000, 150000, 20)
      ),
      (
        'places_sorted_bookmarks_perc',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.places_sorted_bookmarks_perc,
        (1, 100, 10)
      ),
      (
        'places_tagged_bookmarks_perc',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.places_tagged_bookmarks_perc,
        (1, 100, 10)
      ),
      (
        'places_tags_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.places_tags_count,
        (1, 200, 10)
      ),
      (
        'plugin_drawing_model',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.plugin_drawing_model,
        (1, 12, 13)
      ),
      (
        'plugin_hang_notice_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.plugin_hang_notice_count,
        (1, 2, 3)
      ),
      (
        'plugin_hang_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.plugin_hang_time,
        (1, 60000, 20)
      ),
      (
        'plugin_hang_ui_dont_ask',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.plugin_hang_ui_dont_ask,
        (1, 2, 3)
      ),
      (
        'plugin_hang_ui_response_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.plugin_hang_ui_response_time,
        (1, 60000, 20)
      ),
      (
        'plugin_hang_ui_user_response',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.plugin_hang_ui_user_response,
        (1, 3, 4)
      ),
      (
        'plugin_load_metadata',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.plugin_load_metadata,
        (1, 5000, 20)
      ),
      (
        'plugin_shutdown_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.plugin_shutdown_ms,
        (1, 5000, 20)
      ),
      (
        'plugins_notification_plugin_count',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.plugins_notification_plugin_count,
        (1, 5, 6)
      ),
      (
        'plugins_notification_shown',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.plugins_notification_shown,
        (1, 2, 3)
      ),
      (
        'plugins_notification_user_action',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.plugins_notification_user_action,
        (1, 3, 4)
      ),
      (
        'plugins_notification_user_action_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.plugins_notification_user_action_2,
        (1, 8, 9)
      ),
      (
        'prclose_tcp_blocking_time_connectivity_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_tcp_blocking_time_connectivity_change,
        (1, 60000, 1000)
      ),
      (
        'prclose_tcp_blocking_time_link_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_tcp_blocking_time_link_change,
        (1, 60000, 1000)
      ),
      (
        'prclose_tcp_blocking_time_normal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_tcp_blocking_time_normal,
        (1, 60000, 1000)
      ),
      (
        'prclose_tcp_blocking_time_offline',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_tcp_blocking_time_offline,
        (1, 60000, 1000)
      ),
      (
        'prclose_tcp_blocking_time_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_tcp_blocking_time_shutdown,
        (1, 60000, 1000)
      ),
      (
        'prclose_udp_blocking_time_connectivity_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_udp_blocking_time_connectivity_change,
        (1, 60000, 1000)
      ),
      (
        'prclose_udp_blocking_time_link_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_udp_blocking_time_link_change,
        (1, 60000, 1000)
      ),
      (
        'prclose_udp_blocking_time_normal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_udp_blocking_time_normal,
        (1, 60000, 1000)
      ),
      (
        'prclose_udp_blocking_time_offline',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_udp_blocking_time_offline,
        (1, 60000, 1000)
      ),
      (
        'prclose_udp_blocking_time_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prclose_udp_blocking_time_shutdown,
        (1, 60000, 1000)
      ),
      (
        'prconnect_blocking_time_connectivity_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_blocking_time_connectivity_change,
        (1, 60000, 1000)
      ),
      (
        'prconnect_blocking_time_link_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_blocking_time_link_change,
        (1, 60000, 1000)
      ),
      (
        'prconnect_blocking_time_normal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_blocking_time_normal,
        (1, 60000, 1000)
      ),
      (
        'prconnect_blocking_time_offline',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_blocking_time_offline,
        (1, 60000, 1000)
      ),
      (
        'prconnect_blocking_time_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_blocking_time_shutdown,
        (1, 60000, 1000)
      ),
      (
        'prconnect_fail_blocking_time_connectivity_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_fail_blocking_time_connectivity_change,
        (1, 60000, 100)
      ),
      (
        'prconnect_fail_blocking_time_link_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_fail_blocking_time_link_change,
        (1, 60000, 100)
      ),
      (
        'prconnect_fail_blocking_time_normal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_fail_blocking_time_normal,
        (1, 60000, 100)
      ),
      (
        'prconnect_fail_blocking_time_offline',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_fail_blocking_time_offline,
        (1, 60000, 100)
      ),
      (
        'prconnect_fail_blocking_time_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnect_fail_blocking_time_shutdown,
        (1, 60000, 100)
      ),
      (
        'prconnectcontinue_blocking_time_connectivity_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnectcontinue_blocking_time_connectivity_change,
        (1, 60000, 1000)
      ),
      (
        'prconnectcontinue_blocking_time_link_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnectcontinue_blocking_time_link_change,
        (1, 60000, 1000)
      ),
      (
        'prconnectcontinue_blocking_time_normal',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnectcontinue_blocking_time_normal,
        (1, 60000, 1000)
      ),
      (
        'prconnectcontinue_blocking_time_offline',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnectcontinue_blocking_time_offline,
        (1, 60000, 1000)
      ),
      (
        'prconnectcontinue_blocking_time_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.prconnectcontinue_blocking_time_shutdown,
        (1, 60000, 1000)
      ),
      (
        'predictor_base_confidence',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.predictor_base_confidence,
        (1, 100, 50)
      ),
      (
        'predictor_confidence',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.predictor_confidence,
        (1, 100, 50)
      ),
      (
        'predictor_global_degradation',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.predictor_global_degradation,
        (1, 100, 50)
      ),
      (
        'predictor_learn_attempts',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_learn_attempts,
        (1, 1000000, 50)
      ),
      (
        'predictor_learn_full_queue',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_learn_full_queue,
        (1, 60000, 50)
      ),
      (
        'predictor_learn_work_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_learn_work_time,
        (1, 3000, 10)
      ),
      (
        'predictor_predict_attempts',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_predict_attempts,
        (1, 1000000, 50)
      ),
      (
        'predictor_predict_full_queue',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_predict_full_queue,
        (1, 60000, 50)
      ),
      (
        'predictor_predict_time_to_action',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_predict_time_to_action,
        (1, 3000, 10)
      ),
      (
        'predictor_predict_time_to_inaction',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_predict_time_to_inaction,
        (1, 3000, 10)
      ),
      (
        'predictor_predict_work_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_predict_work_time,
        (1, 3000, 10)
      ),
      (
        'predictor_predictions_calculated',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_predictions_calculated,
        (1, 1000000, 50)
      ),
      (
        'predictor_prefetch_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_prefetch_time,
        (1, 3000, 10)
      ),
      (
        'predictor_subresource_degradation',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.predictor_subresource_degradation,
        (1, 100, 50)
      ),
      (
        'predictor_total_preconnects',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_preconnects,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_preconnects_created',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_preconnects_created,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_preconnects_unused',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_preconnects_unused,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_preconnects_used',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_preconnects_used,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_predictions',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_predictions,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_prefetches',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_prefetches,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_prefetches_used',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_prefetches_used,
        (1, 1000000, 50)
      ),
      (
        'predictor_total_preresolves',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_total_preresolves,
        (1, 1000000, 50)
      ),
      (
        'predictor_wait_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.predictor_wait_time,
        (1, 3000, 10)
      ),
      (
        'presshell_reqs_per_style_flush',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.presshell_reqs_per_style_flush,
        (1, 256, 32)
      ),
      (
        'print_preview_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.print_preview_opened_count,
        (1, 2, 3)
      ),
      (
        'print_preview_simplify_page_opened_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.print_preview_simplify_page_opened_count,
        (1, 2, 3)
      ),
      (
        'print_preview_simplify_page_unavailable_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.print_preview_simplify_page_unavailable_count,
        (1, 2, 3)
      ),
      (
        'push_api_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.push_api_notify,
        (1, 2, 3)
      ),
      (
        'pwmgr_blocklist_num_sites',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.pwmgr_blocklist_num_sites,
        (1, 100, 10)
      ),
      (
        'pwmgr_form_autofill_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pwmgr_form_autofill_result,
        (1, 20, 21)
      ),
      (
        'pwmgr_login_last_used_days',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.pwmgr_login_last_used_days,
        (1, 750, 40)
      ),
      (
        'pwmgr_login_page_safety',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pwmgr_login_page_safety,
        (1, 8, 9)
      ),
      (
        'pwmgr_manage_copied_password',
        'histogram-count',
        'content',
        payload.processes.content.histograms.pwmgr_manage_copied_password,
        (1, 2, 3)
      ),
      (
        'pwmgr_manage_copied_username',
        'histogram-count',
        'content',
        payload.processes.content.histograms.pwmgr_manage_copied_username,
        (1, 2, 3)
      ),
      (
        'pwmgr_manage_deleted',
        'histogram-count',
        'content',
        payload.processes.content.histograms.pwmgr_manage_deleted,
        (1, 2, 3)
      ),
      (
        'pwmgr_manage_deleted_all',
        'histogram-count',
        'content',
        payload.processes.content.histograms.pwmgr_manage_deleted_all,
        (1, 2, 3)
      ),
      (
        'pwmgr_manage_opened',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pwmgr_manage_opened,
        (1, 5, 6)
      ),
      (
        'pwmgr_manage_visibility_toggled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pwmgr_manage_visibility_toggled,
        (1, 2, 3)
      ),
      (
        'pwmgr_num_httpauth_passwords',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.pwmgr_num_httpauth_passwords,
        (1, 750, 50)
      ),
      (
        'pwmgr_num_passwords_per_hostname',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.pwmgr_num_passwords_per_hostname,
        (1, 21, 20)
      ),
      (
        'pwmgr_num_saved_passwords',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.pwmgr_num_saved_passwords,
        (1, 750, 50)
      ),
      (
        'pwmgr_password_input_in_form',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pwmgr_password_input_in_form,
        (1, 2, 3)
      ),
      (
        'pwmgr_prompt_remember_action',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pwmgr_prompt_remember_action,
        (1, 5, 6)
      ),
      (
        'pwmgr_prompt_update_action',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.pwmgr_prompt_update_action,
        (1, 5, 6)
      ),
      (
        'pwmgr_saving_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pwmgr_saving_enabled,
        (1, 2, 3)
      ),
      (
        'pwmgr_username_present',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.pwmgr_username_present,
        (1, 2, 3)
      ),
      (
        'quirks_mode',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.quirks_mode,
        (1, 50, 51)
      ),
      (
        'range_checksum_errors',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.range_checksum_errors,
        (1, 3000, 10)
      ),
      (
        'reader_mode_download_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.reader_mode_download_result,
        (1, 5, 6)
      ),
      (
        'reader_mode_parse_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.reader_mode_parse_result,
        (1, 5, 6)
      ),
      (
        'refresh_driver_tick',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.refresh_driver_tick,
        (1, 1000, 50)
      ),
      (
        'region_location_services_difference',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.region_location_services_difference,
        (1, 12742000, 20)
      ),
      (
        'requests_of_original_content',
        'histogram-count',
        'content',
        payload.processes.content.histograms.requests_of_original_content,
        (1, 2, 3)
      ),
      (
        'safe_mode_usage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.safe_mode_usage,
        (1, 3, 4)
      ),
      (
        'sandbox_content_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.sandbox_content_enabled,
        (1, 2, 3)
      ),
      (
        'sandbox_has_seccomp_bpf',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.sandbox_has_seccomp_bpf,
        (1, 2, 3)
      ),
      (
        'sandbox_has_seccomp_tsync',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.sandbox_has_seccomp_tsync,
        (1, 2, 3)
      ),
      (
        'sandbox_has_user_namespaces',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.sandbox_has_user_namespaces,
        (1, 2, 3)
      ),
      (
        'sandbox_has_user_namespaces_privileged',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.sandbox_has_user_namespaces_privileged,
        (1, 2, 3)
      ),
      (
        'sandbox_media_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.sandbox_media_enabled,
        (1, 2, 3)
      ),
      (
        'script_block_incorrect_mime',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.script_block_incorrect_mime,
        (1, 15, 16)
      ),
      (
        'script_block_incorrect_mime_2',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.script_block_incorrect_mime_2,
        (1, 50, 51)
      ),
      (
        'script_block_incorrect_mime_3',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.script_block_incorrect_mime_3,
        (1, 50, 51)
      ),
      (
        'script_preloader_requests',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.script_preloader_requests,
        (1, 50, 51)
      ),
      (
        'script_preloader_wait_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.script_preloader_wait_time,
        (1, 10000, 20)
      ),
      (
        'scroll_anchor_adjustment_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.scroll_anchor_adjustment_count,
        (1, 500, 50)
      ),
      (
        'scroll_anchor_adjustment_length',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.scroll_anchor_adjustment_length,
        (1, 10000, 50)
      ),
      (
        'scroll_input_methods',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.scroll_input_methods,
        (1, 64, 65)
      ),
      (
        'search_reset_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.search_reset_result,
        (1, 5, 6)
      ),
      (
        'search_service_country_fetch_caused_sync_init',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_country_fetch_caused_sync_init,
        (1, 2, 3)
      ),
      (
        'search_service_country_fetch_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.search_service_country_fetch_result,
        (1, 8, 9)
      ),
      (
        'search_service_country_fetch_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.search_service_country_fetch_time_ms,
        (1, 100000, 30)
      ),
      (
        'search_service_country_timeout',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_country_timeout,
        (1, 2, 3)
      ),
      (
        'search_service_engine_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.search_service_engine_count,
        (1, 200, 50)
      ),
      (
        'search_service_init_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.search_service_init_ms,
        (1, 1000, 15)
      ),
      (
        'search_service_init_sync',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_init_sync,
        (1, 2, 3)
      ),
      (
        'search_service_nonus_country_mismatched_platform_osx',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_nonus_country_mismatched_platform_osx,
        (1, 2, 3)
      ),
      (
        'search_service_nonus_country_mismatched_platform_win',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_nonus_country_mismatched_platform_win,
        (1, 2, 3)
      ),
      (
        'search_service_us_country_mismatched_platform_osx',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_us_country_mismatched_platform_osx,
        (1, 2, 3)
      ),
      (
        'search_service_us_country_mismatched_platform_win',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.search_service_us_country_mismatched_platform_win,
        (1, 2, 3)
      ),
      (
        'search_service_us_country_mismatched_timezone',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.search_service_us_country_mismatched_timezone,
        (1, 2, 3)
      ),
      (
        'search_service_us_timezone_mismatched_country',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.search_service_us_timezone_mismatched_country,
        (1, 2, 3)
      ),
      (
        'security_ui',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.security_ui,
        (1, 100, 101)
      ),
      (
        'service_worker_controlled_documents',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_controlled_documents,
        (1, 2, 3)
      ),
      (
        'service_worker_destroyed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_destroyed,
        (1, 2, 3)
      ),
      (
        'service_worker_launch_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.service_worker_launch_time,
        (1, 5000, 25)
      ),
      (
        'service_worker_life_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.service_worker_life_time,
        (1, 120000, 20)
      ),
      (
        'service_worker_registration_loading',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.service_worker_registration_loading,
        (1, 5000, 20)
      ),
      (
        'service_worker_registrations',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_registrations,
        (1, 2, 3)
      ),
      (
        'service_worker_request_passthrough',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.service_worker_request_passthrough,
        (1, 2, 3)
      ),
      (
        'service_worker_spawn_attempts',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_spawn_attempts,
        (1, 2, 3)
      ),
      (
        'service_worker_spawn_gets_queued',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_spawn_gets_queued,
        (1, 2, 3)
      ),
      (
        'service_worker_updated',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_updated,
        (1, 2, 3)
      ),
      (
        'service_worker_was_spawned',
        'histogram-count',
        'content',
        payload.processes.content.histograms.service_worker_was_spawned,
        (1, 2, 3)
      ),
      (
        'shared_memory_ua_sheets_mapped_child',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.shared_memory_ua_sheets_mapped_child,
        (1, 2, 3)
      ),
      (
        'shared_worker_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.shared_worker_count,
        (1, 2, 3)
      ),
      (
        'shared_worker_destroyed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.shared_worker_destroyed,
        (1, 2, 3)
      ),
      (
        'shared_worker_spawn_gets_queued',
        'histogram-count',
        'content',
        payload.processes.content.histograms.shared_worker_spawn_gets_queued,
        (1, 2, 3)
      ),
      (
        'should_auto_detect_language',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.should_auto_detect_language,
        (1, 2, 3)
      ),
      (
        'should_translation_ui_appear',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.should_translation_ui_appear,
        (1, 2, 3)
      ),
      (
        'shutdown_ok',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.shutdown_ok,
        (1, 2, 3)
      ),
      (
        'shutdown_phase_duration_ticks_profile_before_change',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.shutdown_phase_duration_ticks_profile_before_change,
        (1, 65, 10)
      ),
      (
        'shutdown_phase_duration_ticks_profile_change_teardown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.shutdown_phase_duration_ticks_profile_change_teardown,
        (1, 65, 10)
      ),
      (
        'shutdown_phase_duration_ticks_quit_application',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.shutdown_phase_duration_ticks_quit_application,
        (1, 65, 10)
      ),
      (
        'shutdown_phase_duration_ticks_xpcom_will_shutdown',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.shutdown_phase_duration_ticks_xpcom_will_shutdown,
        (1, 65, 10)
      ),
      (
        'slow_addon_warning_response_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.slow_addon_warning_response_time,
        (1, 86400000, 30)
      ),
      (
        'slow_addon_warning_states',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.slow_addon_warning_states,
        (1, 20, 21)
      ),
      (
        'slow_script_notice_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.slow_script_notice_count,
        (1, 2, 3)
      ),
      (
        'slow_script_notify_delay',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.slow_script_notify_delay,
        (1, 60000, 50)
      ),
      (
        'slow_script_page_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.slow_script_page_count,
        (1, 2, 3)
      ),
      (
        'social_enabled_on_session',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.social_enabled_on_session,
        (1, 2, 3)
      ),
      (
        'social_panel_clicks',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.social_panel_clicks,
        (1, 4, 5)
      ),
      (
        'social_sidebar_open_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.social_sidebar_open_duration,
        (1, 10000000, 10)
      ),
      (
        'social_sidebar_state',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.social_sidebar_state,
        (1, 2, 3)
      ),
      (
        'social_toolbar_buttons',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.social_toolbar_buttons,
        (1, 3, 4)
      ),
      (
        'spdy_chunk_recvd',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_chunk_recvd,
        (1, 1000, 100)
      ),
      (
        'spdy_continued_headers',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_continued_headers,
        (1, 32000000, 75)
      ),
      (
        'spdy_goaway_local',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.spdy_goaway_local,
        (1, 32, 33)
      ),
      (
        'spdy_goaway_peer',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.spdy_goaway_peer,
        (1, 32, 33)
      ),
      (
        'spdy_kbread_per_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_kbread_per_conn,
        (1, 3000, 50)
      ),
      (
        'spdy_kbread_per_conn2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_kbread_per_conn2,
        (1, 100000, 50)
      ),
      (
        'spdy_npn_connect',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.spdy_npn_connect,
        (1, 2, 3)
      ),
      (
        'spdy_npn_join',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.spdy_npn_join,
        (1, 2, 3)
      ),
      (
        'spdy_parallel_streams',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_parallel_streams,
        (1, 1000, 50)
      ),
      (
        'spdy_request_per_conn',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_request_per_conn,
        (1, 1000, 50)
      ),
      (
        'spdy_request_per_conn_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_request_per_conn_2,
        (1, 1000, 50)
      ),
      (
        'spdy_server_initiated_streams',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_server_initiated_streams,
        (1, 100000, 250)
      ),
      (
        'spdy_settings_iw',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_settings_iw,
        (1, 1000, 50)
      ),
      (
        'spdy_settings_max_streams',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_settings_max_streams,
        (1, 5000, 100)
      ),
      (
        'spdy_syn_ratio',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.spdy_syn_ratio,
        (1, 99, 20)
      ),
      (
        'spdy_syn_reply_ratio',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.spdy_syn_reply_ratio,
        (1, 99, 20)
      ),
      (
        'spdy_syn_reply_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_syn_reply_size,
        (16, 20000, 50)
      ),
      (
        'spdy_syn_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.spdy_syn_size,
        (20, 20000, 50)
      ),
      (
        'spdy_version2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.spdy_version2,
        (1, 48, 49)
      ),
      (
        'ssl_auth_algorithm_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_auth_algorithm_full,
        (1, 16, 17)
      ),
      (
        'ssl_auth_ecdsa_curve_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_auth_ecdsa_curve_full,
        (1, 36, 37)
      ),
      (
        'ssl_auth_rsa_key_size_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_auth_rsa_key_size_full,
        (1, 24, 25)
      ),
      (
        'ssl_bytes_before_cert_callback',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ssl_bytes_before_cert_callback,
        (1, 32000, 64)
      ),
      (
        'ssl_cert_error_overrides',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_cert_error_overrides,
        (1, 24, 25)
      ),
      (
        'ssl_cert_verification_errors',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_cert_verification_errors,
        (1, 100, 101)
      ),
      (
        'ssl_cipher_suite_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_cipher_suite_full,
        (1, 128, 129)
      ),
      (
        'ssl_cipher_suite_resumed',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_cipher_suite_resumed,
        (1, 128, 129)
      ),
      (
        'ssl_ct_policy_compliance_of_ev_certs',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_ct_policy_compliance_of_ev_certs,
        (1, 10, 11)
      ),
      (
        'ssl_ct_policy_compliant_connections_by_ca',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_ct_policy_compliant_connections_by_ca,
        (1, 256, 257)
      ),
      (
        'ssl_ct_policy_non_compliant_connections_by_ca',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_ct_policy_non_compliant_connections_by_ca,
        (1, 256, 257)
      ),
      (
        'ssl_handshake_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_handshake_result,
        (1, 672, 673)
      ),
      (
        'ssl_handshake_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_handshake_type,
        (1, 8, 9)
      ),
      (
        'ssl_handshake_version',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_handshake_version,
        (1, 16, 17)
      ),
      (
        'ssl_initial_failed_cert_validation_time_mozillapkix',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ssl_initial_failed_cert_validation_time_mozillapkix,
        (1, 60000, 50)
      ),
      (
        'ssl_kea_dhe_key_size_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_kea_dhe_key_size_full,
        (1, 24, 25)
      ),
      (
        'ssl_kea_ecdhe_curve_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_kea_ecdhe_curve_full,
        (1, 36, 37)
      ),
      (
        'ssl_kea_rsa_key_size_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_kea_rsa_key_size_full,
        (1, 24, 25)
      ),
      (
        'ssl_key_exchange_algorithm_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_key_exchange_algorithm_full,
        (1, 16, 17)
      ),
      (
        'ssl_key_exchange_algorithm_resumed',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_key_exchange_algorithm_resumed,
        (1, 16, 17)
      ),
      (
        'ssl_npn_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_npn_type,
        (1, 16, 17)
      ),
      (
        'ssl_observed_end_entity_certificate_lifetime',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_observed_end_entity_certificate_lifetime,
        (1, 125, 126)
      ),
      (
        'ssl_ocsp_may_fetch',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_ocsp_may_fetch,
        (1, 8, 9)
      ),
      (
        'ssl_ocsp_stapling',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_ocsp_stapling,
        (1, 8, 9)
      ),
      (
        'ssl_permanent_cert_error_overrides',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ssl_permanent_cert_error_overrides,
        (1, 1024, 10)
      ),
      (
        'ssl_reasons_for_not_false_starting',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_reasons_for_not_false_starting,
        (1, 512, 513)
      ),
      (
        'ssl_resumed_session',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.ssl_resumed_session,
        (1, 2, 3)
      ),
      (
        'ssl_scts_origin',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_scts_origin,
        (1, 10, 11)
      ),
      (
        'ssl_scts_per_connection',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_scts_per_connection,
        (1, 10, 11)
      ),
      (
        'ssl_scts_verification_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_scts_verification_status,
        (1, 10, 11)
      ),
      (
        'ssl_server_auth_eku',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_server_auth_eku,
        (1, 10, 11)
      ),
      (
        'ssl_succesful_cert_validation_time_mozillapkix',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ssl_succesful_cert_validation_time_mozillapkix,
        (1, 60000, 50)
      ),
      (
        'ssl_symmetric_cipher_full',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_symmetric_cipher_full,
        (1, 32, 33)
      ),
      (
        'ssl_symmetric_cipher_resumed',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_symmetric_cipher_resumed,
        (1, 32, 33)
      ),
      (
        'ssl_time_until_handshake_finished',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ssl_time_until_handshake_finished,
        (1, 60000, 200)
      ),
      (
        'ssl_time_until_ready',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.ssl_time_until_ready,
        (1, 60000, 200)
      ),
      (
        'ssl_tls10_intolerance_reason_post',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls10_intolerance_reason_post,
        (1, 64, 65)
      ),
      (
        'ssl_tls10_intolerance_reason_pre',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls10_intolerance_reason_pre,
        (1, 64, 65)
      ),
      (
        'ssl_tls11_intolerance_reason_post',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls11_intolerance_reason_post,
        (1, 64, 65)
      ),
      (
        'ssl_tls11_intolerance_reason_pre',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls11_intolerance_reason_pre,
        (1, 64, 65)
      ),
      (
        'ssl_tls12_intolerance_reason_post',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls12_intolerance_reason_post,
        (1, 64, 65)
      ),
      (
        'ssl_tls12_intolerance_reason_pre',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls12_intolerance_reason_pre,
        (1, 64, 65)
      ),
      (
        'ssl_tls13_intolerance_reason_post',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls13_intolerance_reason_post,
        (1, 64, 65)
      ),
      (
        'ssl_tls13_intolerance_reason_pre',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_tls13_intolerance_reason_pre,
        (1, 64, 65)
      ),
      (
        'ssl_version_fallback_inappropriate',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.ssl_version_fallback_inappropriate,
        (1, 64, 65)
      ),
      (
        'startup_crash_detected',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.startup_crash_detected,
        (1, 2, 3)
      ),
      (
        'startup_measurement_errors',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.startup_measurement_errors,
        (1, 16, 17)
      ),
      (
        'storage_access_api_ui',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.storage_access_api_ui,
        (1, 50, 51)
      ),
      (
        'sts_number_of_onsocketready_calls',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_number_of_onsocketready_calls,
        (1, 2000, 100)
      ),
      (
        'sts_number_of_pending_events',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_number_of_pending_events,
        (1, 2000, 100)
      ),
      (
        'sts_number_of_pending_events_in_the_last_cycle',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_number_of_pending_events_in_the_last_cycle,
        (1, 2000, 100)
      ),
      (
        'sts_poll_and_event_the_last_cycle',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_poll_and_event_the_last_cycle,
        (1, 60000, 1000)
      ),
      (
        'sts_poll_and_events_cycle',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_poll_and_events_cycle,
        (1, 60000, 1000)
      ),
      (
        'sts_poll_block_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_poll_block_time,
        (1, 60000, 1000)
      ),
      (
        'sts_poll_cycle',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.sts_poll_cycle,
        (1, 60000, 1000)
      ),
      (
        'stumbler_observations_per_day',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_observations_per_day,
        (1, 10000, 50)
      ),
      (
        'stumbler_time_between_received_locations_sec',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_time_between_received_locations_sec,
        (1, 86400, 50)
      ),
      (
        'stumbler_time_between_start_sec',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_time_between_start_sec,
        (1, 259200, 50)
      ),
      (
        'stumbler_time_between_uploads_sec',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_time_between_uploads_sec,
        (1, 259200, 50)
      ),
      (
        'stumbler_upload_bytes',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_upload_bytes,
        (1, 1000000, 50)
      ),
      (
        'stumbler_upload_cell_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_upload_cell_count,
        (1, 10000, 50)
      ),
      (
        'stumbler_upload_observation_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_upload_observation_count,
        (1, 10000, 50)
      ),
      (
        'stumbler_upload_wifi_ap_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_upload_wifi_ap_count,
        (1, 10000, 50)
      ),
      (
        'stumbler_volume_bytes_uploaded_per_sec',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.stumbler_volume_bytes_uploaded_per_sec,
        (1, 1000000, 50)
      ),
      (
        'stylo_parallel_restyle_fraction',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.stylo_parallel_restyle_fraction,
        (1, 100, 50)
      ),
      (
        'stylo_parallel_restyle_fraction_weighted_styled',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.stylo_parallel_restyle_fraction_weighted_styled,
        (1, 100, 50)
      ),
      (
        'stylo_parallel_restyle_fraction_weighted_traversed',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.stylo_parallel_restyle_fraction_weighted_traversed,
        (1, 100, 50)
      ),
      (
        'subject_principal_accessed_without_script_on_stack',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.subject_principal_accessed_without_script_on_stack,
        (1, 2, 3)
      ),
      (
        'system_font_fallback',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.system_font_fallback,
        (1, 100000, 50)
      ),
      (
        'system_font_fallback_first',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.system_font_fallback_first,
        (1, 40000, 20)
      ),
      (
        'system_font_fallback_script',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.system_font_fallback_script,
        (1, 110, 111)
      ),
      (
        'tab_audio_indicator_used',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tab_audio_indicator_used,
        (1, 6, 7)
      ),
      (
        'tab_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.tab_count,
        (1, 1000, 100)
      ),
      (
        'tab_media_blocking_time_ms',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.tab_media_blocking_time_ms,
        (1000, 90000, 91)
      ),
      (
        'tabchild_paint_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.tabchild_paint_time,
        (1, 1000, 50)
      ),
      (
        'tap_to_load_enabled',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tap_to_load_enabled,
        (1, 3, 4)
      ),
      (
        'tap_to_load_image_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.tap_to_load_image_size,
        (1, 32768, 50)
      ),
      (
        'tcp_fast_open',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tcp_fast_open,
        (1, 8, 9)
      ),
      (
        'tcp_fast_open_2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tcp_fast_open_2,
        (1, 16, 17)
      ),
      (
        'tcp_fast_open_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tcp_fast_open_status,
        (1, 8, 9)
      ),
      (
        'telemetry_archive_checking_over_quota_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_archive_checking_over_quota_ms,
        (1, 300000, 20)
      ),
      (
        'telemetry_archive_directories_count',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_archive_directories_count,
        (1, 13, 12)
      ),
      (
        'telemetry_archive_evicted_old_dirs',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_archive_evicted_old_dirs,
        (1, 13, 12)
      ),
      (
        'telemetry_archive_evicted_over_quota',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_archive_evicted_over_quota,
        (1, 100000, 100)
      ),
      (
        'telemetry_archive_evicting_dirs_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_archive_evicting_dirs_ms,
        (1, 300000, 20)
      ),
      (
        'telemetry_archive_evicting_over_quota_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_archive_evicting_over_quota_ms,
        (1, 300000, 20)
      ),
      (
        'telemetry_archive_oldest_directory_age',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_archive_oldest_directory_age,
        (1, 13, 12)
      ),
      (
        'telemetry_archive_scan_ping_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_archive_scan_ping_count,
        (1, 100000, 100)
      ),
      (
        'telemetry_archive_session_ping_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_archive_session_ping_count,
        (1, 2, 3)
      ),
      (
        'telemetry_archive_size_mb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_archive_size_mb,
        (1, 300, 60)
      ),
      (
        'telemetry_assemble_payload_exception',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_assemble_payload_exception,
        (1, 2, 3)
      ),
      (
        'telemetry_compress',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_compress,
        (1, 3000, 10)
      ),
      (
        'telemetry_discarded_archived_pings_size_mb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_discarded_archived_pings_size_mb,
        (1, 30, 29)
      ),
      (
        'telemetry_discarded_content_pings_count',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_discarded_content_pings_count,
        (1, 2, 3)
      ),
      (
        'telemetry_discarded_pending_pings_size_mb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_discarded_pending_pings_size_mb,
        (1, 30, 29)
      ),
      (
        'telemetry_discarded_send_pings_size_mb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_discarded_send_pings_size_mb,
        (1, 30, 29)
      ),
      (
        'telemetry_event_recording_error',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.telemetry_event_recording_error,
        (1, 50, 51)
      ),
      (
        'telemetry_invalid_payload_submitted',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_invalid_payload_submitted,
        (1, 2, 3)
      ),
      (
        'telemetry_memory_reporter_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_memory_reporter_ms,
        (1, 5000, 10)
      ),
      (
        'telemetry_pending_checking_over_quota_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_pending_checking_over_quota_ms,
        (1, 300000, 20)
      ),
      (
        'telemetry_pending_evicting_over_quota_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_pending_evicting_over_quota_ms,
        (1, 300000, 20)
      ),
      (
        'telemetry_pending_load_failure_parse',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_pending_load_failure_parse,
        (1, 2, 3)
      ),
      (
        'telemetry_pending_load_failure_read',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_pending_load_failure_read,
        (1, 2, 3)
      ),
      (
        'telemetry_pending_pings_age',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_pending_pings_age,
        (1, 365, 30)
      ),
      (
        'telemetry_pending_pings_evicted_over_quota',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_pending_pings_evicted_over_quota,
        (1, 100000, 100)
      ),
      (
        'telemetry_pending_pings_size_mb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.telemetry_pending_pings_size_mb,
        (1, 17, 16)
      ),
      (
        'telemetry_ping_evicted_for_server_errors',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_ping_evicted_for_server_errors,
        (1, 2, 3)
      ),
      (
        'telemetry_ping_size_exceeded_archived',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_ping_size_exceeded_archived,
        (1, 2, 3)
      ),
      (
        'telemetry_ping_size_exceeded_pending',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_ping_size_exceeded_pending,
        (1, 2, 3)
      ),
      (
        'telemetry_ping_size_exceeded_send',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_ping_size_exceeded_send,
        (1, 2, 3)
      ),
      (
        'telemetry_ping_submission_waiting_clientid',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_ping_submission_waiting_clientid,
        (1, 2, 3)
      ),
      (
        'telemetry_scheduler_tick_exception',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_scheduler_tick_exception,
        (1, 2, 3)
      ),
      (
        'telemetry_scheduler_wakeup',
        'histogram-count',
        'content',
        payload.processes.content.histograms.telemetry_scheduler_wakeup,
        (1, 2, 3)
      ),
      (
        'telemetry_send_failure',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_send_failure,
        (1, 120000, 20)
      ),
      (
        'telemetry_send_success',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_send_success,
        (1, 120000, 20)
      ),
      (
        'telemetry_sessiondata_failed_load',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.telemetry_sessiondata_failed_load,
        (1, 2, 3)
      ),
      (
        'telemetry_sessiondata_failed_parse',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.telemetry_sessiondata_failed_parse,
        (1, 2, 3)
      ),
      (
        'telemetry_sessiondata_failed_save',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.telemetry_sessiondata_failed_save,
        (1, 2, 3)
      ),
      (
        'telemetry_sessiondata_failed_validation',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.telemetry_sessiondata_failed_validation,
        (1, 2, 3)
      ),
      (
        'telemetry_stringify',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.telemetry_stringify,
        (1, 3000, 10)
      ),
      (
        'telemetry_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.telemetry_success,
        (1, 2, 3)
      ),
      (
        'thunderbird_conversations_time_to_2nd_gloda_query_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.thunderbird_conversations_time_to_2nd_gloda_query_ms,
        (1, 10000, 30)
      ),
      (
        'thunderbird_gloda_size_mb',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.thunderbird_gloda_size_mb,
        (1, 1000, 40)
      ),
      (
        'thunderbird_indexing_rate_msg_per_s',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.thunderbird_indexing_rate_msg_per_s,
        (1, 100, 20)
      ),
      (
        'time_between_unlabeled_runnables_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_between_unlabeled_runnables_ms,
        (1, 30000, 30)
      ),
      (
        'time_to_dom_complete_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_complete_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_dom_content_loaded_end_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_content_loaded_end_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_dom_content_loaded_start_active_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_content_loaded_start_active_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_dom_content_loaded_start_active_netopt_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_content_loaded_start_active_netopt_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_dom_content_loaded_start_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_content_loaded_start_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_dom_interactive_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_interactive_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_dom_loading_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_dom_loading_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_first_click_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_first_click_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_first_contentful_paint_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_first_contentful_paint_ms,
        (1, 100000, 100)
      ),
      (
        'time_to_first_interaction_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_first_interaction_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_first_key_input_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_first_key_input_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_first_mouse_move_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_first_mouse_move_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_first_scroll_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_first_scroll_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_load_event_end_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_load_event_end_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_load_event_start_active_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_load_event_start_active_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_load_event_start_active_netopt_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_load_event_start_active_netopt_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_load_event_start_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_load_event_start_ms,
        (1, 50000, 100)
      ),
      (
        'time_to_non_blank_paint_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_non_blank_paint_ms,
        (1, 100000, 100)
      ),
      (
        'time_to_non_blank_paint_netopt_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_non_blank_paint_netopt_ms,
        (1, 100000, 100)
      ),
      (
        'time_to_non_blank_paint_no_netopt_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_non_blank_paint_no_netopt_ms,
        (1, 100000, 100)
      ),
      (
        'time_to_response_start_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.time_to_response_start_ms,
        (1, 50000, 100)
      ),
      (
        'timeout_execution_bg_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.timeout_execution_bg_ms,
        (1, 1000, 20)
      ),
      (
        'timeout_execution_bg_tracking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.timeout_execution_bg_tracking_ms,
        (1, 1000, 10)
      ),
      (
        'timeout_execution_fg_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.timeout_execution_fg_ms,
        (1, 1000, 20)
      ),
      (
        'timeout_execution_fg_tracking_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.timeout_execution_fg_tracking_ms,
        (1, 1000, 20)
      ),
      (
        'tls_early_data_accepted',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.tls_early_data_accepted,
        (1, 2, 3)
      ),
      (
        'tls_early_data_bytes_written',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.tls_early_data_bytes_written,
        (1, 60000, 100)
      ),
      (
        'tls_early_data_negotiated',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tls_early_data_negotiated,
        (1, 3, 4)
      ),
      (
        'tls_error_report_ui',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tls_error_report_ui,
        (1, 15, 16)
      ),
      (
        'top_level_content_documents_destroyed',
        'histogram-count',
        'content',
        payload.processes.content.histograms.top_level_content_documents_destroyed,
        (1, 2, 3)
      ),
      (
        'total_containers_opened',
        'histogram-count',
        'content',
        payload.processes.content.histograms.total_containers_opened,
        (1, 2, 3)
      ),
      (
        'total_content_page_load_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.total_content_page_load_time,
        (100, 30000, 100)
      ),
      (
        'total_count_high_errors',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.total_count_high_errors,
        (1, 3000, 10)
      ),
      (
        'total_count_low_errors',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.total_count_low_errors,
        (1, 3000, 10)
      ),
      (
        'total_docgroups_per_tabgroup',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.total_docgroups_per_tabgroup,
        (1, 50, 20)
      ),
      (
        'total_http_docgroups_per_tabgroup',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.total_http_docgroups_per_tabgroup,
        (1, 50, 20)
      ),
      (
        'total_scroll_y',
        'histogram-count',
        'content',
        payload.processes.content.histograms.total_scroll_y,
        (1, 2, 3)
      ),
      (
        'touch_enabled_device',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.touch_enabled_device,
        (1, 2, 3)
      ),
      (
        'tracking_protection_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.tracking_protection_enabled,
        (1, 2, 3)
      ),
      (
        'tracking_protection_events',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tracking_protection_events,
        (1, 3, 4)
      ),
      (
        'tracking_protection_pbm_disabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.tracking_protection_pbm_disabled,
        (1, 2, 3)
      ),
      (
        'tracking_protection_shield',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.tracking_protection_shield,
        (1, 4, 5)
      ),
      (
        'transaction_wait_time_http',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.transaction_wait_time_http,
        (1, 5000, 100)
      ),
      (
        'transaction_wait_time_http3',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.transaction_wait_time_http3,
        (1, 5000, 100)
      ),
      (
        'transaction_wait_time_spdy',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.transaction_wait_time_spdy,
        (1, 5000, 100)
      ),
      (
        'translated_characters',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.translated_characters,
        (1, 10240, 50)
      ),
      (
        'translated_pages',
        'histogram-count',
        'content',
        payload.processes.content.histograms.translated_pages,
        (1, 2, 3)
      ),
      (
        'translation_opportunities',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.translation_opportunities,
        (1, 2, 3)
      ),
      (
        'types_of_user_clicks',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.types_of_user_clicks,
        (1, 50, 51)
      ),
      (
        'unique_containers_opened',
        'histogram-count',
        'content',
        payload.processes.content.histograms.unique_containers_opened,
        (1, 2, 3)
      ),
      (
        'update_cannot_stage_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_cannot_stage_external,
        (1, 2, 3)
      ),
      (
        'update_cannot_stage_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_cannot_stage_notify,
        (1, 2, 3)
      ),
      (
        'update_check_code_external',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_check_code_external,
        (1, 50, 51)
      ),
      (
        'update_check_code_notify',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_check_code_notify,
        (1, 50, 51)
      ),
      (
        'update_check_no_update_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_check_no_update_external,
        (1, 2, 3)
      ),
      (
        'update_check_no_update_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_check_no_update_notify,
        (1, 2, 3)
      ),
      (
        'update_download_code_complete',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_download_code_complete,
        (1, 50, 51)
      ),
      (
        'update_download_code_partial',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_download_code_partial,
        (1, 50, 51)
      ),
      (
        'update_invalid_lastupdatetime_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_invalid_lastupdatetime_external,
        (1, 2, 3)
      ),
      (
        'update_invalid_lastupdatetime_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_invalid_lastupdatetime_notify,
        (1, 2, 3)
      ),
      (
        'update_last_notify_interval_days_external',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.update_last_notify_interval_days_external,
        (1, 365, 60)
      ),
      (
        'update_last_notify_interval_days_notify',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.update_last_notify_interval_days_notify,
        (1, 180, 30)
      ),
      (
        'update_not_pref_update_auto_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_auto_external,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_auto_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_auto_notify,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_enabled_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_enabled_external,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_enabled_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_enabled_notify,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_service_enabled_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_service_enabled_external,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_service_enabled_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_service_enabled_notify,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_staging_enabled_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_staging_enabled_external,
        (1, 2, 3)
      ),
      (
        'update_not_pref_update_staging_enabled_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_not_pref_update_staging_enabled_notify,
        (1, 2, 3)
      ),
      (
        'update_notification_dismissed',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.update_notification_dismissed,
        (1, 50, 51)
      ),
      (
        'update_notification_main_action_doorhanger',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.update_notification_main_action_doorhanger,
        (1, 50, 51)
      ),
      (
        'update_notification_main_action_menu',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.update_notification_main_action_menu,
        (1, 50, 51)
      ),
      (
        'update_notification_shown',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.update_notification_shown,
        (1, 50, 51)
      ),
      (
        'update_ping_count_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_ping_count_external,
        (1, 2, 3)
      ),
      (
        'update_ping_count_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_ping_count_notify,
        (1, 2, 3)
      ),
      (
        'update_pref_service_errors_external',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_pref_service_errors_external,
        (1, 30, 31)
      ),
      (
        'update_pref_service_errors_notify',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_pref_service_errors_notify,
        (1, 30, 31)
      ),
      (
        'update_pref_update_cancelations_external',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_pref_update_cancelations_external,
        (1, 100, 101)
      ),
      (
        'update_pref_update_cancelations_notify',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_pref_update_cancelations_notify,
        (1, 100, 101)
      ),
      (
        'update_service_installed_external',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.update_service_installed_external,
        (1, 2, 3)
      ),
      (
        'update_service_installed_notify',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.update_service_installed_notify,
        (1, 2, 3)
      ),
      (
        'update_service_manually_uninstalled_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_service_manually_uninstalled_external,
        (1, 2, 3)
      ),
      (
        'update_service_manually_uninstalled_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_service_manually_uninstalled_notify,
        (1, 2, 3)
      ),
      (
        'update_state_code_complete_stage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_state_code_complete_stage,
        (1, 20, 21)
      ),
      (
        'update_state_code_complete_startup',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_state_code_complete_startup,
        (1, 20, 21)
      ),
      (
        'update_state_code_partial_stage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_state_code_partial_stage,
        (1, 20, 21)
      ),
      (
        'update_state_code_partial_startup',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_state_code_partial_startup,
        (1, 20, 21)
      ),
      (
        'update_state_code_unknown_stage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_state_code_unknown_stage,
        (1, 20, 21)
      ),
      (
        'update_state_code_unknown_startup',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_state_code_unknown_startup,
        (1, 20, 21)
      ),
      (
        'update_status_error_code_complete_stage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_status_error_code_complete_stage,
        (1, 100, 101)
      ),
      (
        'update_status_error_code_complete_startup',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_status_error_code_complete_startup,
        (1, 100, 101)
      ),
      (
        'update_status_error_code_partial_stage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_status_error_code_partial_stage,
        (1, 100, 101)
      ),
      (
        'update_status_error_code_partial_startup',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_status_error_code_partial_startup,
        (1, 100, 101)
      ),
      (
        'update_status_error_code_unknown_stage',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_status_error_code_unknown_stage,
        (1, 100, 101)
      ),
      (
        'update_status_error_code_unknown_startup',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_status_error_code_unknown_startup,
        (1, 100, 101)
      ),
      (
        'update_unable_to_apply_external',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_unable_to_apply_external,
        (1, 2, 3)
      ),
      (
        'update_unable_to_apply_notify',
        'histogram-count',
        'content',
        payload.processes.content.histograms.update_unable_to_apply_notify,
        (1, 2, 3)
      ),
      (
        'update_wiz_last_page_code',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.update_wiz_last_page_code,
        (1, 30, 31)
      ),
      (
        'urlclassifier_async_classifylocal_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_async_classifylocal_time,
        (1, 60000, 30)
      ),
      (
        'urlclassifier_cl_check_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_cl_check_time,
        (1, 500, 10)
      ),
      (
        'urlclassifier_classifylocal_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_classifylocal_time,
        (1, 60000, 30)
      ),
      (
        'urlclassifier_completion_error',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_completion_error,
        (1, 16, 17)
      ),
      (
        'urlclassifier_lc_completions',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_lc_completions,
        (1, 200, 10)
      ),
      (
        'urlclassifier_lc_prefixes',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.urlclassifier_lc_prefixes,
        (1, 1500000, 15)
      ),
      (
        'urlclassifier_lookup_time_2',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_lookup_time_2,
        (1, 5000, 30)
      ),
      (
        'urlclassifier_match_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_match_result,
        (1, 16, 17)
      ),
      (
        'urlclassifier_match_threat_type_result',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_match_threat_type_result,
        (1, 64, 65)
      ),
      (
        'urlclassifier_negative_cache_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_negative_cache_duration,
        (1, 86400000, 50)
      ),
      (
        'urlclassifier_positive_cache_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_positive_cache_duration,
        (1, 86400000, 50)
      ),
      (
        'urlclassifier_ps_construct_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_ps_construct_time,
        (1, 5000, 15)
      ),
      (
        'urlclassifier_ps_fallocate_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_ps_fallocate_time,
        (1, 1000, 10)
      ),
      (
        'urlclassifier_ps_fileload_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_ps_fileload_time,
        (1, 1000, 10)
      ),
      (
        'urlclassifier_shutdown_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_shutdown_time,
        (1, 60000, 50)
      ),
      (
        'urlclassifier_threathit_network_error',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_threathit_network_error,
        (1, 30, 31)
      ),
      (
        'urlclassifier_threathit_remote_status',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_threathit_remote_status,
        (1, 16, 17)
      ),
      (
        'urlclassifier_ui_events',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_ui_events,
        (1, 64, 65)
      ),
      (
        'urlclassifier_vlps_construct_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_vlps_construct_time,
        (1, 5000, 15)
      ),
      (
        'urlclassifier_vlps_fallocate_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_vlps_fallocate_time,
        (1, 1000, 10)
      ),
      (
        'urlclassifier_vlps_fileload_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.urlclassifier_vlps_fileload_time,
        (1, 1000, 10)
      ),
      (
        'urlclassifier_vlps_load_corrupt',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.urlclassifier_vlps_load_corrupt,
        (1, 2, 3)
      ),
      (
        'urlclassifier_vlps_long_prefixes',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.urlclassifier_vlps_long_prefixes,
        (1, 32, 33)
      ),
      (
        'urlclassifier_vlps_metadata_corrupt',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.urlclassifier_vlps_metadata_corrupt,
        (1, 2, 3)
      ),
      (
        'user_font_families_per_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.user_font_families_per_page,
        (1, 256, 64)
      ),
      (
        'vfc_clearcurrentframe_lock_hold_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_clearcurrentframe_lock_hold_ms,
        (1000, 100000, 100)
      ),
      (
        'vfc_clearfutureframes_lock_hold_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_clearfutureframes_lock_hold_ms,
        (1000, 100000, 100)
      ),
      (
        'vfc_invalidate_lock_hold_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_invalidate_lock_hold_ms,
        (1000, 100000, 100)
      ),
      (
        'vfc_invalidate_lock_wait_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_invalidate_lock_wait_ms,
        (1000, 100000, 100)
      ),
      (
        'vfc_setcurrentframe_lock_hold_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_setcurrentframe_lock_hold_ms,
        (1000, 100000, 100)
      ),
      (
        'vfc_setimages_lock_hold_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_setimages_lock_hold_ms,
        (1000, 100000, 100)
      ),
      (
        'vfc_setvideosegment_lock_hold_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.vfc_setvideosegment_lock_hold_ms,
        (1000, 100000, 100)
      ),
      (
        'video_as_content_source',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_as_content_source,
        (1, 12, 13)
      ),
      (
        'video_as_content_source_in_tree_or_not',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_as_content_source_in_tree_or_not,
        (1, 12, 13)
      ),
      (
        'video_can_create_aac_decoder',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.video_can_create_aac_decoder,
        (1, 2, 3)
      ),
      (
        'video_can_create_h264_decoder',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.video_can_create_h264_decoder,
        (1, 2, 3)
      ),
      (
        'video_canplaytype_h264_constraint_set_flag',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_canplaytype_h264_constraint_set_flag,
        (1, 128, 129)
      ),
      (
        'video_canplaytype_h264_level',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_canplaytype_h264_level,
        (1, 51, 52)
      ),
      (
        'video_canplaytype_h264_profile',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_canplaytype_h264_profile,
        (1, 244, 245)
      ),
      (
        'video_cdm_created',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_cdm_created,
        (1, 6, 7)
      ),
      (
        'video_chromium_cdm_max_shmems',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_chromium_cdm_max_shmems,
        (1, 50, 51)
      ),
      (
        'video_decoded_h264_sps_constraint_set_flag',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_decoded_h264_sps_constraint_set_flag,
        (1, 128, 129)
      ),
      (
        'video_decoded_h264_sps_level',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_decoded_h264_sps_level,
        (1, 51, 52)
      ),
      (
        'video_decoded_h264_sps_profile',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_decoded_h264_sps_profile,
        (1, 244, 245)
      ),
      (
        'video_dropped_frames_proportion',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.video_dropped_frames_proportion,
        (1, 100, 50)
      ),
      (
        'video_dropped_frames_proportion_exponential',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_dropped_frames_proportion_exponential,
        (1, 10000, 100)
      ),
      (
        'video_eme_play_success',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.video_eme_play_success,
        (1, 2, 3)
      ),
      (
        'video_eme_request_failure_latency_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_eme_request_failure_latency_ms,
        (1, 60000, 60)
      ),
      (
        'video_eme_request_success_latency_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_eme_request_success_latency_ms,
        (1, 60000, 60)
      ),
      (
        'video_fastseek_used',
        'histogram-count',
        'content',
        payload.processes.content.histograms.video_fastseek_used,
        (1, 2, 3)
      ),
      (
        'video_h264_sps_max_num_ref_frames',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_h264_sps_max_num_ref_frames,
        (1, 17, 18)
      ),
      (
        'video_hidden_play_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_hidden_play_time_ms,
        (1, 7200000, 100)
      ),
      (
        'video_hw_decoder_crash_recovery_time_since_gpu_crashed_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_hw_decoder_crash_recovery_time_since_gpu_crashed_ms,
        (1, 64000, 100)
      ),
      (
        'video_hw_decoder_crash_recovery_time_since_mfr_notified_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_hw_decoder_crash_recovery_time_since_mfr_notified_ms,
        (1, 64000, 100)
      ),
      (
        'video_mft_output_null_samples',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_mft_output_null_samples,
        (1, 10, 11)
      ),
      (
        'video_play_time_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.video_play_time_ms,
        (1, 7200000, 100)
      ),
      (
        'video_unload_state',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.video_unload_state,
        (1, 5, 6)
      ),
      (
        'video_vp9_benchmark_fps',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.video_vp9_benchmark_fps,
        (1, 1000, 100)
      ),
      (
        'view_source_external_result_boolean',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.view_source_external_result_boolean,
        (1, 2, 3)
      ),
      (
        'view_source_in_browser_opened_boolean',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.view_source_in_browser_opened_boolean,
        (1, 2, 3)
      ),
      (
        'view_source_in_window_opened_boolean',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.view_source_in_window_opened_boolean,
        (1, 2, 3)
      ),
      (
        'weave_complete_success_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.weave_complete_success_count,
        (1, 1000, 10)
      ),
      (
        'weave_configured',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.weave_configured,
        (1, 2, 3)
      ),
      (
        'weave_configured_master_password',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.weave_configured_master_password,
        (1, 2, 3)
      ),
      (
        'weave_device_count_desktop',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.weave_device_count_desktop,
        (1, 10, 11)
      ),
      (
        'weave_device_count_mobile',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.weave_device_count_mobile,
        (1, 10, 11)
      ),
      (
        'weave_start_count',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.weave_start_count,
        (1, 1000, 10)
      ),
      (
        'weave_wipe_server_succeeded',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.weave_wipe_server_succeeded,
        (1, 2, 3)
      ),
      (
        'web_audio_autoplay',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.web_audio_autoplay,
        (1, 50, 51)
      ),
      (
        'web_audio_becomes_audible_time',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.web_audio_becomes_audible_time,
        (1, 60, 10)
      ),
      (
        'web_font_families_per_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.web_font_families_per_page,
        (1, 256, 64)
      ),
      (
        'web_notification_clicked',
        'histogram-count',
        'content',
        payload.processes.content.histograms.web_notification_clicked,
        (1, 2, 3)
      ),
      (
        'web_notification_exceptions_opened',
        'histogram-count',
        'content',
        payload.processes.content.histograms.web_notification_exceptions_opened,
        (1, 2, 3)
      ),
      (
        'web_notification_menu',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.web_notification_menu,
        (1, 5, 6)
      ),
      (
        'web_notification_permissions',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.web_notification_permissions,
        (1, 10, 11)
      ),
      (
        'web_notification_shown',
        'histogram-count',
        'content',
        payload.processes.content.histograms.web_notification_shown,
        (1, 2, 3)
      ),
      (
        'webcrypto_alg',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webcrypto_alg,
        (1, 30, 31)
      ),
      (
        'webcrypto_extractable_enc',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webcrypto_extractable_enc,
        (1, 2, 3)
      ),
      (
        'webcrypto_extractable_generate',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webcrypto_extractable_generate,
        (1, 2, 3)
      ),
      (
        'webcrypto_extractable_import',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webcrypto_extractable_import,
        (1, 2, 3)
      ),
      (
        'webcrypto_extractable_sig',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webcrypto_extractable_sig,
        (1, 2, 3)
      ),
      (
        'webcrypto_method',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webcrypto_method,
        (1, 20, 21)
      ),
      (
        'webcrypto_method_secure',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webcrypto_method_secure,
        (1, 2, 3)
      ),
      (
        'webcrypto_resolved',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webcrypto_resolved,
        (1, 2, 3)
      ),
      (
        'webext_content_script_injection_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webext_content_script_injection_ms,
        (1, 50000, 100)
      ),
      (
        'webext_storage_local_get_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webext_storage_local_get_ms,
        (1, 50000, 100)
      ),
      (
        'webext_storage_local_idb_get_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webext_storage_local_idb_get_ms,
        (1, 50000, 100)
      ),
      (
        'webext_storage_local_idb_set_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webext_storage_local_idb_set_ms,
        (1, 50000, 100)
      ),
      (
        'webext_storage_local_set_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webext_storage_local_set_ms,
        (1, 50000, 100)
      ),
      (
        'webext_user_script_injection_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webext_user_script_injection_ms,
        (1, 50000, 100)
      ),
      (
        'webfont_compression_woff',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webfont_compression_woff,
        (1, 50, 51)
      ),
      (
        'webfont_compression_woff2',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webfont_compression_woff2,
        (1, 50, 51)
      ),
      (
        'webfont_download_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webfont_download_time,
        (1, 60000, 50)
      ),
      (
        'webfont_download_time_after_start',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webfont_download_time_after_start,
        (1, 60000, 50)
      ),
      (
        'webfont_fonttype',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webfont_fonttype,
        (1, 10, 11)
      ),
      (
        'webfont_per_page',
        'histogram-count',
        'content',
        payload.processes.content.histograms.webfont_per_page,
        (1, 2, 3)
      ),
      (
        'webfont_size',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webfont_size,
        (1, 5000, 50)
      ),
      (
        'webfont_size_per_page',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webfont_size_per_page,
        (1, 5000, 50)
      ),
      (
        'webfont_srctype',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webfont_srctype,
        (1, 5, 6)
      ),
      (
        'webkit_directory_used',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webkit_directory_used,
        (1, 2, 3)
      ),
      (
        'webrtc_audio_quality_inbound_bandwidth_kbits',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_inbound_bandwidth_kbits,
        (1, 1000000, 1000)
      ),
      (
        'webrtc_audio_quality_inbound_jitter',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_inbound_jitter,
        (1, 10000, 1000)
      ),
      (
        'webrtc_audio_quality_inbound_packetloss_rate',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_inbound_packetloss_rate,
        (1, 1000, 100)
      ),
      (
        'webrtc_audio_quality_outbound_bandwidth_kbits',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_outbound_bandwidth_kbits,
        (1, 1000000, 1000)
      ),
      (
        'webrtc_audio_quality_outbound_jitter',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_outbound_jitter,
        (1, 10000, 1000)
      ),
      (
        'webrtc_audio_quality_outbound_packetloss_rate',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_outbound_packetloss_rate,
        (1, 1000, 100)
      ),
      (
        'webrtc_audio_quality_outbound_rtt',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_audio_quality_outbound_rtt,
        (1, 10000, 1000)
      ),
      (
        'webrtc_avsync_when_audio_lags_video_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_avsync_when_audio_lags_video_ms,
        (1, 60000, 1000)
      ),
      (
        'webrtc_avsync_when_video_lags_audio_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_avsync_when_video_lags_audio_ms,
        (1, 60000, 1000)
      ),
      (
        'webrtc_call_count_2',
        'histogram-count',
        'content',
        payload.processes.content.histograms.webrtc_call_count_2,
        (1, 2, 3)
      ),
      (
        'webrtc_call_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_call_duration,
        (1, 10000, 1000)
      ),
      (
        'webrtc_call_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_call_type,
        (1, 8, 9)
      ),
      (
        'webrtc_datachannel_negotiated',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webrtc_datachannel_negotiated,
        (1, 2, 3)
      ),
      (
        'webrtc_dtls_cipher',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_dtls_cipher,
        (1, 14, 15)
      ),
      (
        'webrtc_dtls_client_abort_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_dtls_client_abort_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_dtls_client_failure_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_dtls_client_failure_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_dtls_client_success_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_dtls_client_success_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_dtls_protocol_version',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.webrtc_dtls_protocol_version,
        (1, 50, 51)
      ),
      (
        'webrtc_dtls_server_abort_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_dtls_server_abort_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_dtls_server_failure_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_dtls_server_failure_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_dtls_server_success_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_dtls_server_success_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_get_user_media_secure_origin',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_get_user_media_secure_origin,
        (1, 15, 16)
      ),
      (
        'webrtc_get_user_media_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_get_user_media_type,
        (1, 8, 9)
      ),
      (
        'webrtc_hostname_obfuscation_disabled_ice_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_hostname_obfuscation_disabled_ice_duration,
        (1, 600, 100)
      ),
      (
        'webrtc_hostname_obfuscation_disabled_ice_duration_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_hostname_obfuscation_disabled_ice_duration_ms,
        (1, 30000, 100)
      ),
      (
        'webrtc_hostname_obfuscation_enabled_ice_duration',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_hostname_obfuscation_enabled_ice_duration,
        (1, 600, 100)
      ),
      (
        'webrtc_hostname_obfuscation_enabled_ice_duration_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_hostname_obfuscation_enabled_ice_duration_ms,
        (1, 30000, 100)
      ),
      (
        'webrtc_ice_add_candidate_errors_given_failure',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_ice_add_candidate_errors_given_failure,
        (1, 30, 29)
      ),
      (
        'webrtc_ice_add_candidate_errors_given_success',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_ice_add_candidate_errors_given_success,
        (1, 30, 29)
      ),
      (
        'webrtc_ice_answerer_abort_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_answerer_abort_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_answerer_failure_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_answerer_failure_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_answerer_success_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_answerer_success_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_checking_rate',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webrtc_ice_checking_rate,
        (1, 2, 3)
      ),
      (
        'webrtc_ice_failure_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_failure_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_final_connection_state',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_ice_final_connection_state,
        (1, 7, 8)
      ),
      (
        'webrtc_ice_late_trickle_arrival_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_late_trickle_arrival_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_nr_ice_gather_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_nr_ice_gather_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_nr_ice_gather_time_immediate_failure',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_nr_ice_gather_time_immediate_failure,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_nr_ice_gather_time_immediate_success',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_nr_ice_gather_time_immediate_success,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_offerer_abort_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_offerer_abort_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_offerer_failure_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_offerer_failure_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_offerer_success_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_offerer_success_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_on_time_trickle_arrival_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_on_time_trickle_arrival_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_ice_success_rate',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webrtc_ice_success_rate,
        (1, 2, 3)
      ),
      (
        'webrtc_ice_success_time',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_ice_success_time,
        (1, 10000, 20)
      ),
      (
        'webrtc_load_state_normal',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_load_state_normal,
        (1, 100, 25)
      ),
      (
        'webrtc_load_state_normal_short',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_load_state_normal_short,
        (1, 100, 25)
      ),
      (
        'webrtc_load_state_relaxed',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_load_state_relaxed,
        (1, 100, 25)
      ),
      (
        'webrtc_load_state_relaxed_short',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_load_state_relaxed_short,
        (1, 100, 25)
      ),
      (
        'webrtc_load_state_stressed',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_load_state_stressed,
        (1, 100, 25)
      ),
      (
        'webrtc_load_state_stressed_short',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_load_state_stressed_short,
        (1, 100, 25)
      ),
      (
        'webrtc_max_audio_receive_track',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_max_audio_receive_track,
        (1, 30, 29)
      ),
      (
        'webrtc_max_audio_send_track',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_max_audio_send_track,
        (1, 20, 19)
      ),
      (
        'webrtc_max_video_receive_track',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_max_video_receive_track,
        (1, 20, 19)
      ),
      (
        'webrtc_max_video_send_track',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_max_video_send_track,
        (1, 10, 9)
      ),
      (
        'webrtc_renegotiations',
        'histogram-linear',
        'content',
        payload.processes.content.histograms.webrtc_renegotiations,
        (1, 21, 20)
      ),
      (
        'webrtc_rtcp_mux',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.webrtc_rtcp_mux,
        (1, 2, 3)
      ),
      (
        'webrtc_srtp_cipher',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.webrtc_srtp_cipher,
        (1, 50, 51)
      ),
      (
        'webrtc_stun_rate_limit_exceeded_by_type_given_failure',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_stun_rate_limit_exceeded_by_type_given_failure,
        (1, 4, 5)
      ),
      (
        'webrtc_stun_rate_limit_exceeded_by_type_given_success',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webrtc_stun_rate_limit_exceeded_by_type_given_success,
        (1, 4, 5)
      ),
      (
        'webrtc_video_decode_error_time_permille',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_decode_error_time_permille,
        (1, 1000, 100)
      ),
      (
        'webrtc_video_decoder_bitrate_avg_per_call_kbps',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_decoder_bitrate_avg_per_call_kbps,
        (1, 10000, 100)
      ),
      (
        'webrtc_video_decoder_bitrate_std_dev_per_call_kbps',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_decoder_bitrate_std_dev_per_call_kbps,
        (1, 5000, 100)
      ),
      (
        'webrtc_video_decoder_discarded_packets_per_call_ppm',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_decoder_discarded_packets_per_call_ppm,
        (1, 50000, 100)
      ),
      (
        'webrtc_video_decoder_framerate_10x_std_dev_per_call',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_decoder_framerate_10x_std_dev_per_call,
        (1, 200, 50)
      ),
      (
        'webrtc_video_decoder_framerate_avg_per_call',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_decoder_framerate_avg_per_call,
        (1, 200, 50)
      ),
      (
        'webrtc_video_encoder_bitrate_avg_per_call_kbps',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_encoder_bitrate_avg_per_call_kbps,
        (1, 10000, 100)
      ),
      (
        'webrtc_video_encoder_bitrate_std_dev_per_call_kbps',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_encoder_bitrate_std_dev_per_call_kbps,
        (1, 5000, 100)
      ),
      (
        'webrtc_video_encoder_dropped_frames_per_call_fpm',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_encoder_dropped_frames_per_call_fpm,
        (1, 50000, 100)
      ),
      (
        'webrtc_video_encoder_framerate_10x_std_dev_per_call',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_encoder_framerate_10x_std_dev_per_call,
        (1, 200, 50)
      ),
      (
        'webrtc_video_encoder_framerate_avg_per_call',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_encoder_framerate_avg_per_call,
        (1, 200, 50)
      ),
      (
        'webrtc_video_error_recovery_ms',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_error_recovery_ms,
        (1, 10000, 500)
      ),
      (
        'webrtc_video_quality_inbound_bandwidth_kbits',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_inbound_bandwidth_kbits,
        (1, 1000000, 1000)
      ),
      (
        'webrtc_video_quality_inbound_jitter',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_inbound_jitter,
        (1, 10000, 100)
      ),
      (
        'webrtc_video_quality_inbound_packetloss_rate',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_inbound_packetloss_rate,
        (1, 1000, 100)
      ),
      (
        'webrtc_video_quality_outbound_bandwidth_kbits',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_outbound_bandwidth_kbits,
        (1, 1000000, 1000)
      ),
      (
        'webrtc_video_quality_outbound_jitter',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_outbound_jitter,
        (1, 10000, 1000)
      ),
      (
        'webrtc_video_quality_outbound_packetloss_rate',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_outbound_packetloss_rate,
        (1, 1000, 100)
      ),
      (
        'webrtc_video_quality_outbound_rtt',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_quality_outbound_rtt,
        (1, 10000, 1000)
      ),
      (
        'webrtc_video_recovery_after_error_per_min',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_recovery_after_error_per_min,
        (1, 1000, 200)
      ),
      (
        'webrtc_video_recovery_before_error_per_min',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webrtc_video_recovery_before_error_per_min,
        (1, 1000, 200)
      ),
      (
        'websockets_handshake_type',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.websockets_handshake_type,
        (1, 16, 17)
      ),
      (
        'webvr_time_spend_for_viewing_in_2d',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webvr_time_spend_for_viewing_in_2d,
        (1, 50000, 100)
      ),
      (
        'webvr_time_spent_viewing_in_2d',
        'histogram-exponential',
        'content',
        payload.processes.content.histograms.webvr_time_spent_viewing_in_2d,
        (1, 1200000, 100)
      ),
      (
        'webvr_users_view_in',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webvr_users_view_in,
        (1, 3, 4)
      ),
      (
        'webvtt_track_kinds',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.webvtt_track_kinds,
        (1, 10, 11)
      ),
      (
        'webvtt_used_vtt_cues',
        'histogram-count',
        'content',
        payload.processes.content.histograms.webvtt_used_vtt_cues,
        (1, 2, 3)
      ),
      (
        'webxr_api_mode',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.webxr_api_mode,
        (1, 50, 51)
      ),
      (
        'window_open_outer_size',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.window_open_outer_size,
        (1, 2, 3)
      ),
      (
        'xcto_nosniff_block_image',
        'histogram-enumerated',
        'content',
        payload.processes.content.histograms.xcto_nosniff_block_image,
        (1, 3, 4)
      ),
      (
        'xcto_nosniff_toplevel_nav_exceptions',
        'histogram-categorical',
        'content',
        payload.processes.content.histograms.xcto_nosniff_toplevel_nav_exceptions,
        (1, 50, 51)
      ),
      (
        'xmlhttprequest_async_or_sync',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.xmlhttprequest_async_or_sync,
        (1, 2, 3)
      ),
      (
        'xul_cache_disabled',
        'histogram-flag',
        'content',
        payload.processes.content.histograms.xul_cache_disabled,
        (1, 2, 3)
      ),
      (
        'zoomed_view_enabled',
        'histogram-boolean',
        'content',
        payload.processes.content.histograms.zoomed_view_enabled,
        (1, 2, 3)
      )
    ] AS histogram_aggregates
  FROM
    sampled_data
),
filtered_aggregates AS (
  SELECT
    submission_date,
    sample_id,
    client_id,
    os,
    app_version,
    app_build_id,
    channel,
    metric,
    metric_type,
    process,
    bucket_range,
    value
  FROM
    histograms
  CROSS JOIN
    UNNEST(histogram_aggregates)
  WHERE
    value IS NOT NULL
),
aggregated AS (
  SELECT
    sample_id,
    client_id,
    submission_date,
    os,
    app_version,
    app_build_id,
    channel,
    metric,
    metric_type,
    process,
    ARRAY_AGG(bucket_range) AS bucket_range,
    ARRAY_AGG(value) AS value
  FROM
    filtered_aggregates
  GROUP BY
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10
)
SELECT
  sample_id,
  client_id,
  submission_date,
  os,
  app_version,
  app_build_id,
  channel,
  ARRAY_AGG(
    STRUCT<
      metric STRING,
      metric_type STRING,
      key STRING,
      process STRING,
      agg_type STRING,
      bucket_range STRUCT<first_bucket INT64, last_bucket INT64, num_buckets INT64>,
      value ARRAY<STRUCT<key STRING, value INT64>>
    >(
      metric,
      metric_type,
      '',
      process,
      'summed_histogram',
      bucket_range[OFFSET(0)],
      udf_aggregate_json_sum(value)
    )
  ) AS histogram_aggregates
FROM
  aggregated
GROUP BY
  1,
  2,
  3,
  4,
  5,
  6,
  7
